@@ -14,13 +14,14 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from oslo_middleware import request_id
 import webob.dec
 import webob.exc
 
 from nova.api.openstack import wsgi
+from nova.api import wsgi as base_wsgi
 import nova.conf
 from nova import context
-from nova import wsgi as base_wsgi
 
 CONF = nova.conf.CONF
 
@@ -50,13 +51,13 @@ class NoAuthMiddlewareBase(base_wsgi.Middleware):
         user_id, _sep, project_id = token.partition(':')
         project_id = project_id or user_id
         remote_address = getattr(req, 'remote_address', '127.0.0.1')
-        if CONF.use_forwarded_for:
+        if CONF.api.use_forwarded_for:
             remote_address = req.headers.get('X-Forwarded-For', remote_address)
         is_admin = always_admin or (user_id == 'admin')
-        ctx = context.RequestContext(user_id,
-                                     project_id,
-                                     is_admin=is_admin,
-                                     remote_address=remote_address)
+        ctx = context.RequestContext(
+            user_id, project_id, is_admin=is_admin,
+            remote_address=remote_address,
+            request_id=req.environ.get(request_id.ENV_REQUEST_ID))
 
         req.environ['nova.context'] = ctx
         return self.application
