@@ -14,7 +14,6 @@
 #    under the License.
 
 import collections
-import functools
 import itertools
 import re
 
@@ -25,15 +24,16 @@ import six.moves.urllib.parse as urlparse
 import webob
 from webob import exc
 
+from nova.api.openstack import api_version_request
 from nova.compute import task_states
-from nova.compute import utils as compute_utils
 from nova.compute import vm_states
 import nova.conf
+from nova import context as nova_context
 from nova import exception
 from nova.i18n import _
-from nova.i18n import _LE
-from nova.i18n import _LW
+from nova.network import constants
 from nova import objects
+from nova.objects import service
 from nova import quota
 from nova import utils
 
@@ -43,6 +43,9 @@ LOG = logging.getLogger(__name__)
 QUOTAS = quota.QUOTAS
 
 
+POWER_ON = 'POWER_ON'
+POWER_OFF = 'POWER_OFF'
+
 _STATE_MAP = {
     vm_states.ACTIVE: {
         'default': 'ACTIVE',
@@ -118,9 +121,9 @@ def status_from_state(vm_state, task_state='default'):
     task_map = _STATE_MAP.get(vm_state, dict(default='UNKNOWN'))
     status = task_map.get(task_state, task_map['default'])
     if status == "UNKNOWN":
-        LOG.error(_LE("status is UNKNOWN from vm_state=%(vm_state)s "
-                      "task_state=%(task_state)s. Bad upgrade or db "
-                      "corrupted?"),
+        LOG.error("status is UNKNOWN from vm_state=%(vm_state)s "
+                  "task_state=%(task_state)s. Bad upgrade or db "
+                  "corrupted?",
                   {'vm_state': vm_state, 'task_state': task_state})
     return status
 
@@ -132,8 +135,8 @@ def task_and_vm_state_from_status(statuses):
     vm_states = set()
     task_states = set()
     lower_statuses = [status.lower() for status in statuses]
-    for state, task_map in six.iteritems(_STATE_MAP):
-        for task_state, mapped_state in six.iteritems(task_map):
+    for state, task_map in _STATE_MAP.items():
+        for task_state, mapped_state in task_map.items():
             status_string = mapped_state
             if status_string.lower() in lower_statuses:
                 vm_states.add(state)
@@ -227,7 +230,7 @@ def limited(items, request):
     """
     params = get_pagination_params(request)
     offset = params.get('offset', 0)
-    limit = CONF.osapi_max_limit
+    limit = CONF.api.max_limit
     limit = min(limit, params.get('limit') or limit)
 
     return items[offset:(offset + limit)]
@@ -236,7 +239,7 @@ def limited(items, request):
 def get_limit_and_marker(request):
     """Get limited parameter from request."""
     params = get_pagination_params(request)
-    limit = CONF.osapi_max_limit
+    limit = CONF.api.max_limit
     limit = min(limit, params.get('limit', limit))
     marker = params.get('marker', None)
 
@@ -290,19 +293,6 @@ def check_img_metadata_properties_quota(context, metadata):
         expl = _("Image metadata limit exceeded")
         raise webob.exc.HTTPForbidden(explanation=expl)
 
-    #  check the key length.
-    if isinstance(metadata, dict):
-        for key, value in six.iteritems(metadata):
-            if len(key) == 0:
-                expl = _("Image metadata key cannot be blank")
-                raise webob.exc.HTTPBadRequest(explanation=expl)
-            if len(key) > 255:
-                expl = _("Image metadata key too long")
-                raise webob.exc.HTTPBadRequest(explanation=expl)
-    else:
-        expl = _("Invalid image metadata")
-        raise webob.exc.HTTPBadRequest(explanation=expl)
-
 
 def get_networks_for_instance_from_nw_info(nw_info):
     networks = collections.OrderedDict()
@@ -338,7 +328,7 @@ def get_networks_for_instance(context, instance):
                                       'mac_address': 'aa:aa:aa:aa:aa:aa'}]},
          ...}
     """
-    nw_info = compute_utils.get_nw_info_for_instance(instance)
+    nw_info = instance.get_network_info()
     return get_networks_for_instance_from_nw_info(nw_info)
 
 
@@ -361,18 +351,6 @@ def raise_http_conflict_for_instance_invalid_state(exc, action, server_id):
     raise webob.exc.HTTPConflict(explanation=msg)
 
 
-def check_snapshots_enabled(f):
-    @functools.wraps(f)
-    def inner(*args, **kwargs):
-        if not CONF.allow_instance_snapshots:
-            LOG.warning(_LW('Rejecting snapshot request, snapshots currently'
-                            ' disabled'))
-            msg = _("Instance snapshots are not permitted at this time.")
-            raise webob.exc.HTTPBadRequest(explanation=msg)
-        return f(*args, **kwargs)
-    return inner
-
-
 def url_join(*parts):
     """Convenience method for joining parts of a URL
 
@@ -396,7 +374,7 @@ class ViewBuilder(object):
         otherwise
         """
         project_id = request.environ["nova.context"].project_id
-        if project_id in request.url:
+        if project_id and project_id in request.url:
             return project_id
         return ''
 
@@ -414,7 +392,7 @@ class ViewBuilder(object):
 
     def _get_next_link(self, request, identifier, collection_name):
         """Return href string with proper limit and marker params."""
-        params = request.params.copy()
+        params = collections.OrderedDict(sorted(request.params.items()))
         params["marker"] = identifier
         prefix = self._update_compute_link_prefix(request.application_url)
         url = url_join(prefix,
@@ -446,15 +424,15 @@ class ViewBuilder(object):
                               id_key="uuid"):
         """Retrieve 'next' link, if applicable. This is included if:
         1) 'limit' param is specified and equals the number of items.
-        2) 'limit' param is specified but it exceeds CONF.osapi_max_limit,
-        in this case the number of items is CONF.osapi_max_limit.
+        2) 'limit' param is specified but it exceeds CONF.api.max_limit,
+        in this case the number of items is CONF.api.max_limit.
         3) 'limit' param is NOT specified but the number of items is
-        CONF.osapi_max_limit.
+        CONF.api.max_limit.
         """
         links = []
         max_items = min(
-            int(request.params.get("limit", CONF.osapi_max_limit)),
-            CONF.osapi_max_limit)
+            int(request.params.get("limit", CONF.api.max_limit)),
+            CONF.api.max_limit)
         if max_items and max_items == len(items):
             last_item = items[-1]
             if id_key in last_item:
@@ -481,19 +459,19 @@ class ViewBuilder(object):
         return urlparse.urlunsplit(url_parts).rstrip('/')
 
     def _update_glance_link_prefix(self, orig_url):
-        return self._update_link_prefix(orig_url,
-                                        CONF.osapi_glance_link_prefix)
+        return self._update_link_prefix(orig_url, CONF.api.glance_link_prefix)
 
     def _update_compute_link_prefix(self, orig_url):
-        return self._update_link_prefix(orig_url,
-                                        CONF.osapi_compute_link_prefix)
+        return self._update_link_prefix(orig_url, CONF.api.compute_link_prefix)
 
 
-def get_instance(compute_api, context, instance_id, expected_attrs=None):
+def get_instance(compute_api, context, instance_id, expected_attrs=None,
+                 cell_down_support=False):
     """Fetch an instance from the compute API, handling error checking."""
     try:
         return compute_api.get(context, instance_id,
-                               expected_attrs=expected_attrs)
+                               expected_attrs=expected_attrs,
+                               cell_down_support=cell_down_support)
     except exception.InstanceNotFound as e:
         raise exc.HTTPNotFound(explanation=e.format_message())
 
@@ -522,15 +500,6 @@ def get_flavor(context, flavor_id):
         raise exc.HTTPNotFound(explanation=error.format_message())
 
 
-def check_cells_enabled(function):
-    @functools.wraps(function)
-    def inner(*args, **kwargs):
-        if not CONF.cells.enable:
-            raise_feature_not_supported()
-        return function(*args, **kwargs)
-    return inner
-
-
 def is_all_tenants(search_opts):
     """Checks to see if the all_tenants flag is in search_opts
 
@@ -547,3 +516,76 @@ def is_all_tenants(search_opts):
         # The empty string is considered enabling all_tenants
         all_tenants = 'all_tenants' in search_opts
     return all_tenants
+
+
+def is_locked(search_opts):
+    """Converts the value of the locked parameter to a boolean. Note that
+    this function will be called only if locked exists in search_opts.
+
+    :param dict search_opts: The search options for a request
+    :returns: boolean indicating if locked is being requested or not
+    """
+    locked = search_opts.get('locked')
+    try:
+        locked = strutils.bool_from_string(locked, strict=True)
+    except ValueError as err:
+        raise exception.InvalidInput(six.text_type(err))
+    return locked
+
+
+def supports_multiattach_volume(req):
+    """Check to see if the requested API version is high enough for multiattach
+
+    Microversion 2.60 adds support for booting from a multiattach volume.
+    The actual validation for a multiattach volume is done in the compute
+    API code, this is just checking the version so we can tell the API
+    code if the request version is high enough to even support it.
+
+    :param req: The incoming API request
+    :returns: True if the requested API microversion is high enough for
+        volume multiattach support, False otherwise.
+    """
+    return api_version_request.is_supported(req, '2.60')
+
+
+def supports_port_resource_request(req):
+    """Check to see if the requested API version is high enough for resource
+    request
+
+    :param req: The incoming API request
+    :returns: True if the requested API microversion is high enough for
+        port resource request support, False otherwise.
+    """
+    return api_version_request.is_supported(req, '2.72')
+
+
+def supports_port_resource_request_during_move():
+    """Check to see if the global compute service version is high enough to
+    support port resource request during move operation.
+
+    :returns: True if the compute service version is high enough for
+        port resource request move support, False otherwise.
+    """
+    return service.get_minimum_version_all_cells(
+        nova_context.get_admin_context(), ['nova-compute']) >= 49
+
+
+def instance_has_port_with_resource_request(instance_uuid, network_api):
+
+    # TODO(gibi): Use instance.info_cache to see if there is VIFs with
+    # allocation key in the profile. If there is no such VIF for an instance
+    # and the instance is not shelve offloaded then we can be sure that the
+    # instance has no port with resource request. If the instance is shelve
+    # offloaded then we still have to hit neutron.
+    search_opts = {'device_id': instance_uuid,
+                   'fields': [constants.RESOURCE_REQUEST]}
+    # NOTE(gibi): We need to use an admin context to query neutron ports as
+    # neutron does not fill the resource_request field in the port response if
+    # we query with a non admin context.
+    admin_context = nova_context.get_admin_context()
+    ports = network_api.list_ports(
+        admin_context, **search_opts).get('ports', [])
+    for port in ports:
+        if port.get(constants.RESOURCE_REQUEST):
+            return True
+    return False
