@@ -12,6 +12,8 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from nova.api.validation import parameter_types
+
 update = {
     'type': 'object',
     'properties': {
@@ -34,3 +36,15 @@ update = {
     },
     'additionalProperties': False
 }
+
+index_query = {
+    'type': 'object',
+    'properties': {
+        'zone': parameter_types.multi_params({'type': 'string'})
+    },
+    # NOTE(gmann): This is kept True to keep backward compatibility.
+    # As of now Schema validation stripped out the additional parameters and
+    # does not raise 400. This API is deprecated in microversion 2.43 so we
+    # do not to update the additionalProperties to False.
+    'additionalProperties': True
+}
