@@ -13,7 +13,6 @@
 import functools
 import re
 
-from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_serialization import jsonutils
 from oslo_utils import versionutils
@@ -23,12 +22,10 @@ import six
 from nova.db.sqlalchemy import api as db
 from nova.db.sqlalchemy import api_models
 from nova import exception
-from nova.i18n import _LE
 from nova import objects
 from nova.objects import base
 from nova.objects import fields
 
-CONF = cfg.CONF
 LOG = logging.getLogger(__name__)
 
 
@@ -37,7 +34,8 @@ class BuildRequest(base.NovaObject):
     # Version 1.0: Initial version
     # Version 1.1: Added block_device_mappings
     # Version 1.2: Added save() method
-    VERSION = '1.2'
+    # Version 1.3: Added tags
+    VERSION = '1.3'
 
     fields = {
         'id': fields.IntegerField(),
@@ -50,6 +48,7 @@ class BuildRequest(base.NovaObject):
         # created_at/updated_at. There is no soft delete for this object.
         'created_at': fields.DateTimeField(nullable=True),
         'updated_at': fields.DateTimeField(nullable=True),
+        'tags': fields.ObjectField('TagList'),
     }
 
     def obj_make_compatible(self, primitive, target_version):
@@ -58,6 +57,8 @@ class BuildRequest(base.NovaObject):
         target_version = versionutils.convert_version_to_tuple(target_version)
         if target_version < (1, 1) and 'block_device_mappings' in primitive:
             del primitive['block_device_mappings']
+        elif target_version < (1, 3) and 'tags' in primitive:
+            del primitive['tags']
 
     def _load_instance(self, db_instance):
         # NOTE(alaski): Be very careful with instance loading because it
@@ -67,9 +68,9 @@ class BuildRequest(base.NovaObject):
                     jsonutils.loads(db_instance))
         except TypeError:
             LOG.debug('Failed to load instance from BuildRequest with uuid '
-                      '%s because it is None' % (self.instance_uuid))
+                      '%s because it is None', self.instance_uuid)
             raise exception.BuildRequestNotFound(uuid=self.instance_uuid)
-        except ovoo_exc.IncompatibleObjectVersion as exc:
+        except ovoo_exc.IncompatibleObjectVersion:
             # This should only happen if proper service upgrade strategies are
             # not followed. Log the exception and raise BuildRequestNotFound.
             # If the instance can't be loaded this object is useless and may
@@ -78,13 +79,18 @@ class BuildRequest(base.NovaObject):
                       'with uuid %(instance_uuid)s. Found version %(version)s '
                       'which is not supported here.',
                       dict(instance_uuid=self.instance_uuid,
-                          version=exc.objver))
-            LOG.exception(_LE('Could not deserialize instance in '
-                              'BuildRequest'))
+                           version=jsonutils.loads(
+                               db_instance)["nova_object.version"]))
+            LOG.exception('Could not deserialize instance in BuildRequest')
             raise exception.BuildRequestNotFound(uuid=self.instance_uuid)
+        # NOTE(sbauza): The instance primitive should already have the deleted
+        # field being set, so when hydrating it back here, we should get the
+        # right value but in case we don't have it, let's suppose that the
+        # instance is not deleted, which is the default value for that field.
+        # NOTE(mriedem): Same for the "hidden" field.
+        self.instance.obj_set_defaults('deleted', 'hidden')
         # NOTE(alaski): Set some fields on instance that are needed by the api,
         # not lazy-loadable, and don't change.
-        self.instance.deleted = 0
         self.instance.disable_terminate = False
         self.instance.terminated_at = None
         self.instance.host = None
@@ -101,7 +107,7 @@ class BuildRequest(base.NovaObject):
         # was never persisted.
         self.instance.created_at = self.created_at
         self.instance.updated_at = self.updated_at
-        self.instance.tags = objects.TagList([])
+        self.instance.tags = self.tags
 
     def _load_block_device_mappings(self, db_bdms):
         # 'db_bdms' is a serialized BlockDeviceMappingList object. If it's None
@@ -119,6 +125,22 @@ class BuildRequest(base.NovaObject):
             objects.BlockDeviceMappingList.obj_from_primitive(
                 jsonutils.loads(db_bdms)))
 
+    def _load_tags(self, db_tags):
+        # 'db_tags' is a serialized TagList object. If it's None
+        # we're in a mixed version nova-api scenario and can't retrieve the
+        # actual list. Set it to an empty list here which will cause a
+        # temporary API inconsistency that will be resolved as soon as the
+        # instance is scheduled and on a compute.
+        if db_tags is None:
+            LOG.debug('Failed to load tags from BuildRequest '
+                      'for instance %s because it is None', self.instance_uuid)
+            self.tags = objects.TagList()
+            return
+
+        self.tags = (
+            objects.TagList.obj_from_primitive(
+                jsonutils.loads(db_tags)))
+
     @staticmethod
     def _from_db_object(context, req, db_req):
         # Set this up front so that it can be pulled for error messages or
@@ -132,7 +154,7 @@ class BuildRequest(base.NovaObject):
                 try:
                     getattr(req, '_load_%s' % key)(db_req[key])
                 except AttributeError:
-                    LOG.exception(_LE('No load handler for %s'), key)
+                    LOG.exception('No load handler for %s', key)
             else:
                 setattr(req, key, db_req[key])
         # Load instance last because other fields on req may be referenced
@@ -165,7 +187,7 @@ class BuildRequest(base.NovaObject):
 
     def _get_update_primitives(self):
         updates = self.obj_get_changes()
-        for key, value in six.iteritems(updates):
+        for key, value in updates.items():
             if isinstance(self.fields[key], fields.ObjectField):
                 updates[key] = jsonutils.dumps(value.obj_to_primitive())
         return updates
@@ -213,6 +235,23 @@ class BuildRequest(base.NovaObject):
         db_req = self._save_in_db(self._context, self.id, updates)
         self._from_db_object(self._context, self, db_req)
 
+    def get_new_instance(self, context):
+        # NOTE(danms): This is a hack to make sure that the returned
+        # instance has all dirty fields. There are probably better
+        # ways to do this, but they kinda involve o.vo internals
+        # so this is okay for the moment.
+        instance = objects.Instance(context)
+        for field in self.instance.obj_fields:
+            # NOTE(danms): Don't copy the defaulted tags field
+            # as instance.create() won't handle it properly.
+            # TODO(zhengzhenyu): Handle this when the API supports creating
+            # servers with tags.
+            if field == 'tags':
+                continue
+            if self.instance.obj_attr_is_set(field):
+                setattr(instance, field, getattr(self.instance, field))
+        return instance
+
 
 @base.NovaObjectRegistry.register
 class BuildRequestList(base.ObjectListBase, base.NovaObject):
@@ -255,6 +294,33 @@ class BuildRequestList(base.ObjectListBase, base.NovaObject):
                         if (k not in instance.metadata or
                                 v != instance.metadata[k]):
                             return False
+            elif filter_key in (
+                    'tags', 'tags-any', 'not-tags', 'not-tags-any'):
+                # Get the list of simple string tags first.
+                tags = ([tag.tag for tag in instance.tags]
+                        if instance.tags else [])
+                if filter_key == 'tags':
+                    for item in filter_val:
+                        if item not in tags:
+                            return False
+                elif filter_key == 'tags-any':
+                    found = []
+                    for item in filter_val:
+                        if item in tags:
+                            found.append(item)
+                    if not found:
+                        return False
+                elif filter_key == 'not-tags':
+                    found = []
+                    for item in filter_val:
+                        if item in tags:
+                            found.append(item)
+                    if len(found) == len(filter_val):
+                        return False
+                elif filter_key == 'not-tags-any':
+                    for item in filter_val:
+                        if item in tags:
+                            return False
             elif isinstance(filter_val, (list, tuple, set, frozenset)):
                 if not filter_val:
                     # Special value to indicate that nothing will match.
@@ -306,14 +372,16 @@ class BuildRequestList(base.ObjectListBase, base.NovaObject):
     @base.remotable_classmethod
     def get_by_filters(cls, context, filters, limit=None, marker=None,
                        sort_keys=None, sort_dirs=None):
-        if limit == 0:
-            return cls(context, objects=[])
+        # Short-circuit on anything that will not yield results.
         # 'deleted' records can not be returned from here since build_requests
         # are not soft deleted.
-        if filters.get('deleted', False):
-            return cls(context, objects=[])
         # 'cleaned' records won't exist as they would need to be deleted.
-        if filters.get('cleaned', False):
+        if (limit == 0 or
+                filters.get('deleted', False) or
+                filters.get('cleaned', False)):
+            # If we have a marker honor the MarkerNotFound semantics.
+            if marker:
+                raise exception.MarkerNotFound(marker=marker)
             return cls(context, objects=[])
 
         # Because the build_requests table stores an instance as a serialized
@@ -324,8 +392,6 @@ class BuildRequestList(base.ObjectListBase, base.NovaObject):
         build_requests = cls.get_all(context)
 
         # Fortunately some filters do not apply here.
-        # 'tags' can not be applied at boot time so will not be set for an
-        # instance here.
         # 'changes-since' works off of the updated_at field which has not yet
         # been set at the point in the boot process where build_request still
         # exists. So it can be ignored.
@@ -339,7 +405,8 @@ class BuildRequestList(base.ObjectListBase, base.NovaObject):
         exact_match_filter_names = ['project_id', 'user_id', 'image_ref',
                                     'vm_state', 'instance_type_id', 'uuid',
                                     'metadata', 'host', 'task_state',
-                                    'system_metadata']
+                                    'system_metadata', 'tags', 'tags-any',
+                                    'not-tags', 'not-tags-any']
         exact_filters = {}
         regex_filters = {}
         for key, value in filters.items():
@@ -369,7 +436,8 @@ class BuildRequestList(base.ObjectListBase, base.NovaObject):
 
             filtered_build_reqs.append(build_req)
 
-        if (len(filtered_build_reqs) < 2) or (not sort_keys):
+        if (((len(filtered_build_reqs) < 2) or (not sort_keys)) and
+                not marker):
             # No need to sort
             return cls(context, objects=filtered_build_reqs)
 
@@ -380,8 +448,13 @@ class BuildRequestList(base.ObjectListBase, base.NovaObject):
         if marker:
             for i, build_req in enumerate(sorted_build_reqs):
                 if build_req.instance.uuid == marker:
-                    marker_index = i
+                    # The marker is the last seen item in the last page, so
+                    # we increment the index to the next item immediately
+                    # after the marker so the marker is not returned.
+                    marker_index = i + 1
                     break
+            else:
+                raise exception.MarkerNotFound(marker=marker)
         len_build_reqs = len(sorted_build_reqs)
         limit_index = len_build_reqs
         if limit:
