@@ -13,12 +13,13 @@
 #    under the License.
 
 from oslo_log import log as logging
+from oslo_utils import uuidutils
 from oslo_utils import versionutils
 
 from nova import availability_zones
-from nova import db
+from nova import context as nova_context
+from nova.db import api as db
 from nova import exception
-from nova.i18n import _LW
 from nova.notifications.objects import base as notification
 from nova.notifications.objects import service as service_notification
 from nova import objects
@@ -30,7 +31,7 @@ LOG = logging.getLogger(__name__)
 
 
 # NOTE(danms): This is the global service version counter
-SERVICE_VERSION = 15
+SERVICE_VERSION = 53
 
 
 # NOTE(danms): This is our SERVICE_VERSION history. The idea is that any
@@ -92,6 +93,103 @@ SERVICE_VERSION_HISTORY = (
     # Version 15: Indicate that nova-conductor will stop a boot if BuildRequest
     # is deleted before RPC to nova-compute.
     {'compute_rpc': '4.13'},
+    # Version 16: Indicate that nova-compute will refuse to start if it doesn't
+    # have a placement section configured.
+    {'compute_rpc': '4.13'},
+    # Version 17: Add 'reserve_volume' to the boot from volume flow and
+    # remove 'check_attach'. The service version bump is needed to fall back to
+    # the old check in the API as the old computes fail if the volume is moved
+    # to 'attaching' state by reserve.
+    {'compute_rpc': '4.13'},
+    # Version 18: Compute RPC version 4.14
+    {'compute_rpc': '4.14'},
+    # Version 19: Compute RPC version 4.15
+    {'compute_rpc': '4.15'},
+    # Version 20: Compute RPC version 4.16
+    {'compute_rpc': '4.16'},
+    # Version 21: Compute RPC version 4.17
+    {'compute_rpc': '4.17'},
+    # Version 22: A marker for the behaviour change of auto-healing code on the
+    # compute host regarding allocations against an instance
+    {'compute_rpc': '4.17'},
+    # Version 23: Compute hosts allow pre-creation of the migration object
+    # for cold migration.
+    {'compute_rpc': '4.18'},
+    # Version 24: Add support for Cinder v3 attach/detach API.
+    {'compute_rpc': '4.18'},
+    # Version 25: Compute hosts allow migration-based allocations
+    # for live migration.
+    {'compute_rpc': '4.18'},
+    # Version 26: Adds a 'host_list' parameter to build_and_run_instance()
+    {'compute_rpc': '4.19'},
+    # Version 27: Compute RPC version 4.20; adds multiattach argument to
+    # reserve_block_device_name().
+    {'compute_rpc': '4.20'},
+    # Version 28: Adds a 'host_list' parameter to prep_resize()
+    {'compute_rpc': '4.21'},
+    # Version 29: Compute RPC version 4.22
+    {'compute_rpc': '4.22'},
+    # Version 30: Compute RPC version 5.0
+    {'compute_rpc': '5.0'},
+    # Version 31: The compute manager checks if 'trusted_certs' are supported
+    {'compute_rpc': '5.0'},
+    # Version 32: Add 'file_backed_memory' support. The service version bump is
+    # needed to allow the destination of a live migration to reject the
+    # migration if 'file_backed_memory' is enabled and the source does not
+    # support 'file_backed_memory'
+    {'compute_rpc': '5.0'},
+    # Version 33: Add support for check on the server group with
+    # 'max_server_per_host' rules
+    {'compute_rpc': '5.0'},
+    # Version 34: Adds support to abort queued/preparing live migrations.
+    {'compute_rpc': '5.0'},
+    # Version 35: Indicates that nova-compute supports live migration with
+    # ports bound early on the destination host using VIFMigrateData.
+    {'compute_rpc': '5.0'},
+    # Version 36: Indicates that nova-compute supports specifying volume
+    # type when booting a volume-backed server.
+    {'compute_rpc': '5.0'},
+    # Version 37: prep_resize takes a RequestSpec object
+    {'compute_rpc': '5.1'},
+    # Version 38: set_host_enabled reflects COMPUTE_STATUS_DISABLED trait
+    {'compute_rpc': '5.1'},
+    # Version 39: resize_instance, finish_resize, revert_resize,
+    # finish_revert_resize, unshelve_instance takes a RequestSpec object
+    {'compute_rpc': '5.2'},
+    # Version 40: Add migration and limits parameters to
+    # check_can_live_migrate_destination(), new
+    # drop_move_claim_at_destination() method, and numa_live_migration
+    # parameter to check_can_live_migrate_source()
+    {'compute_rpc': '5.3'},
+    # Version 41: Add cache_images() to compute rpcapi (version 5.4)
+    {'compute_rpc': '5.4'},
+    # Version 42: Compute RPC version 5.5; +prep_snapshot_based_resize_at_dest
+    {'compute_rpc': '5.5'},
+    # Version 43: Compute RPC version 5.6: prep_snapshot_based_resize_at_source
+    {'compute_rpc': '5.6'},
+    # Version 44: Compute RPC version 5.7: finish_snapshot_based_resize_at_dest
+    {'compute_rpc': '5.7'},
+    # Version 45: Compute RPC v5.8: confirm_snapshot_based_resize_at_source
+    {'compute_rpc': '5.8'},
+    # Version 46: Compute RPC v5.9: revert_snapshot_based_resize_at_dest
+    {'compute_rpc': '5.9'},
+    # Version 47: Compute RPC v5.10:
+    # finish_revert_snapshot_based_resize_at_source
+    {'compute_rpc': '5.10'},
+    # Version 48: Drivers report COMPUTE_SAME_HOST_COLD_MIGRATE trait.
+    {'compute_rpc': '5.10'},
+    # Version 49: Compute now support server move operations with qos ports
+    {'compute_rpc': '5.10'},
+    # Version 50: Compute RPC v5.11:
+    # Add accel_uuids (accelerator requests) param to build_and_run_instance
+    {'compute_rpc': '5.11'},
+    # Version 51: Add support for live migration with vpmem
+    {'compute_rpc': '5.11'},
+    # Version 52: Add support for the 'mixed' CPU allocation policy
+    {'compute_rpc': '5.11'},
+    # Version 53: Compute RPC v5.12:
+    # Add accel_uuids (accelerator requests) param to rebuild_instance
+    {'compute_rpc': '5.12'},
 )
 
 
@@ -120,10 +218,13 @@ class Service(base.NovaPersistentObject, base.NovaObject,
     # Version 1.18: ComputeNode version 1.14
     # Version 1.19: Added get_minimum_version()
     # Version 1.20: Added get_minimum_version_multi()
-    VERSION = '1.20'
+    # Version 1.21: Added uuid
+    # Version 1.22: Added get_by_uuid()
+    VERSION = '1.22'
 
     fields = {
         'id': fields.IntegerField(read_only=True),
+        'uuid': fields.UUIDField(),
         'host': fields.StringField(nullable=True),
         'binary': fields.StringField(nullable=True),
         'topic': fields.StringField(nullable=True),
@@ -163,6 +264,8 @@ class Service(base.NovaPersistentObject, base.NovaObject,
         super(Service, self).obj_make_compatible_from_manifest(
             primitive, target_version, version_manifest)
         _target_version = versionutils.convert_version_to_tuple(target_version)
+        if _target_version < (1, 21) and 'uuid' in primitive:
+            del primitive['uuid']
         if _target_version < (1, 16) and 'version' in primitive:
             del primitive['version']
         if _target_version < (1, 14) and 'forced_down' in primitive:
@@ -178,7 +281,7 @@ class Service(base.NovaPersistentObject, base.NovaObject,
     def _do_compute_node(self, context, primitive, version_manifest):
         try:
             target_version = version_manifest['ComputeNode']
-            # NOTE(sbauza): Some drivers (VMware, Ironic) can have multiple
+            # NOTE(sbauza): Ironic deployments can have multiple
             # nodes for the same service, but for keeping same behaviour,
             # returning only the first elem of the list
             compute = objects.ComputeNodeList.get_all_by_host(
@@ -202,10 +305,16 @@ class Service(base.NovaPersistentObject, base.NovaObject,
                 # NOTE(danms): Special handling of the version field, since
                 # it is read_only and set in our init.
                 setattr(service, base.get_attrname(key), db_service[key])
+            elif key == 'uuid' and not db_service.get(key):
+                # Leave uuid off the object if undefined in the database
+                # so that it will be generated below.
+                continue
             else:
                 service[key] = db_service[key]
+
         service._context = context
         service.obj_reset_changes()
+
         return service
 
     def obj_load_attr(self, attrname):
@@ -230,7 +339,7 @@ class Service(base.NovaPersistentObject, base.NovaObject,
             # NOTE(sbauza); Previous behaviour was raising a ServiceNotFound,
             # we keep it for backwards compatibility
             raise exception.ServiceNotFound(service_id=self.id)
-        # NOTE(sbauza): Some drivers (VMware, Ironic) can have multiple nodes
+        # NOTE(sbauza): Ironic deployments can have multiple nodes
         # for the same service, but for keeping same behaviour, returning only
         # the first elem of the list
         self.compute_node = compute_nodes[0]
@@ -240,6 +349,11 @@ class Service(base.NovaPersistentObject, base.NovaObject,
         db_service = db.service_get(context, service_id)
         return cls._from_db_object(context, cls(), db_service)
 
+    @base.remotable_classmethod
+    def get_by_uuid(cls, context, service_uuid):
+        db_service = db.service_get_by_uuid(context, service_uuid)
+        return cls._from_db_object(context, cls(), db_service)
+
     @base.remotable_classmethod
     def get_by_host_and_topic(cls, context, host, topic):
         db_service = db.service_get_by_host_and_topic(context, host, topic)
@@ -303,8 +417,14 @@ class Service(base.NovaPersistentObject, base.NovaObject,
                                               reason='already created')
         self._check_minimum_version()
         updates = self.obj_get_changes()
+
+        if 'uuid' not in updates:
+            updates['uuid'] = uuidutils.generate_uuid()
+            self.uuid = updates['uuid']
+
         db_service = db.service_create(self._context, updates)
         self._from_db_object(self._context, self, db_service)
+        self._send_notification(fields.NotificationAction.CREATE)
 
     @base.remotable
     def save(self):
@@ -322,19 +442,23 @@ class Service(base.NovaPersistentObject, base.NovaObject,
         # every other field change. See the comment in save() too.
         if set(updates.keys()).intersection(
                 {'disabled', 'disabled_reason', 'forced_down'}):
-            payload = service_notification.ServiceStatusPayload(self)
-            service_notification.ServiceStatusNotification(
-                publisher=notification.NotificationPublisher.from_service_obj(
-                    self),
-                event_type=notification.EventType(
-                    object='service',
-                    action=fields.NotificationAction.UPDATE),
-                priority=fields.NotificationPriority.INFO,
-                payload=payload).emit(self._context)
+            self._send_notification(fields.NotificationAction.UPDATE)
+
+    def _send_notification(self, action):
+        payload = service_notification.ServiceStatusPayload(self)
+        service_notification.ServiceStatusNotification(
+            publisher=notification.NotificationPublisher.from_service_obj(
+                self),
+            event_type=notification.EventType(
+                object='service',
+                action=action),
+            priority=fields.NotificationPriority.INFO,
+            payload=payload).emit(self._context)
 
     @base.remotable
     def destroy(self):
         db.service_destroy(self._context, self.id)
+        self._send_notification(fields.NotificationAction.DELETE)
 
     @classmethod
     def enable_min_version_cache(cls):
@@ -353,8 +477,8 @@ class Service(base.NovaPersistentObject, base.NovaObject,
     @base.remotable_classmethod
     def get_minimum_version_multi(cls, context, binaries, use_slave=False):
         if not all(binary.startswith('nova-') for binary in binaries):
-            LOG.warning(_LW('get_minimum_version called with likely-incorrect '
-                            'binaries `%s\''), ','.join(binaries))
+            LOG.warning('get_minimum_version called with likely-incorrect '
+                        'binaries `%s\'', ','.join(binaries))
             raise exception.ObjectActionError(action='get_minimum_version',
                                               reason='Invalid binary prefix')
 
@@ -388,6 +512,74 @@ class Service(base.NovaPersistentObject, base.NovaObject,
                                              use_slave=use_slave)
 
 
+def get_minimum_version_all_cells(context, binaries, require_all=False):
+    """Get the minimum service version, checking all cells.
+
+    This attempts to calculate the minimum service version for a set
+    of binaries across all the cells in the system. If require_all
+    is False, then any cells that fail to report a version will be
+    ignored (assuming they won't be candidates for scheduling and thus
+    excluding them from the minimum version calculation is reasonable).
+    If require_all is True, then a failing cell will cause this to raise
+    exception.CellTimeout, as would be appropriate for gating some
+    data migration until everything is new enough.
+
+    Note that services that do not report a positive version are excluded
+    from this, as it crosses all cells which will naturally not have all
+    services.
+    """
+
+    if not all(binary.startswith('nova-') for binary in binaries):
+        LOG.warning('get_minimum_version_all_cells called with '
+                    'likely-incorrect binaries `%s\'', ','.join(binaries))
+        raise exception.ObjectActionError(
+            action='get_minimum_version_all_cells',
+            reason='Invalid binary prefix')
+
+    # NOTE(danms): Instead of using Service.get_minimum_version_multi(), we
+    # replicate the call directly to the underlying DB method here because
+    # we want to defeat the caching and we need to filter non-present
+    # services differently from the single-cell method.
+
+    results = nova_context.scatter_gather_all_cells(
+        context,
+        Service._db_service_get_minimum_version,
+        binaries)
+
+    min_version = None
+    for cell_uuid, result in results.items():
+        if result is nova_context.did_not_respond_sentinel:
+            LOG.warning('Cell %s did not respond when getting minimum '
+                        'service version', cell_uuid)
+            if require_all:
+                raise exception.CellTimeout()
+        elif isinstance(result, Exception):
+            LOG.warning('Failed to get minimum service version for cell %s',
+                        cell_uuid)
+            if require_all:
+                # NOTE(danms): Okay, this isn't necessarily a timeout, but
+                # it's functionally the same from the caller's perspective
+                # and we logged the fact that it was actually a failure
+                # for the forensic investigator during the scatter/gather
+                # routine.
+                raise exception.CellTimeout()
+        else:
+            # NOTE(danms): Don't consider a zero or None result as the minimum
+            # since we're crossing cells and will likely not have all the
+            # services being probed.
+            relevant_versions = [version for version in result.values()
+                                 if version]
+            if relevant_versions:
+                min_version_cell = min(relevant_versions)
+                min_version = (min(min_version, min_version_cell)
+                               if min_version else min_version_cell)
+
+    # NOTE(danms): If we got no matches at all (such as at first startup)
+    # then report that as zero to be consistent with the other such
+    # methods.
+    return min_version or 0
+
+
 @base.NovaObjectRegistry.register
 class ServiceList(base.ObjectListBase, base.NovaObject):
     # Version 1.0: Initial version
