@@ -19,16 +19,20 @@ from nova.policies import base
 
 
 BASE_POLICY_NAME = 'os_compute_api:os-console-output'
-POLICY_ROOT = 'os_compute_api:os-console-output:%s'
 
 
 console_output_policies = [
-    policy.RuleDefault(
-        name=POLICY_ROOT % 'discoverable',
-        check_str=base.RULE_ANY),
-    policy.RuleDefault(
+    policy.DocumentedRuleDefault(
         name=BASE_POLICY_NAME,
-        check_str=base.RULE_ADMIN_OR_OWNER),
+        check_str=base.PROJECT_MEMBER_OR_SYSTEM_ADMIN,
+        description='Show console output for a server',
+        operations=[
+            {
+                'method': 'POST',
+                'path': '/servers/{server_id}/action (os-getConsoleOutput)'
+            }
+        ],
+        scope_types=['system', 'project'])
 ]
 
 
