@@ -19,16 +19,20 @@ from nova.policies import base
 
 
 BASE_POLICY_NAME = 'os_compute_api:os-evacuate'
-POLICY_ROOT = 'os_compute_api:os-evacuate:%s'
 
 
 evacuate_policies = [
-    policy.RuleDefault(
-        name=POLICY_ROOT % 'discoverable',
-        check_str=base.RULE_ANY),
-    policy.RuleDefault(
+    policy.DocumentedRuleDefault(
         name=BASE_POLICY_NAME,
-        check_str=base.RULE_ADMIN_API),
+        check_str=base.SYSTEM_ADMIN,
+        description="Evacuate a server from a failed host to a new host",
+        operations=[
+            {
+                'path': '/servers/{server_id}/action (evacuate)',
+                'method': 'POST'
+            }
+        ],
+        scope_types=['system', 'project']),
 ]
 
 
