@@ -22,15 +22,28 @@ POLICY_ROOT = 'os_compute_api:os-suspend-server:%s'
 
 
 suspend_server_policies = [
-    policy.RuleDefault(
+    policy.DocumentedRuleDefault(
         name=POLICY_ROOT % 'resume',
-        check_str=base.RULE_ADMIN_OR_OWNER),
-    policy.RuleDefault(
+        check_str=base.PROJECT_MEMBER_OR_SYSTEM_ADMIN,
+        description="Resume suspended server",
+        operations=[
+            {
+                'method': 'POST',
+                'path': '/servers/{server_id}/action (resume)'
+            }
+        ],
+        scope_types=['system', 'project']),
+    policy.DocumentedRuleDefault(
         name=POLICY_ROOT % 'suspend',
-        check_str=base.RULE_ADMIN_OR_OWNER),
-    policy.RuleDefault(
-        name=POLICY_ROOT % 'discoverable',
-        check_str=base.RULE_ANY),
+        check_str=base.PROJECT_MEMBER_OR_SYSTEM_ADMIN,
+        description="Suspend server",
+        operations=[
+            {
+                'method': 'POST',
+                'path': '/servers/{server_id}/action (suspend)'
+            }
+        ],
+        scope_types=['system', 'project']),
 ]
 
 
