@@ -67,7 +67,6 @@ def objectify(data):
 
 
 class ApiSampleTestBase(integrated_helpers._IntegratedTestBase):
-    all_extensions = True
     sample_dir = None
     microversion = None
     _use_common_server_api_samples = False
@@ -246,7 +245,15 @@ class ApiSampleTestBase(integrated_helpers._IntegratedTestBase):
 
             expected = expected % self.subs
             expected = '^%s$' % expected
-            match = re.match(expected, result)
+            try:
+                match = re.match(expected, result)
+            except TypeError as e:
+                raise NoMatch(
+                    'Values do not match:\n'
+                    'Template: %(expected)s\n%(result_str)s: %(result)s\n'
+                    'Error: %(error)s' %
+                    {'expected': expected, 'result_str': result_str,
+                     'result': result, 'error': e})
             if not match:
                 raise NoMatch(
                     'Values do not match:\n'
@@ -303,6 +310,24 @@ class ApiSampleTestBase(integrated_helpers._IntegratedTestBase):
 
         return matched_value
 
+    @property
+    def project_id(self):
+        # We'll allow test cases to override the default project id. This is
+        # useful when using multiple tenants.
+        project_id = None
+        try:
+            project_id = self.api.project_id
+        except AttributeError:
+            pass
+
+        return project_id or PROJECT_ID
+
+    @project_id.setter
+    def project_id(self, project_id):
+        self.api.project_id = project_id
+        # Reset cached credentials
+        self.api.auth_result = None
+
     def generalize_subs(self, subs, vanilla_regexes):
         """Give the test a chance to modify subs after the server response
         was verified, and before the on-disk doc/api_samples file is checked.
@@ -317,17 +342,20 @@ class ApiSampleTestBase(integrated_helpers._IntegratedTestBase):
     def _update_links(self, sample_data):
         """Process sample data and update version specific links."""
         # replace version urls
-        url_re = self._get_host() + "/v(2|2\.1)/" + PROJECT_ID
+
+        project_id_exp = '(%s|%s)' % (PROJECT_ID, self.project_id)
+
+        url_re = self._get_host() + r"/v(2|2\.1)/" + project_id_exp
         new_url = self._get_host() + "/" + self.api_major_version
-        if self._project_id:
-            new_url += "/" + PROJECT_ID
+        if self.USE_PROJECT_ID:
+            new_url += "/" + self.project_id
         updated_data = re.sub(url_re, new_url, sample_data)
 
         # replace unversioned urls
-        url_re = self._get_host() + "/" + PROJECT_ID
+        url_re = self._get_host() + "/" + project_id_exp
         new_url = self._get_host()
-        if self._project_id:
-            new_url += "/" + PROJECT_ID
+        if self.USE_PROJECT_ID:
+            new_url += "/" + self.project_id
         updated_data = re.sub(url_re, new_url, updated_data)
         return updated_data
 
@@ -342,7 +370,8 @@ class ApiSampleTestBase(integrated_helpers._IntegratedTestBase):
         regexes.update(subs)
         subs = regexes
         self.subs = subs
-        self.assertEqual(exp_code, response.status_code)
+        message = response.text if response.status_code >= 400 else None
+        self.assertEqual(exp_code, response.status_code, message)
         response_data = response.content
         response_data = pretty_data(response_data)
         if not os.path.exists(self._get_template(name,
@@ -372,7 +401,7 @@ class ApiSampleTestBase(integrated_helpers._IntegratedTestBase):
         except NoMatch as e:
             raise NoMatch("\nFailed to match Template to Response: \n%s\n"
                           "Template: %s\n\n"
-                          "Respones: %s\n\n" %
+                          "Response: %s\n\n" %
                           (e,
                            pp.pformat(template_data),
                            pp.pformat(response_data)))
@@ -395,7 +424,9 @@ class ApiSampleTestBase(integrated_helpers._IntegratedTestBase):
         except NoMatch as e:
             raise NoMatch("\nFailed to match Template to Sample: \n%s\n"
                           "Template: %s\n\n"
-                          "Sample: %s\n\n" %
+                          "Sample: %s\n\n"
+                          "Hint: does your test need to override "
+                          "ApiSampleTestBase.generalize_subs()?" %
                           (e,
                            pp.pformat(template_data),
                            pp.pformat(sample_data)))
@@ -408,17 +439,20 @@ class ApiSampleTestBase(integrated_helpers._IntegratedTestBase):
 
     def _get_regexes(self):
         text = r'(\\"|[^"])*'
-        isotime_re = '\d{4}-[0,1]\d-[0-3]\dT\d{2}:\d{2}:\d{2}Z'
-        strtime_re = '\d{4}-[0,1]\d-[0-3]\dT\d{2}:\d{2}:\d{2}\.\d{6}'
-        xmltime_re = ('\d{4}-[0,1]\d-[0-3]\d '
-                      '\d{2}:\d{2}:\d{2}'
-                      '(\.\d{6})?(\+00:00)?')
+        isotime_re = r'\d{4}-[0,1]\d-[0-3]\dT\d{2}:\d{2}:\d{2}Z'
+        strtime_re = r'\d{4}-[0,1]\d-[0-3]\dT\d{2}:\d{2}:\d{2}\.\d{6}'
+        strtime_url_re = (r'\d{4}-[0,1]\d-[0-3]\d'
+                          r'\+\d{2}\%3A\d{2}\%3A\d{2}\.\d{6}')
+        xmltime_re = (r'\d{4}-[0,1]\d-[0-3]\d '
+                      r'\d{2}:\d{2}:\d{2}'
+                      r'(\.\d{6})?(\+00:00)?')
 
         # NOTE(claudiub): the x509 keypairs are different from the
         # ssh keypairs. For example, the x509 fingerprint has 40 bytes.
         return {
             'isotime': isotime_re,
             'strtime': strtime_re,
+            'strtime_url': strtime_url_re,
             'strtime_or_none': r'None|%s' % strtime_re,
             'xmltime': xmltime_re,
             'password': '[0-9a-zA-Z]{1,12}',
@@ -428,6 +462,8 @@ class ApiSampleTestBase(integrated_helpers._IntegratedTestBase):
                   '-[0-9a-f]{4}-[0-9a-f]{12})',
             'uuid': '[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}'
                     '-[0-9a-f]{4}-[0-9a-f]{12}',
+            'request_id': 'req-[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}'
+                          '-[0-9a-f]{4}-[0-9a-f]{12}',
             'reservation_id': 'r-[0-9a-zA-Z]{8}',
             'private_key': '(-----BEGIN RSA PRIVATE KEY-----|)'
                            '[a-zA-Z0-9\n/+=]*'
@@ -453,17 +489,17 @@ class ApiSampleTestBase(integrated_helpers._IntegratedTestBase):
     def _get_compute_endpoint(self):
         # NOTE(sdague): "openstack" is stand in for project_id, it
         # should be more generic in future.
-        if self._project_id:
-            return '%s/%s' % (self._get_host(), PROJECT_ID)
+        if self.USE_PROJECT_ID:
+            return '%s/%s' % (self._get_host(), self.project_id)
         else:
             return self._get_host()
 
     def _get_vers_compute_endpoint(self):
         # NOTE(sdague): "openstack" is stand in for project_id, it
         # should be more generic in future.
-        if self._project_id:
+        if self.USE_PROJECT_ID:
             return '%s/%s/%s' % (self._get_host(), self.api_major_version,
-                                 PROJECT_ID)
+                                 self.project_id)
         else:
             return '%s/%s' % (self._get_host(), self.api_major_version)
 
@@ -479,15 +515,21 @@ class ApiSampleTestBase(integrated_helpers._IntegratedTestBase):
         return self._get_response(url, 'OPTIONS', strip_version=strip_version,
                                   headers=headers)
 
-    def _do_get(self, url, strip_version=False, headers=None):
-        return self._get_response(url, 'GET', strip_version=strip_version,
-                                  headers=headers)
+    def _do_get(self, url, strip_version=False, headers=None,
+                return_json_body=False):
+        response = self._get_response(url, 'GET', strip_version=strip_version,
+                                      headers=headers)
+        if return_json_body and hasattr(response, 'content'):
+            return jsonutils.loads(response.content)
+        return response
 
-    def _do_post(self, url, name, subs, method='POST', headers=None):
-        self.subs = subs
-        body = self._read_template(name) % self.subs
-        sample = self._get_sample(name, self.microversion)
-        if self.generate_samples and not os.path.exists(sample):
+    def _do_post(self, url, name=None, subs=None, method='POST', headers=None):
+        self.subs = {} if subs is None else subs
+        body = None
+        if name:
+            body = self._read_template(name) % self.subs
+            sample = self._get_sample(name, self.microversion)
+            if self.generate_samples and not os.path.exists(sample):
                 self._write_sample(name, body)
         return self._get_response(url, method, body, headers=headers)
 
