@@ -11,22 +11,20 @@
 #    under the License.
 
 from copy import deepcopy
+
 import mock
 from oslo_db import exception as db_exc
+from oslo_utils.fixture import uuidsentinel
 from oslo_utils import timeutils
 
 from nova import context
-from nova import db
 from nova.db.sqlalchemy import api as db_api
 from nova.db.sqlalchemy import api_models
 from nova import exception
+import nova.objects.aggregate as aggregate_obj
 from nova import test
-from nova.tests import fixtures
 from nova.tests.unit import matchers
 from nova.tests.unit.objects.test_objects import compare_obj as base_compare
-from nova.tests import uuidsentinel
-
-import nova.objects.aggregate as aggregate_obj
 
 
 SUBS = {'metadata': 'metadetails'}
@@ -110,31 +108,12 @@ def _aggregate_metadata_get_all(context, aggregate_id):
     return metadata
 
 
-class AggregateObjectDbTestCase(test.NoDBTestCase):
-
-    USES_DB_SELF = True
+class AggregateObjectDbTestCase(test.TestCase):
 
     def setUp(self):
         super(AggregateObjectDbTestCase, self).setUp()
-        self.useFixture(fixtures.Database())
-        self.useFixture(fixtures.Database(database='api'))
         self.context = context.RequestContext('fake-user', 'fake-project')
 
-    def test_in_api(self):
-        ca1 = _create_aggregate(self.context, values={'name': 'fake_agg_1',
-                                                      'id': 1,
-                                                      'uuid': 'nonce'})
-        ca2 = db.aggregate_create(self.context, {'name': 'fake_agg_2',
-                                                 'id': 2,
-                                                 'uuid': 'nonce'})
-
-        api_db_agg = aggregate_obj.Aggregate.get_by_id(self.context, ca1['id'])
-        cell_db_agg = aggregate_obj.Aggregate.get_by_id(
-                                                    self.context, ca2['id'])
-
-        self.assertEqual(api_db_agg.in_api, True)
-        self.assertEqual(cell_db_agg.in_api, False)
-
     def test_aggregate_get_from_db(self):
         result = _create_aggregate_with_hosts(self.context)
         expected = aggregate_obj._aggregate_get_from_db(self.context,
@@ -202,7 +181,7 @@ class AggregateObjectDbTestCase(test.NoDBTestCase):
         _create_aggregate(self.context,
                           values={'name': 'aggregate_3'},
                           metadata={'badkey': 'good'})
-        rl1 = aggregate_obj._get_by_metadata_key_from_db(self.context,
+        rl1 = aggregate_obj._get_by_metadata_from_db(self.context,
                                                      key='goodkey')
         self.assertEqual(2, len(rl1))
 
@@ -225,8 +204,8 @@ class AggregateObjectDbTestCase(test.NoDBTestCase):
                                                 metadata={'goodkey': 'good'})
         result = aggregate_obj._aggregate_get_from_db(self.context,
                                                       agg.id)
-        md = aggregate_obj._get_by_metadata_key_from_db(self.context,
-                                                        key='goodkey')
+        md = aggregate_obj._get_by_metadata_from_db(self.context,
+                                                    key='goodkey')
         self.assertEqual(len(md), 1)
         self.assertEqual(md[0]['id'], agg.id)
         self.assertEqual(result.name, fake_create_aggregate['name'])
@@ -470,19 +449,12 @@ class AggregateObjectDbTestCase(test.NoDBTestCase):
                           self.context, result['id'], 'foo_key')
 
 
-def create_aggregate(context, db_id, in_api=True):
-    if in_api:
-        fake_aggregate = _get_fake_aggregate(db_id, in_api=False, result=False)
-        aggregate_obj._aggregate_create_in_db(context, fake_aggregate,
-                                       metadata=_get_fake_metadata(db_id))
-        for host in _get_fake_hosts(db_id):
-            aggregate_obj._host_add_to_db(context, fake_aggregate['id'], host)
-    else:
-        fake_aggregate = _get_fake_aggregate(db_id, in_api=False, result=False)
-        db.aggregate_create(context, fake_aggregate,
-                            metadata=_get_fake_metadata(db_id))
-        for host in _get_fake_hosts(db_id):
-            db.aggregate_host_add(context, fake_aggregate['id'], host)
+def create_aggregate(context, db_id):
+    fake_aggregate = _get_fake_aggregate(db_id, in_api=False, result=False)
+    aggregate_obj._aggregate_create_in_db(context, fake_aggregate,
+                                          metadata=_get_fake_metadata(db_id))
+    for host in _get_fake_hosts(db_id):
+        aggregate_obj._host_add_to_db(context, fake_aggregate['id'], host)
 
 
 def compare_obj(test, result, source):
@@ -495,20 +467,22 @@ def compare_obj(test, result, source):
                         comparators={'updated_at': updated_at_comparator})
 
 
-class AggregateObjectCellTestCase(test.NoDBTestCase):
-    """Tests for the case where all aggregate data is in Cell DB"""
-    USES_DB_SELF = True
-
+class AggregateObjectTestCase(test.TestCase):
     def setUp(self):
-        super(AggregateObjectCellTestCase, self).setUp()
+        super(AggregateObjectTestCase, self).setUp()
         self.context = context.RequestContext('fake-user', 'fake-project')
-        self.useFixture(fixtures.Database())
-        self.useFixture(fixtures.Database(database='api'))
         self._seed_data()
 
     def _seed_data(self):
         for i in range(1, 10):
-            create_aggregate(self.context, i, in_api=False)
+            create_aggregate(self.context, i)
+
+    def test_create(self):
+        new_agg = aggregate_obj.Aggregate(self.context)
+        new_agg.name = 'new-aggregate'
+        new_agg.create()
+        result = aggregate_obj.Aggregate.get_by_id(self.context, new_agg.id)
+        self.assertEqual(new_agg.name, result.name)
 
     def test_get_by_id(self):
         for i in range(1, 10):
@@ -565,84 +539,143 @@ class AggregateObjectCellTestCase(test.NoDBTestCase):
             compare_obj(self, agg, fake_agg)
             compare_obj(self, result, fake_agg)
 
+    def test_get_by_metadata(self):
+        agg = aggregate_obj.Aggregate.get_by_id(self.context, 1)
+        agg.update_metadata({'foo': 'bar'})
 
-class AggregateObjectApiTestCase(AggregateObjectCellTestCase):
-    """Tests the aggregate in the case where all data is in the API DB"""
-    def _seed_data(self):
-        for i in range(1, 10):
-            create_aggregate(self.context, i)
+        agg = aggregate_obj.Aggregate.get_by_id(self.context, 2)
+        agg.update_metadata({'foo': 'baz',
+                             'fu': 'bar'})
 
-    def test_create(self):
-        new_agg = aggregate_obj.Aggregate(self.context)
-        new_agg.name = 'new-aggregate'
-        new_agg.create()
-        result = aggregate_obj.Aggregate.get_by_id(self.context, new_agg.id)
-        self.assertEqual(new_agg.name, result.name)
+        aggs = aggregate_obj.AggregateList.get_by_metadata(
+            self.context, key='foo', value='bar')
+        self.assertEqual(1, len(aggs))
+        self.assertEqual(1, aggs[0].id)
 
+        aggs = aggregate_obj.AggregateList.get_by_metadata(
+            self.context, value='bar')
+        self.assertEqual(2, len(aggs))
+        self.assertEqual(set([1, 2]), set([a.id for a in aggs]))
 
-class AggregateObjectMixedTestCase(AggregateObjectCellTestCase):
-    """Tests the aggregate in the case where data is in both databases"""
-    def _seed_data(self):
-        for i in range(1, 6):
-            create_aggregate(self.context, i)
-        for i in range(6, 10):
-            create_aggregate(self.context, i, in_api=False)
+    def test_get_by_metadata_from_db_assertion(self):
+        self.assertRaises(AssertionError,
+                          aggregate_obj._get_by_metadata_from_db,
+                          self.context)
 
-    def test_create(self):
-        new_agg = aggregate_obj.Aggregate(self.context)
-        new_agg.name = 'new-aggregate'
-        self.assertRaises(exception.ObjectActionError,
-                          new_agg.create)
+    def test_get_non_matching_by_metadata_keys(self):
+        """Test aggregates that are not matching with metadata."""
 
+        agg = aggregate_obj.Aggregate.get_by_id(self.context, 1)
+        agg.update_metadata({'trait:HW_CPU_X86_MMX': 'required'})
 
-class AggregateObjectMigrationTestCase(AggregateObjectCellTestCase):
-    """Tests the aggregate in the case where data is migrated to the API db"""
-    def _seed_data(self):
-        for i in range(1, 10):
-            create_aggregate(self.context, i, in_api=False)
-        aggregate_obj.migrate_aggregates(self.context, 50)
+        agg = aggregate_obj.Aggregate.get_by_id(self.context, 2)
+        agg.update_metadata({'trait:HW_CPU_X86_SGX': 'required'})
 
-    def test_create(self):
-        new_agg = aggregate_obj.Aggregate(self.context)
-        new_agg.name = 'new-aggregate'
-        new_agg.create()
-        result = aggregate_obj.Aggregate.get_by_id(self.context, new_agg.id)
-        self.assertEqual(new_agg.name, result.name)
+        agg = aggregate_obj.Aggregate.get_by_id(self.context, 3)
+        agg.update_metadata({'trait:HW_CPU_X86_MMX': 'required',
+                             'trait:HW_CPU_X86_SGX': 'required'})
 
+        agg = aggregate_obj.Aggregate.get_by_id(self.context, 4)
+        agg.update_metadata({'trait:HW_CPU_X86_MMX': 'required',
+                             'trait:HW_CPU_X86_SGX': 'just_for_marking'})
 
-class AggregateMigrationTestCase(test.NoDBTestCase):
-    USES_DB_SELF = True
+        agg = aggregate_obj.Aggregate.get_by_id(self.context, 5)
+        agg.update_metadata({'trait:HW_CPU_X86_MMX': 'just_for_marking'})
 
-    def setUp(self):
-        super(AggregateMigrationTestCase, self).setUp()
-        self.useFixture(fixtures.Database())
-        self.useFixture(fixtures.Database(database='api'))
-        self.context = context.get_admin_context()
+        aggs = aggregate_obj.AggregateList.get_non_matching_by_metadata_keys(
+            self.context, ['trait:HW_CPU_X86_MMX'], 'trait:',
+            value='required')
 
-    def test_migration(self):
-        db.aggregate_create(self.context, {'name': 'foo'})
-        main_aggregates_len = len(db.aggregate_get_all(self.context))
-        match, done = aggregate_obj.migrate_aggregates(self.context, 50)
-        self.assertEqual(1, main_aggregates_len)
-        self.assertEqual(main_aggregates_len, match)
-        self.assertEqual(main_aggregates_len, done)
-        self.assertEqual(0, len(db.aggregate_get_all(self.context)))
-        self.assertEqual(main_aggregates_len,
-                         len(aggregate_obj.AggregateList.get_all(
-                                                                self.context)))
+        self.assertEqual(2, len(aggs))
+        self.assertCountEqual([2, 3], [a.id for a in aggs])
 
-    def test_migrate_aggregate_reset_autoincrement(self):
-        agg = aggregate_obj.Aggregate(self.context, name='foo')
-        agg.create()
-        match, done = aggregate_obj.migrate_aggregate_reset_autoincrement(
-            self.context, 0)
-        self.assertEqual(0, match)
-        self.assertEqual(0, done)
+    def test_matching_aggregates_multiple_keys(self):
+        """All matching aggregates for multiple keys."""
 
-    def test_migrate_aggregate_reset_autoincrement_no_aggregates(self):
-        # NOTE(danms): This validates the "or 0" default if there are no
-        # aggregates (and thus no max id).
-        match, done = aggregate_obj.migrate_aggregate_reset_autoincrement(
-            self.context, 0)
-        self.assertEqual(0, match)
-        self.assertEqual(0, done)
+        agg = aggregate_obj.Aggregate.get_by_id(self.context, 1)
+        agg.update_metadata({'trait:HW_CPU_X86_MMX': 'required'})
+
+        agg = aggregate_obj.Aggregate.get_by_id(self.context, 2)
+        agg.update_metadata({'trait:HW_CPU_X86_SGX': 'required'})
+
+        agg = aggregate_obj.Aggregate.get_by_id(self.context, 3)
+        agg.update_metadata({'trait:HW_CPU_X86_MMX': 'required',
+                             'trait:HW_CPU_X86_SGX': 'required'})
+
+        agg = aggregate_obj.Aggregate.get_by_id(self.context, 4)
+        agg.update_metadata({'trait:HW_CPU_X86_MMX': 'required',
+                             'trait:HW_CPU_X86_SGX': 'just_for_marking'})
+
+        aggs = aggregate_obj.AggregateList.get_non_matching_by_metadata_keys(
+            self.context, ['trait:HW_CPU_X86_MMX', 'trait:HW_CPU_X86_SGX'],
+            'trait:', value='required')
+
+        self.assertEqual(0, len(aggs))
+
+    def test_get_non_matching_aggregates_multiple_keys(self):
+        """Return non matching aggregates for multiple keys."""
+
+        agg = aggregate_obj.Aggregate.get_by_id(self.context, 1)
+        agg.update_metadata({'trait:HW_CPU_X86_MMX': 'required'})
+
+        agg = aggregate_obj.Aggregate.get_by_id(self.context, 2)
+        agg.update_metadata({'trait:HW_CPU_X86_MMX': 'required',
+                             'trait:HW_CPU_X86_SGX': 'required',
+                             'trait:HW_CPU_API_DXVA': 'required'})
+
+        agg = aggregate_obj.Aggregate.get_by_id(self.context, 3)
+        agg.update_metadata({'trait:HW_CPU_X86_MMX': 'required',
+                             'trait:HW_CPU_X86_SGX': 'required'})
+
+        agg = aggregate_obj.Aggregate.get_by_id(self.context, 4)
+        agg.update_metadata({'trait:HW_CPU_X86_MMX': 'required',
+                             'trait:HW_CPU_X86_SGX': 'just_for_marking'})
+
+        agg = aggregate_obj.Aggregate.get_by_id(self.context, 5)
+        agg.update_metadata({'trait:HW_CPU_X86_MMX': 'required',
+                             'trait:HW_CPU_X86_SGX': 'just_for_marking',
+                             'trait:HW_CPU_X86_SSE': 'required'})
+
+        aggs = aggregate_obj.AggregateList.get_non_matching_by_metadata_keys(
+            self.context, ['trait:HW_CPU_X86_MMX', 'trait:HW_CPU_X86_SGX'],
+            'trait:', value='required')
+
+        self.assertEqual(2, len(aggs))
+        self.assertCountEqual([2, 5], [a.id for a in aggs])
+
+    def test_get_non_matching_by_metadata_keys_empty_keys(self):
+        """Test aggregates non matching by metadata with empty keys."""
+
+        agg = aggregate_obj.Aggregate.get_by_id(self.context, 1)
+        agg.update_metadata({'trait:HW_CPU_X86_MMX': 'required'})
+
+        agg = aggregate_obj.Aggregate.get_by_id(self.context, 2)
+        agg.update_metadata({'trait:HW_CPU_X86_SGX': 'required'})
+
+        agg = aggregate_obj.Aggregate.get_by_id(self.context, 3)
+        agg.update_metadata({'trait:HW_CPU_X86_MMX': 'required',
+                             'trait:HW_CPU_X86_SGX': 'required'})
+
+        agg = aggregate_obj.Aggregate.get_by_id(self.context, 4)
+        agg.update_metadata({'trait:HW_CPU_X86_MMX': 'required',
+                             'trait:HW_CPU_X86_SGX': 'just_for_marking'})
+
+        agg = aggregate_obj.Aggregate.get_by_id(self.context, 5)
+        agg.update_metadata({'trait:HW_CPU_X86_MMX': 'required',
+                             'trait:HW_CPU_X86_SGX': 'just_for_marking',
+                             'trait:HW_CPU_X86_SSE': 'required'})
+
+        aggs = aggregate_obj.AggregateList.get_non_matching_by_metadata_keys(
+            self.context, [], 'trait:', value='required')
+
+        self.assertEqual(5, len(aggs))
+        self.assertCountEqual([1, 2, 3, 4, 5], [a.id for a in aggs])
+
+    def test_get_non_matching_by_metadata_keys_empty_key_prefix(self):
+        """Test aggregates non matching by metadata with empty key_prefix."""
+
+        self.assertRaises(
+                ValueError,
+                aggregate_obj.AggregateList.get_non_matching_by_metadata_keys,
+                self.context, ['trait:HW_CPU_X86_MMX'], '',
+                value='required')
