@@ -14,16 +14,18 @@
 #    under the License.
 
 import mock
+import six
 from webob import exc
 
+from nova.api.openstack import common
 from nova.api.openstack.compute import attach_interfaces \
         as attach_interfaces_v21
 from nova.compute import api as compute_api
 from nova import exception
-from nova.network import api as network_api
 from nova import objects
 from nova import test
 from nova.tests.unit.api.openstack import fakes
+from nova.tests.unit import fake_instance
 from nova.tests.unit import fake_network_cache_model
 
 
@@ -74,15 +76,7 @@ fake_networks = [FAKE_NET_ID1, FAKE_NET_ID2]
 ports = [port_data1, port_data2, port_data3]
 
 
-def fake_list_ports(self, *args, **kwargs):
-    result = []
-    for port in ports:
-        if port['device_id'] == kwargs['device_id']:
-            result.append(port)
-    return {'ports': result}
-
-
-def fake_show_port(self, context, port_id, **kwargs):
+def fake_show_port(context, port_id, **kwargs):
     for port in ports:
         if port['id'] == port_id:
             return {'port': port}
@@ -91,7 +85,7 @@ def fake_show_port(self, context, port_id, **kwargs):
 
 
 def fake_attach_interface(self, context, instance, network_id, port_id,
-                          requested_ip='192.168.1.3'):
+                          requested_ip='192.168.1.3', tag=None):
     if not network_id:
         # if no network_id is given when add a port to an instance, use the
         # first default network.
@@ -116,8 +110,10 @@ def fake_detach_interface(self, context, instance, port_id):
     raise exception.PortNotFound(port_id=port_id)
 
 
-def fake_get_instance(self, *args, **kwargs):
-    return objects.Instance(uuid=FAKE_UUID1)
+def fake_get_instance(self, context, instance_id, expected_attrs=None,
+                      cell_down_support=False):
+    return fake_instance.fake_instance_obj(
+            context, id=1, uuid=instance_id, project_id=context.project_id)
 
 
 class InterfaceAttachTestsV21(test.NoDBTestCase):
@@ -130,9 +126,7 @@ class InterfaceAttachTestsV21(test.NoDBTestCase):
     def setUp(self):
         super(InterfaceAttachTestsV21, self).setUp()
         self.flags(timeout=30, group='neutron')
-        self.stubs.Set(network_api.API, 'show_port', fake_show_port)
-        self.stubs.Set(network_api.API, 'list_ports', fake_list_ports)
-        self.stubs.Set(compute_api.API, 'get', fake_get_instance)
+        self.stub_out('nova.compute.api.API.get', fake_get_instance)
         self.expected_show = {'interfaceAttachment':
             {'net_id': FAKE_NET_ID1,
              'port_id': FAKE_PORT_ID1,
@@ -141,6 +135,10 @@ class InterfaceAttachTestsV21(test.NoDBTestCase):
              'fixed_ips': port_data1['fixed_ips'],
             }}
         self.attachments = self.controller_cls()
+        show_port_patch = mock.patch.object(self.attachments.network_api,
+                                            'show_port', fake_show_port)
+        show_port_patch.start()
+        self.addCleanup(show_port_patch.stop)
         self.req = fakes.HTTPRequest.blank('')
 
     @mock.patch.object(compute_api.API, 'get',
@@ -173,35 +171,43 @@ class InterfaceAttachTestsV21(test.NoDBTestCase):
                           self.attachments.show, self.req, FAKE_UUID2,
                           FAKE_PORT_ID1)
 
-    @mock.patch.object(network_api.API, 'show_port',
-                       side_effect=exception.Forbidden)
-    def test_show_forbidden(self, show_port_mock):
-        self.assertRaises(exc.HTTPForbidden,
-                          self.attachments.show, self.req, FAKE_UUID1,
-                          FAKE_PORT_ID1)
+    def test_show_forbidden(self):
+        with mock.patch.object(self.attachments.network_api, 'show_port',
+                               side_effect=exception.Forbidden):
+            self.assertRaises(exc.HTTPForbidden,
+                              self.attachments.show, self.req, FAKE_UUID1,
+                              FAKE_PORT_ID1)
 
     def test_delete(self):
-        self.stubs.Set(compute_api.API, 'detach_interface',
-                       fake_detach_interface)
-
-        result = self.attachments.delete(self.req, FAKE_UUID1, FAKE_PORT_ID1)
-        # NOTE: on v2.1, http status code is set as wsgi_code of API
-        # method instead of status_int in a response object.
-        if isinstance(self.attachments,
-                      attach_interfaces_v21.InterfaceAttachmentController):
-            status_int = self.attachments.delete.wsgi_code
-        else:
-            status_int = result.status_int
-        self.assertEqual(202, status_int)
+        self.stub_out('nova.compute.api.API.detach_interface',
+                      fake_detach_interface)
+        req_context = self.req.environ['nova.context']
+        inst = objects.Instance(uuid=FAKE_UUID1,
+                                project_id=req_context.project_id)
+        with mock.patch.object(common, 'get_instance',
+                               return_value=inst) as mock_get_instance:
+            result = self.attachments.delete(self.req, FAKE_UUID1,
+                                             FAKE_PORT_ID1)
+            # NOTE: on v2.1, http status code is set as wsgi_code of API
+            # method instead of status_int in a response object.
+            if isinstance(self.attachments,
+                          attach_interfaces_v21.InterfaceAttachmentController):
+                status_int = self.attachments.delete.wsgi_code
+            else:
+                status_int = result.status_int
+            self.assertEqual(202, status_int)
+            ctxt = self.req.environ['nova.context']
+            mock_get_instance.assert_called_with(
+                self.attachments.compute_api, ctxt, FAKE_UUID1,
+                expected_attrs=['device_metadata'])
 
     def test_detach_interface_instance_locked(self):
         def fake_detach_interface_from_locked_server(self, context,
             instance, port_id):
             raise exception.InstanceIsLocked(instance_uuid=FAKE_UUID1)
 
-        self.stubs.Set(compute_api.API,
-                       'detach_interface',
-                       fake_detach_interface_from_locked_server)
+        self.stub_out('nova.compute.api.API.detach_interface',
+                      fake_detach_interface_from_locked_server)
 
         self.assertRaises(exc.HTTPConflict,
                           self.attachments.delete,
@@ -210,8 +216,8 @@ class InterfaceAttachTestsV21(test.NoDBTestCase):
                           FAKE_PORT_ID1)
 
     def test_delete_interface_not_found(self):
-        self.stubs.Set(compute_api.API, 'detach_interface',
-                       fake_detach_interface)
+        self.stub_out('nova.compute.api.API.detach_interface',
+                      fake_detach_interface)
 
         self.assertRaises(exc.HTTPNotFound,
                           self.attachments.delete,
@@ -221,43 +227,50 @@ class InterfaceAttachTestsV21(test.NoDBTestCase):
 
     def test_attach_interface_instance_locked(self):
         def fake_attach_interface_to_locked_server(self, context,
-            instance, network_id, port_id, requested_ip):
+            instance, network_id, port_id, requested_ip, tag=None):
             raise exception.InstanceIsLocked(instance_uuid=FAKE_UUID1)
 
-        self.stubs.Set(compute_api.API,
-                       'attach_interface',
-                       fake_attach_interface_to_locked_server)
+        self.stub_out('nova.compute.api.API.attach_interface',
+                      fake_attach_interface_to_locked_server)
         body = {}
         self.assertRaises(exc.HTTPConflict,
                           self.attachments.create, self.req, FAKE_UUID1,
                           body=body)
 
     def test_attach_interface_without_network_id(self):
-        self.stubs.Set(compute_api.API, 'attach_interface',
-                       fake_attach_interface)
+        self.stub_out('nova.compute.api.API.attach_interface',
+                      fake_attach_interface)
         body = {}
         result = self.attachments.create(self.req, FAKE_UUID1, body=body)
         self.assertEqual(result['interfaceAttachment']['net_id'],
             FAKE_NET_ID1)
 
+    @mock.patch.object(
+        compute_api.API, 'attach_interface',
+        side_effect=exception.NetworkInterfaceTaggedAttachNotSupported())
+    def test_interface_tagged_attach_not_supported(self, mock_attach):
+        body = {'interfaceAttachment': {'net_id': FAKE_NET_ID2}}
+        self.assertRaises(exc.HTTPBadRequest, self.attachments.create,
+                          self.req, FAKE_UUID1, body=body)
+
     def test_attach_interface_with_network_id(self):
-        self.stubs.Set(compute_api.API, 'attach_interface',
-                       fake_attach_interface)
+        self.stub_out('nova.compute.api.API.attach_interface',
+                      fake_attach_interface)
         body = {'interfaceAttachment': {'net_id': FAKE_NET_ID2}}
         result = self.attachments.create(self.req, FAKE_UUID1, body=body)
         self.assertEqual(result['interfaceAttachment']['net_id'],
             FAKE_NET_ID2)
 
     def _attach_interface_bad_request_case(self, body):
-        self.stubs.Set(compute_api.API, 'attach_interface',
-                       fake_attach_interface)
+        self.stub_out('nova.compute.api.API.attach_interface',
+                      fake_attach_interface)
         self.assertRaises(exc.HTTPBadRequest,
                           self.attachments.create, self.req, FAKE_UUID1,
                           body=body)
 
     def _attach_interface_not_found_case(self, body):
-        self.stubs.Set(compute_api.API, 'attach_interface',
-                       fake_attach_interface)
+        self.stub_out('nova.compute.api.API.attach_interface',
+                      fake_attach_interface)
         self.assertRaises(self.not_found_exc,
                           self.attachments.create, self.req, FAKE_UUID1,
                           body=body)
@@ -293,21 +306,35 @@ class InterfaceAttachTestsV21(test.NoDBTestCase):
                 instance_uuid='', attr='', state='',
                 method='attach_interface')
 
-        self.stubs.Set(compute_api.API, 'attach_interface',
-                       fake_attach_interface_invalid_state)
+        self.stub_out('nova.compute.api.API.attach_interface',
+                      fake_attach_interface_invalid_state)
         body = {'interfaceAttachment': {'net_id': FAKE_NET_ID1}}
         self.assertRaises(exc.HTTPConflict,
                           self.attachments.create, self.req, FAKE_UUID1,
                           body=body)
 
+    def test_attach_interface_port_limit_exceeded(self):
+        """Tests the scenario where nova-compute attempts to create a port to
+        attach but the tenant port quota is exceeded and PortLimitExceeded
+        is raised from the neutron API code which results in a 403 response.
+        """
+        with mock.patch.object(self.attachments.compute_api,
+                               'attach_interface',
+                               side_effect=exception.PortLimitExceeded):
+            body = {'interfaceAttachment': {}}
+            ex = self.assertRaises(
+                exc.HTTPForbidden, self.attachments.create,
+                self.req, FAKE_UUID1, body=body)
+        self.assertIn('Maximum number of ports exceeded', six.text_type(ex))
+
     def test_detach_interface_with_invalid_state(self):
         def fake_detach_interface_invalid_state(*args, **kwargs):
             raise exception.InstanceInvalidState(
                 instance_uuid='', attr='', state='',
                 method='detach_interface')
 
-        self.stubs.Set(compute_api.API, 'detach_interface',
-                       fake_detach_interface_invalid_state)
+        self.stub_out('nova.compute.api.API.detach_interface',
+                      fake_detach_interface_invalid_state)
         self.assertRaises(exc.HTTPConflict,
                           self.attachments.delete,
                           self.req,
@@ -337,7 +364,9 @@ class InterfaceAttachTestsV21(test.NoDBTestCase):
     def test_attach_interface_fixed_ip_already_in_use(self,
                                                       attach_mock,
                                                       get_mock):
-        fake_instance = objects.Instance(uuid=FAKE_UUID1)
+        req_context = self.req.environ['nova.context']
+        fake_instance = objects.Instance(uuid=FAKE_UUID1,
+                                         project_id=req_context.project_id)
         get_mock.return_value = fake_instance
         attach_mock.side_effect = exception.FixedIpAlreadyInUse(
             address='10.0.2.2', instance_uuid=FAKE_UUID1)
@@ -347,16 +376,19 @@ class InterfaceAttachTestsV21(test.NoDBTestCase):
                           body=body)
         ctxt = self.req.environ['nova.context']
         attach_mock.assert_called_once_with(ctxt, fake_instance, None,
-                                            None, None)
+                                            None, None, tag=None)
         get_mock.assert_called_once_with(ctxt, FAKE_UUID1,
-                                         expected_attrs=None)
+                                         expected_attrs=None,
+                                         cell_down_support=False)
 
     @mock.patch.object(compute_api.API, 'get')
     @mock.patch.object(compute_api.API, 'attach_interface')
     def test_attach_interface_port_in_use(self,
                                           attach_mock,
                                           get_mock):
-        fake_instance = objects.Instance(uuid=FAKE_UUID1)
+        req_context = self.req.environ['nova.context']
+        fake_instance = objects.Instance(uuid=FAKE_UUID1,
+                                         project_id=req_context.project_id)
         get_mock.return_value = fake_instance
         attach_mock.side_effect = exception.PortInUse(
             port_id=FAKE_PORT_ID1)
@@ -366,16 +398,19 @@ class InterfaceAttachTestsV21(test.NoDBTestCase):
                           body=body)
         ctxt = self.req.environ['nova.context']
         attach_mock.assert_called_once_with(ctxt, fake_instance, None,
-                                            None, None)
+                                            None, None, tag=None)
         get_mock.assert_called_once_with(ctxt, FAKE_UUID1,
-                                         expected_attrs=None)
+                                         expected_attrs=None,
+                                         cell_down_support=False)
 
     @mock.patch.object(compute_api.API, 'get')
     @mock.patch.object(compute_api.API, 'attach_interface')
     def test_attach_interface_port_not_usable(self,
                                               attach_mock,
                                               get_mock):
-        fake_instance = objects.Instance(uuid=FAKE_UUID1)
+        req_context = self.req.environ['nova.context']
+        fake_instance = objects.Instance(uuid=FAKE_UUID1,
+                                         project_id=req_context.project_id)
         get_mock.return_value = fake_instance
         attach_mock.side_effect = exception.PortNotUsable(
             port_id=FAKE_PORT_ID1,
@@ -386,15 +421,17 @@ class InterfaceAttachTestsV21(test.NoDBTestCase):
                           body=body)
         ctxt = self.req.environ['nova.context']
         attach_mock.assert_called_once_with(ctxt, fake_instance, None,
-                                            None, None)
+                                            None, None, tag=None)
         get_mock.assert_called_once_with(ctxt, FAKE_UUID1,
-                                         expected_attrs=None)
+                                         expected_attrs=None,
+                                         cell_down_support=False)
 
     @mock.patch.object(compute_api.API, 'get')
     @mock.patch.object(compute_api.API, 'attach_interface')
     def test_attach_interface_failed_no_network(self, attach_mock, get_mock):
+        req_context = self.req.environ['nova.context']
         fake_instance = objects.Instance(uuid=FAKE_UUID1,
-                                         project_id=FAKE_UUID2)
+                                         project_id=req_context.project_id)
         get_mock.return_value = fake_instance
         attach_mock.side_effect = (
             exception.InterfaceAttachFailedNoNetwork(project_id=FAKE_UUID2))
@@ -402,16 +439,19 @@ class InterfaceAttachTestsV21(test.NoDBTestCase):
                           self.req, FAKE_UUID1, body={})
         ctxt = self.req.environ['nova.context']
         attach_mock.assert_called_once_with(ctxt, fake_instance, None,
-                                            None, None)
+                                            None, None, tag=None)
         get_mock.assert_called_once_with(ctxt, FAKE_UUID1,
-                                         expected_attrs=None)
+                                         expected_attrs=None,
+                                         cell_down_support=False)
 
     @mock.patch.object(compute_api.API, 'get')
     @mock.patch.object(compute_api.API, 'attach_interface')
     def test_attach_interface_no_more_fixed_ips(self,
                                           attach_mock,
                                           get_mock):
-        fake_instance = objects.Instance(uuid=FAKE_UUID1)
+        req_context = self.req.environ['nova.context']
+        fake_instance = objects.Instance(uuid=FAKE_UUID1,
+                                         project_id=req_context.project_id)
         get_mock.return_value = fake_instance
         attach_mock.side_effect = exception.NoMoreFixedIps(
               net=FAKE_NET_ID1)
@@ -421,16 +461,18 @@ class InterfaceAttachTestsV21(test.NoDBTestCase):
                           body=body)
         ctxt = self.req.environ['nova.context']
         attach_mock.assert_called_once_with(ctxt, fake_instance, None,
-                                            None, None)
+                                            None, None, tag=None)
         get_mock.assert_called_once_with(ctxt, FAKE_UUID1,
-                                         expected_attrs=None)
+                                         expected_attrs=None,
+                                         cell_down_support=False)
 
     @mock.patch.object(compute_api.API, 'get')
     @mock.patch.object(compute_api.API, 'attach_interface')
     def test_attach_interface_failed_securitygroup_cannot_be_applied(
         self, attach_mock, get_mock):
+        req_context = self.req.environ['nova.context']
         fake_instance = objects.Instance(uuid=FAKE_UUID1,
-                                         project_id=FAKE_UUID2)
+                                         project_id=req_context.project_id)
         get_mock.return_value = fake_instance
         attach_mock.side_effect = (
             exception.SecurityGroupCannotBeApplied())
@@ -438,13 +480,14 @@ class InterfaceAttachTestsV21(test.NoDBTestCase):
                           self.req, FAKE_UUID1, body={})
         ctxt = self.req.environ['nova.context']
         attach_mock.assert_called_once_with(ctxt, fake_instance, None,
-                                            None, None)
+                                            None, None, tag=None)
         get_mock.assert_called_once_with(ctxt, FAKE_UUID1,
-                                         expected_attrs=None)
+                                         expected_attrs=None,
+                                         cell_down_support=False)
 
     def _test_attach_interface_with_invalid_parameter(self, param):
-        self.stubs.Set(compute_api.API, 'attach_interface',
-                       fake_attach_interface)
+        self.stub_out('nova.compute.api.API.attach_interface',
+                      fake_attach_interface)
         body = {'interface_attachment': param}
         self.assertRaises(exception.ValidationError,
                           self.attachments.create, self.req, FAKE_UUID1,
@@ -463,66 +506,79 @@ class InterfaceAttachTestsV21(test.NoDBTestCase):
         self._test_attach_interface_with_invalid_parameter(param)
 
 
-class AttachInterfacesPolicyEnforcementv21(test.NoDBTestCase):
+class InterfaceAttachTestsV249(test.NoDBTestCase):
+    controller_cls = attach_interfaces_v21.InterfaceAttachmentController
 
     def setUp(self):
-        super(AttachInterfacesPolicyEnforcementv21, self).setUp()
-        self.controller = \
-            attach_interfaces_v21.InterfaceAttachmentController()
-        self.req = fakes.HTTPRequest.blank('')
-        self.rule_name = "os_compute_api:os-attach-interfaces"
-        self.policy.set_rules({self.rule_name: "project:non_fake"})
+        super(InterfaceAttachTestsV249, self).setUp()
+        self.attachments = self.controller_cls()
+        self.req = fakes.HTTPRequest.blank('', version='2.49')
 
-    def test_index_attach_interfaces_policy_failed(self):
-        exc = self.assertRaises(
-            exception.PolicyNotAuthorized,
-            self.controller.index, self.req, fakes.FAKE_UUID)
-        self.assertEqual(
-            "Policy doesn't allow %s to be performed." % self.rule_name,
-            exc.format_message())
+    def test_tagged_interface_attach_invalid_tag_comma(self):
+        body = {'interfaceAttachment': {'net_id': FAKE_NET_ID2,
+                                        'tag': ','}}
+        self.assertRaises(exception.ValidationError, self.attachments.create,
+                          self.req, FAKE_UUID1, body=body)
 
-    def test_show_attach_interfaces_policy_failed(self):
-        exc = self.assertRaises(
-            exception.PolicyNotAuthorized,
-            self.controller.show, self.req, fakes.FAKE_UUID, FAKE_PORT_ID1)
-        self.assertEqual(
-            "Policy doesn't allow %s to be performed." % self.rule_name,
-            exc.format_message())
+    def test_tagged_interface_attach_invalid_tag_slash(self):
+        body = {'interfaceAttachment': {'net_id': FAKE_NET_ID2,
+                                        'tag': '/'}}
+        self.assertRaises(exception.ValidationError, self.attachments.create,
+                          self.req, FAKE_UUID1, body=body)
 
-    def test_create_attach_interfaces_policy_failed(self):
-        exc = self.assertRaises(
-            exception.PolicyNotAuthorized,
-            self.controller.create, self.req, fakes.FAKE_UUID, body={})
-        self.assertEqual(
-            "Policy doesn't allow %s to be performed." % self.rule_name,
-            exc.format_message())
+    def test_tagged_interface_attach_invalid_tag_too_long(self):
+        tag = ''.join(map(str, range(10, 41)))
+        body = {'interfaceAttachment': {'net_id': FAKE_NET_ID2,
+                                        'tag': tag}}
+        self.assertRaises(exception.ValidationError, self.attachments.create,
+                          self.req, FAKE_UUID1, body=body)
 
-    def test_delete_attach_interfaces_policy_failed(self):
-        exc = self.assertRaises(
-            exception.PolicyNotAuthorized,
-            self.controller.delete, self.req, fakes.FAKE_UUID, FAKE_PORT_ID1)
-        self.assertEqual(
-            "Policy doesn't allow %s to be performed." % self.rule_name,
-            exc.format_message())
+    @mock.patch('nova.compute.api.API.attach_interface')
+    @mock.patch('nova.compute.api.API.get', fake_get_instance)
+    def test_tagged_interface_attach_valid_tag(self, _):
+        body = {'interfaceAttachment': {'net_id': FAKE_NET_ID2,
+                                        'tag': 'foo'}}
+        with mock.patch.object(self.attachments, 'show'):
+            self.attachments.create(self.req, FAKE_UUID1, body=body)
 
-    def test_attach_interfaces_create_policy_failed(self):
-        self.policy.set_rules({self.rule_name: "@",
-                               'os_compute_api:os-attach-interfaces:create':
-                               "!"})
-        exc = self.assertRaises(
-            exception.PolicyNotAuthorized,
-            self.controller.create, self.req, fakes.FAKE_UUID, body={})
-        self.assertEqual(
-            "Policy doesn't allow os_compute_api:os-attach-interfaces:create "
-            "to be performed.", exc.format_message())
 
-    def test_attach_interfaces_delete_policy_failed(self):
-        self.policy.set_rules({self.rule_name: "@",
-                               'os_compute_api:os-attach-interfaces:delete':
-                               "!"})
-        exc = self.assertRaises(
-            exception.PolicyNotAuthorized,
-            self.controller.delete, self.req, fakes.FAKE_UUID, FAKE_PORT_ID1)
-        self.assertEqual(
-            "Policy doesn't allow os_compute_api:os-attach-interfaces:delete "
-            "to be performed.", exc.format_message())
+class InterfaceAttachTestsV270(test.NoDBTestCase):
+    """os-interface API tests for microversion 2.70"""
+    def setUp(self):
+        super(InterfaceAttachTestsV270, self).setUp()
+        self.attachments = (
+            attach_interfaces_v21.InterfaceAttachmentController())
+        self.req = fakes.HTTPRequest.blank('', version='2.70')
+        self.stub_out('nova.compute.api.API.get', fake_get_instance)
+
+    @mock.patch('nova.objects.VirtualInterface.get_by_uuid', return_value=None)
+    def test_show_interface_no_vif(self, mock_get_by_uuid):
+        """Tests GET /servers/{server_id}/os-interface/{id} where there is no
+        corresponding VirtualInterface database record for the attached port.
+        """
+        with mock.patch.object(self.attachments.network_api, 'show_port',
+                               fake_show_port):
+            attachment = self.attachments.show(
+                self.req, FAKE_UUID1, FAKE_PORT_ID1)['interfaceAttachment']
+        self.assertIn('tag', attachment)
+        self.assertIsNone(attachment['tag'])
+        ctxt = self.req.environ['nova.context']
+        mock_get_by_uuid.assert_called_once_with(ctxt, FAKE_PORT_ID1)
+
+    @mock.patch('nova.objects.VirtualInterfaceList.get_by_instance_uuid',
+                return_value=objects.VirtualInterfaceList())
+    def test_list_interfaces_no_vifs(self, mock_get_by_instance_uuid):
+        """Tests GET /servers/{server_id}/os-interface where there is no
+        corresponding VirtualInterface database record for the attached ports.
+        """
+        with mock.patch.object(self.attachments.network_api, 'list_ports',
+                               return_value={'ports': ports}) as list_ports:
+            attachments = self.attachments.index(
+                self.req, FAKE_UUID1)['interfaceAttachments']
+        for attachment in attachments:
+            self.assertIn('tag', attachment)
+            self.assertIsNone(attachment['tag'])
+        ctxt = self.req.environ['nova.context']
+        list_ports.assert_called_once_with(ctxt, device_id=FAKE_UUID1)
+        mock_get_by_instance_uuid.assert_called_once_with(
+            self.req.environ['nova.context'], FAKE_UUID1)
