@@ -20,74 +20,56 @@ from oslo_serialization import jsonutils
 
 from nova.api.openstack import compute
 from nova.compute import api as compute_api
-from nova.compute import flavors
+from nova import context as nova_context
+from nova import exception
 from nova import objects
 from nova import test
+from nova.tests import fixtures
 from nova.tests.unit.api.openstack import fakes
 from nova.tests.unit import fake_instance
-import nova.tests.unit.image.fake
 
 
 MANUAL_INSTANCE_UUID = fakes.FAKE_UUID
 AUTO_INSTANCE_UUID = fakes.FAKE_UUID.replace('a', 'b')
 
-stub_instance = fakes.stub_instance
-
 API_DISK_CONFIG = 'OS-DCF:diskConfig'
 
 
-def instance_addresses(context, instance_id):
-    return None
-
-
 class DiskConfigTestCaseV21(test.TestCase):
+    project_id = fakes.FAKE_PROJECT_ID
 
     def setUp(self):
         super(DiskConfigTestCaseV21, self).setUp()
+        fakes.stub_out_nw_api(self)
+        fakes.stub_out_secgroup_api(self)
         self._set_up_app()
         self._setup_fake_image_service()
-
-        fakes.stub_out_nw_api(self)
-
+        ctxt = nova_context.RequestContext(
+            # These values match what is used in fakes.HTTPRequest.blank.
+            user_id='fake_user', project_id=self.project_id)
         FAKE_INSTANCES = [
-            fakes.stub_instance(1,
-                                uuid=MANUAL_INSTANCE_UUID,
-                                auto_disk_config=False),
-            fakes.stub_instance(2,
-                                uuid=AUTO_INSTANCE_UUID,
-                                auto_disk_config=True)
+            fakes.stub_instance_obj(ctxt,
+                                    uuid=MANUAL_INSTANCE_UUID,
+                                    auto_disk_config=False),
+            fakes.stub_instance_obj(ctxt,
+                                    uuid=AUTO_INSTANCE_UUID,
+                                    auto_disk_config=True)
         ]
 
-        def fake_instance_get(context, id_):
+        def fake_instance_get(_self, context, server_id, *args, **kwargs):
             for instance in FAKE_INSTANCES:
-                if id_ == instance['id']:
+                if server_id == instance.uuid:
                     return instance
+            raise exception.InstanceNotFound(instance_id=server_id)
 
-        self.stub_out('nova.db.instance_get', fake_instance_get)
-
-        def fake_instance_get_by_uuid(context, uuid,
-                                      columns_to_join=None, use_slave=False):
-            for instance in FAKE_INSTANCES:
-                if uuid == instance['uuid']:
-                    return instance
-
-        self.stub_out('nova.db.instance_get_by_uuid',
-                      fake_instance_get_by_uuid)
-
-        def fake_instance_get_all(context, *args, **kwargs):
-            return FAKE_INSTANCES
-
-        self.stub_out('nova.db.instance_get_all', fake_instance_get_all)
-        self.stub_out('nova.db.instance_get_all_by_filters',
-                      fake_instance_get_all)
-
-        self.stubs.Set(objects.Instance, 'save',
-                       lambda *args, **kwargs: None)
+        self.stub_out('nova.compute.api.API.get', fake_instance_get)
+        self.stub_out('nova.objects.Instance.save',
+                      lambda *args, **kwargs: None)
 
         def fake_rebuild(*args, **kwargs):
             pass
 
-        self.stubs.Set(compute_api.API, 'rebuild', fake_rebuild)
+        self.stub_out('nova.compute.api.API.rebuild', fake_rebuild)
 
         def fake_instance_create(context, inst_, session=None):
             inst = fake_instance.fake_db_instance(**{
@@ -101,44 +83,24 @@ class DiskConfigTestCaseV21(test.TestCase):
                     'vm_state': '',
                     'auto_disk_config': inst_['auto_disk_config'],
                     'security_groups': inst_['security_groups'],
-                    'instance_type': flavors.get_default_flavor(),
+                    'instance_type': objects.Flavor.get_by_name(context,
+                                                                'm1.small'),
                     })
 
-            def fake_instance_get_for_create(context, id_, *args, **kwargs):
-                return (inst, inst)
-
-            self.stub_out('nova.db.instance_update_and_get_original',
-                          fake_instance_get_for_create)
-
-            def fake_instance_get_all_for_create(context, *args, **kwargs):
-                return [inst]
-            self.stub_out('nova.db.instance_get_all',
-                           fake_instance_get_all_for_create)
-            self.stub_out('nova.db.instance_get_all_by_filters',
-                           fake_instance_get_all_for_create)
-
-            def fake_instance_add_security_group(context, instance_id,
-                                                 security_group_id):
-                pass
-
-            self.stub_out('nova.db.instance_add_security_group',
-                          fake_instance_add_security_group)
-
             return inst
 
-        self.stub_out('nova.db.instance_create', fake_instance_create)
+        self.stub_out('nova.db.api.instance_create', fake_instance_create)
 
     def _set_up_app(self):
         self.app = compute.APIRouterV21()
 
     def _get_expected_msg_for_invalid_disk_config(self):
         return ('{{"badRequest": {{"message": "Invalid input for'
-                ' field/attribute {0}. Value: {1}. u\'{1}\' is'
+                ' field/attribute {0}. Value: {1}. \'{1}\' is'
                 ' not one of [\'AUTO\', \'MANUAL\']", "code": 400}}}}')
 
     def _setup_fake_image_service(self):
-        self.image_service = nova.tests.unit.image.fake.stub_out_image_service(
-                self)
+        self.image_service = self.useFixture(fixtures.GlanceFixture(self))
         timestamp = datetime.datetime(2011, 1, 1, 1, 2, 3)
         image = {'id': '88580842-f50a-11e2-8d3a-f23c91aec05e',
                  'name': 'fakeimage7',
@@ -154,29 +116,25 @@ class DiskConfigTestCaseV21(test.TestCase):
                  'properties': {'auto_disk_config': 'Disabled'}}
         self.image_service.create(None, image)
 
-    def tearDown(self):
-        super(DiskConfigTestCaseV21, self).tearDown()
-        nova.tests.unit.image.fake.FakeImageService_reset()
-
     def assertDiskConfig(self, dict_, value):
         self.assertIn(API_DISK_CONFIG, dict_)
         self.assertEqual(dict_[API_DISK_CONFIG], value)
 
     def test_show_server(self):
         req = fakes.HTTPRequest.blank(
-            '/fake/servers/%s' % MANUAL_INSTANCE_UUID)
+            '/%s/servers/%s' % (self.project_id, MANUAL_INSTANCE_UUID))
         res = req.get_response(self.app)
         server_dict = jsonutils.loads(res.body)['server']
         self.assertDiskConfig(server_dict, 'MANUAL')
 
         req = fakes.HTTPRequest.blank(
-            '/fake/servers/%s' % AUTO_INSTANCE_UUID)
+            '/%s/servers/%s' % (self.project_id, AUTO_INSTANCE_UUID))
         res = req.get_response(self.app)
         server_dict = jsonutils.loads(res.body)['server']
         self.assertDiskConfig(server_dict, 'AUTO')
 
     def test_detail_servers(self):
-        req = fakes.HTTPRequest.blank('/fake/servers/detail')
+        req = fakes.HTTPRequest.blank('/%s/servers/detail' % self.project_id)
         res = req.get_response(self.app)
         server_dicts = jsonutils.loads(res.body)['servers']
 
@@ -188,19 +146,21 @@ class DiskConfigTestCaseV21(test.TestCase):
         self.flags(group='glance', api_servers=['http://localhost:9292'])
 
         req = fakes.HTTPRequest.blank(
-            '/fake/images/a440c04b-79fa-479c-bed1-0b816eaec379')
+            '/%s/images/a440c04b-79fa-479c-bed1-0b816eaec379' %
+            self.project_id)
         res = req.get_response(self.app)
         image_dict = jsonutils.loads(res.body)['image']
         self.assertDiskConfig(image_dict, 'MANUAL')
 
         req = fakes.HTTPRequest.blank(
-            '/fake/images/70a599e0-31e7-49b7-b260-868f441e862b')
+            '/%s/images/70a599e0-31e7-49b7-b260-868f441e862b' %
+            self.project_id)
         res = req.get_response(self.app)
         image_dict = jsonutils.loads(res.body)['image']
         self.assertDiskConfig(image_dict, 'AUTO')
 
     def test_detail_image(self):
-        req = fakes.HTTPRequest.blank('/fake/images/detail')
+        req = fakes.HTTPRequest.blank('/%s/images/detail' % self.project_id)
         res = req.get_response(self.app)
         image_dicts = jsonutils.loads(res.body)['images']
 
@@ -212,7 +172,7 @@ class DiskConfigTestCaseV21(test.TestCase):
                 self.assertDiskConfig(image_dict, expected)
 
     def test_create_server_override_auto(self):
-        req = fakes.HTTPRequest.blank('/fake/servers')
+        req = fakes.HTTPRequest.blank('/%s/servers' % self.project_id)
         req.method = 'POST'
         req.content_type = 'application/json'
         body = {'server': {
@@ -228,7 +188,7 @@ class DiskConfigTestCaseV21(test.TestCase):
         self.assertDiskConfig(server_dict, 'AUTO')
 
     def test_create_server_override_manual(self):
-        req = fakes.HTTPRequest.blank('/fake/servers')
+        req = fakes.HTTPRequest.blank('/%s/servers' % self.project_id)
         req.method = 'POST'
         req.content_type = 'application/json'
         body = {'server': {
@@ -247,7 +207,7 @@ class DiskConfigTestCaseV21(test.TestCase):
         """If user doesn't pass in diskConfig for server, use image metadata
         to specify AUTO or MANUAL.
         """
-        req = fakes.HTTPRequest.blank('/fake/servers')
+        req = fakes.HTTPRequest.blank('/%s/servers' % self.project_id)
         req.method = 'POST'
         req.content_type = 'application/json'
         body = {'server': {
@@ -261,7 +221,7 @@ class DiskConfigTestCaseV21(test.TestCase):
         server_dict = jsonutils.loads(res.body)['server']
         self.assertDiskConfig(server_dict, 'MANUAL')
 
-        req = fakes.HTTPRequest.blank('/fake/servers')
+        req = fakes.HTTPRequest.blank('/%s/servers' % self.project_id)
         req.method = 'POST'
         req.content_type = 'application/json'
         body = {'server': {
@@ -276,7 +236,7 @@ class DiskConfigTestCaseV21(test.TestCase):
         self.assertDiskConfig(server_dict, 'AUTO')
 
     def test_create_server_detect_from_image_disabled_goes_to_manual(self):
-        req = fakes.HTTPRequest.blank('/fake/servers')
+        req = fakes.HTTPRequest.blank('/%s/servers' % self.project_id)
         req.method = 'POST'
         req.content_type = 'application/json'
         body = {'server': {
@@ -291,7 +251,7 @@ class DiskConfigTestCaseV21(test.TestCase):
         self.assertDiskConfig(server_dict, 'MANUAL')
 
     def test_create_server_errors_when_disabled_and_auto(self):
-        req = fakes.HTTPRequest.blank('/fake/servers')
+        req = fakes.HTTPRequest.blank('/%s/servers' % self.project_id)
         req.method = 'POST'
         req.content_type = 'application/json'
         body = {'server': {
@@ -306,7 +266,7 @@ class DiskConfigTestCaseV21(test.TestCase):
         self.assertEqual(res.status_int, 400)
 
     def test_create_server_when_disabled_and_manual(self):
-        req = fakes.HTTPRequest.blank('/fake/servers')
+        req = fakes.HTTPRequest.blank('/%s/servers' % self.project_id)
         req.method = 'POST'
         req.content_type = 'application/json'
         body = {'server': {
@@ -325,7 +285,7 @@ class DiskConfigTestCaseV21(test.TestCase):
     def _test_update_server_disk_config(self, uuid, disk_config,
                                         get_instance_mock):
         req = fakes.HTTPRequest.blank(
-            '/fake/servers/%s' % uuid)
+            '/%s/servers/%s' % (self.project_id, uuid))
         req.method = 'PUT'
         req.content_type = 'application/json'
         body = {'server': {API_DISK_CONFIG: disk_config}}
@@ -350,7 +310,7 @@ class DiskConfigTestCaseV21(test.TestCase):
     def test_update_server_invalid_disk_config(self):
         # Return BadRequest if user passes an invalid diskConfig value.
         req = fakes.HTTPRequest.blank(
-            '/fake/servers/%s' % MANUAL_INSTANCE_UUID)
+            '/%s/servers/%s' % (self.project_id, MANUAL_INSTANCE_UUID))
         req.method = 'PUT'
         req.content_type = 'application/json'
         body = {'server': {API_DISK_CONFIG: 'server_test'}}
@@ -360,14 +320,14 @@ class DiskConfigTestCaseV21(test.TestCase):
         expected_msg = self._get_expected_msg_for_invalid_disk_config()
         expected_msg = expected_msg.format(API_DISK_CONFIG, 'server_test')
 
-        self.assertEqual(jsonutils.loads(expected_msg),
-                         jsonutils.loads(res.body))
+        self.assertJsonEqual(jsonutils.loads(expected_msg),
+                             jsonutils.loads(res.body))
 
     @mock.patch('nova.api.openstack.common.get_instance')
     def _test_rebuild_server_disk_config(self, uuid, disk_config,
                                          get_instance_mock):
         req = fakes.HTTPRequest.blank(
-            '/fake/servers/%s/action' % uuid)
+            '/%s/servers/%s/action' % (self.project_id, uuid))
         req.method = 'POST'
         req.content_type = 'application/json'
         auto_disk_config = (disk_config == 'AUTO')
@@ -393,7 +353,7 @@ class DiskConfigTestCaseV21(test.TestCase):
         self._test_rebuild_server_disk_config(MANUAL_INSTANCE_UUID, 'MANUAL')
 
     def test_create_server_with_auto_disk_config(self):
-        req = fakes.HTTPRequest.blank('/fake/servers')
+        req = fakes.HTTPRequest.blank('/%s/servers' % self.project_id)
         req.method = 'POST'
         req.content_type = 'application/json'
         body = {'server': {
@@ -409,7 +369,7 @@ class DiskConfigTestCaseV21(test.TestCase):
             self.assertTrue(kwargs['auto_disk_config'])
             return old_create(*args, **kwargs)
 
-        self.stubs.Set(compute_api.API, 'create', create)
+        self.stub_out('nova.compute.api.API.create', create)
 
         req.body = jsonutils.dump_as_bytes(body)
         res = req.get_response(self.app)
@@ -419,7 +379,7 @@ class DiskConfigTestCaseV21(test.TestCase):
     @mock.patch('nova.api.openstack.common.get_instance')
     def test_rebuild_server_with_auto_disk_config(self, get_instance_mock):
         req = fakes.HTTPRequest.blank(
-            '/fake/servers/%s/action' % AUTO_INSTANCE_UUID)
+            '/%s/servers/%s/action' % (self.project_id, AUTO_INSTANCE_UUID))
         req.method = 'POST'
         req.content_type = 'application/json'
         instance = fakes.stub_instance_obj(
@@ -437,7 +397,7 @@ class DiskConfigTestCaseV21(test.TestCase):
             self.assertIn('auto_disk_config', kwargs)
             self.assertTrue(kwargs['auto_disk_config'])
 
-        self.stubs.Set(compute_api.API, 'rebuild', rebuild)
+        self.stub_out('nova.compute.api.API.rebuild', rebuild)
 
         req.body = jsonutils.dump_as_bytes(body)
         res = req.get_response(self.app)
@@ -446,7 +406,7 @@ class DiskConfigTestCaseV21(test.TestCase):
 
     def test_resize_server_with_auto_disk_config(self):
         req = fakes.HTTPRequest.blank(
-            '/fake/servers/%s/action' % AUTO_INSTANCE_UUID)
+            '/%s/servers/%s/action' % (self.project_id, AUTO_INSTANCE_UUID))
         req.method = 'POST'
         req.content_type = 'application/json'
         body = {"resize": {
@@ -458,7 +418,7 @@ class DiskConfigTestCaseV21(test.TestCase):
             self.assertIn('auto_disk_config', kwargs)
             self.assertTrue(kwargs['auto_disk_config'])
 
-        self.stubs.Set(compute_api.API, 'resize', resize)
+        self.stub_out('nova.compute.api.API.resize', resize)
 
         req.body = jsonutils.dump_as_bytes(body)
         req.get_response(self.app)
