@@ -12,9 +12,10 @@
 #   License for the specific language governing permissions and limitations
 #   under the License.
 
-import uuid
-
+import fixtures
 import mock
+from oslo_utils.fixture import uuidsentinel as uuids
+import six
 import testtools
 import webob
 
@@ -48,6 +49,8 @@ def fake_compute_api_get(self, context, instance_id, **kwargs):
 def fake_service_get_by_compute_host(self, context, host):
     if host == 'bad-host':
         raise exception.ComputeHostNotFound(host=host)
+    elif host == 'unmapped-host':
+        raise exception.HostMappingNotFound(name=host)
     else:
         return {
             'host_name': host,
@@ -62,12 +65,16 @@ class EvacuateTestV21(test.NoDBTestCase):
 
     def setUp(self):
         super(EvacuateTestV21, self).setUp()
-        self.stubs.Set(compute_api.API, 'get', fake_compute_api_get)
-        self.stubs.Set(compute_api.HostAPI, 'service_get_by_compute_host',
-                       fake_service_get_by_compute_host)
-        self.UUID = uuid.uuid4()
+        self.stub_out('nova.compute.api.API.get', fake_compute_api_get)
+        self.stub_out('nova.compute.api.HostAPI.service_get_by_compute_host',
+                      fake_service_get_by_compute_host)
+        self.mock_list_port = self.useFixture(
+            fixtures.MockPatch('nova.network.neutron.API.list_ports')).mock
+        self.mock_list_port.return_value = {'ports': []}
+        self.UUID = uuids.fake
         for _method in self._methods:
-            self.stubs.Set(compute_api.API, _method, fake_compute_api)
+            self.stub_out('nova.compute.api.API.%s' % _method,
+                          fake_compute_api)
         self._set_up_controller()
         self.admin_req = fakes.HTTPRequest.blank('', use_admin_context=True)
         self.req = fakes.HTTPRequest.blank('')
@@ -87,9 +94,8 @@ class EvacuateTestV21(test.NoDBTestCase):
                                 controller=None):
         controller = controller or self.controller
         body = {'evacuate': body}
-        self.assertRaises(exception,
-                          controller._evacuate,
-                          self.admin_req, uuid or self.UUID, body=body)
+        return self.assertRaises(exception, controller._evacuate,
+                                 self.admin_req, uuid or self.UUID, body=body)
 
     def test_evacuate_with_valid_instance(self):
         admin_pass = 'MyNewPass'
@@ -106,11 +112,19 @@ class EvacuateTestV21(test.NoDBTestCase):
                                       'adminPass': 'MyNewPass'},
                                      uuid='BAD_UUID')
 
+    @mock.patch('nova.compute.api.API.evacuate')
+    def test_evacuate__with_vtpm(self, mock_evacuate):
+        mock_evacuate.side_effect = exception.OperationNotSupportedForVTPM(
+            instance_uuid=uuids.instance, operation='foo')
+        self._check_evacuate_failure(
+            webob.exc.HTTPConflict,
+            {'host': 'foo', 'onSharedStorage': 'False', 'adminPass': 'bar'})
+
     def test_evacuate_with_active_service(self):
         def fake_evacuate(*args, **kwargs):
             raise exception.ComputeServiceInUse("Service still in use")
 
-        self.stubs.Set(compute_api.API, 'evacuate', fake_evacuate)
+        self.stub_out('nova.compute.api.API.evacuate', fake_evacuate)
         self._check_evacuate_failure(webob.exc.HTTPBadRequest,
                                      {'host': 'my-host',
                                       'onSharedStorage': 'False',
@@ -154,6 +168,12 @@ class EvacuateTestV21(test.NoDBTestCase):
                                       'onSharedStorage': 'False',
                                       'adminPass': 'MyNewPass'})
 
+    def test_evacuate_instance_with_unmapped_target(self):
+        self._check_evacuate_failure(webob.exc.HTTPNotFound,
+                                     {'host': 'unmapped-host',
+                                      'onSharedStorage': 'False',
+                                      'adminPass': 'MyNewPass'})
+
     def test_evacuate_instance_with_target(self):
         admin_pass = 'MyNewPass'
         res = self._get_evacuate_response({'host': 'my-host',
@@ -180,13 +200,6 @@ class EvacuateTestV21(test.NoDBTestCase):
         self._get_evacuate_response({'host': 'my-host',
                                      'onSharedStorage': 'True'})
 
-    def test_not_admin(self):
-        body = {'evacuate': {'host': 'my-host',
-                             'onSharedStorage': 'False'}}
-        self.assertRaises(exception.PolicyNotAuthorized,
-                          self.controller._evacuate,
-                          self.req, self.UUID, body=body)
-
     def test_evacuate_to_same_host(self):
         self._check_evacuate_failure(webob.exc.HTTPBadRequest,
                                      {'host': 'host1',
@@ -221,7 +234,7 @@ class EvacuateTestV21(test.NoDBTestCase):
     @mock.patch('nova.objects.Instance.save')
     def _test_evacuate_enable_instance_password_conf(self, mock_save,
                                                      enable_pass):
-        self.flags(enable_instance_password=enable_pass)
+        self.flags(enable_instance_password=enable_pass, group='api')
 
         res = self._get_evacuate_response({'host': 'underscore_hostname',
                                            'onSharedStorage': 'False'})
@@ -231,89 +244,6 @@ class EvacuateTestV21(test.NoDBTestCase):
             self.assertIsNone(res)
 
 
-class EvacuatePolicyEnforcementv21(test.NoDBTestCase):
-
-    def setUp(self):
-        super(EvacuatePolicyEnforcementv21, self).setUp()
-        self.controller = evacuate_v21.EvacuateController()
-        self.req = fakes.HTTPRequest.blank('')
-        req_context = self.req.environ['nova.context']
-        self.stub_out('nova.compute.api.HostAPI.service_get_by_compute_host',
-                       fake_service_get_by_compute_host)
-
-        def fake_get_instance(self, context, id):
-            return fake_instance.fake_instance_obj(
-                req_context,
-                project_id=req_context.project_id,
-                user_id=req_context.user_id)
-
-        self.stub_out(
-            'nova.api.openstack.common.get_instance', fake_get_instance)
-
-    def test_evacuate_policy_failed_with_other_project(self):
-        rule_name = "os_compute_api:os-evacuate"
-        self.policy.set_rules({rule_name: "project_id:%(project_id)s"})
-        req = fakes.HTTPRequest.blank('')
-        # Change the project_id in request context.
-        req.environ['nova.context'].project_id = 'other-project'
-        body = {'evacuate': {'host': 'my-host',
-                             'onSharedStorage': 'False',
-                             'adminPass': 'MyNewPass'
-                             }}
-        exc = self.assertRaises(
-            exception.PolicyNotAuthorized,
-            self.controller._evacuate, req, fakes.FAKE_UUID,
-            body=body)
-        self.assertEqual(
-            "Policy doesn't allow %s to be performed." % rule_name,
-            exc.format_message())
-
-    @mock.patch('nova.compute.api.API.evacuate')
-    def test_evacuate_overridden_policy_pass_with_same_project(self,
-                                                               evacuate_mock):
-        rule_name = "os_compute_api:os-evacuate"
-        self.policy.set_rules({rule_name: "project_id:%(project_id)s"})
-        body = {'evacuate': {'host': 'my-host',
-                             'onSharedStorage': 'False',
-                             'adminPass': 'MyNewPass'
-                             }}
-        self.controller._evacuate(self.req, fakes.FAKE_UUID, body=body)
-        evacuate_mock.assert_called_once_with(self.req.environ['nova.context'],
-                                              mock.ANY, 'my-host', False,
-                                              'MyNewPass', None)
-
-    def test_evacuate_overridden_policy_failed_with_other_user(self):
-        rule_name = "os_compute_api:os-evacuate"
-        self.policy.set_rules({rule_name: "user_id:%(user_id)s"})
-        req = fakes.HTTPRequest.blank('')
-        # Change the user_id in request context.
-        req.environ['nova.context'].user_id = 'other-user'
-        body = {'evacuate': {'host': 'my-host',
-                             'onSharedStorage': 'False',
-                             'adminPass': 'MyNewPass'
-                             }}
-        exc = self.assertRaises(exception.PolicyNotAuthorized,
-                                self.controller._evacuate, req,
-                                fakes.FAKE_UUID, body=body)
-        self.assertEqual(
-                      "Policy doesn't allow %s to be performed." % rule_name,
-                      exc.format_message())
-
-    @mock.patch('nova.compute.api.API.evacuate')
-    def test_evacuate_overridden_policy_pass_with_same_user(self,
-                                                        evacuate_mock):
-        rule_name = "os_compute_api:os-evacuate"
-        self.policy.set_rules({rule_name: "user_id:%(user_id)s"})
-        body = {'evacuate': {'host': 'my-host',
-                             'onSharedStorage': 'False',
-                             'adminPass': 'MyNewPass'
-                             }}
-        self.controller._evacuate(self.req, fakes.FAKE_UUID, body=body)
-        evacuate_mock.assert_called_once_with(self.req.environ['nova.context'],
-                                              mock.ANY, 'my-host', False,
-                                              'MyNewPass', None)
-
-
 class EvacuateTestV214(EvacuateTestV21):
     def setUp(self):
         super(EvacuateTestV214, self).setUp()
@@ -334,9 +264,8 @@ class EvacuateTestV214(EvacuateTestV21):
         controller = controller or self.controller
         body.pop('onSharedStorage', None)
         body = {'evacuate': body}
-        self.assertRaises(exception,
-                          controller._evacuate,
-                          self.admin_req, uuid or self.UUID, body=body)
+        return self.assertRaises(exception, controller._evacuate,
+                                 self.admin_req, uuid or self.UUID, body=body)
 
     @mock.patch.object(compute_api.API, 'evacuate')
     def test_evacuate_instance(self, mock_evacuate):
@@ -388,12 +317,6 @@ class EvacuateTestV214(EvacuateTestV21):
             self.assertEqual(admin_pass,
                              mock_evacuate.call_args_list[0][0][4])
 
-    def test_not_admin(self):
-        body = {'evacuate': {'host': 'my-host'}}
-        self.assertRaises(exception.PolicyNotAuthorized,
-                          self.controller._evacuate,
-                          self.req, self.UUID, body=body)
-
     @testtools.skip('onSharedStorage was removed from Microversion 2.14')
     @mock.patch('nova.objects.Instance.save')
     def test_evacuate_shared_and_pass(self, mock_save):
@@ -427,7 +350,7 @@ class EvacuateTestV229(EvacuateTestV214):
         force = mock_evacuate.call_args_list[0][0][5]
         self.assertEqual(CONF.password_length, len(admin_pass))
         self.assertIsNone(on_shared_storage)
-        self.assertEqual(False, force)
+        self.assertFalse(force)
 
     def test_evacuate_with_valid_instance(self):
         admin_pass = 'MyNewPass'
@@ -441,8 +364,34 @@ class EvacuateTestV229(EvacuateTestV214):
         self._get_evacuate_response({'host': 'my-host',
                                      'force': 'true'})
         force = mock_evacuate.call_args_list[0][0][5]
-        self.assertEqual(True, force)
+        self.assertTrue(force)
 
     def test_forced_evacuate_with_no_host_provided(self):
         self._check_evacuate_failure(webob.exc.HTTPBadRequest,
                                      {'force': 'true'})
+
+
+class EvacuateTestV268(EvacuateTestV229):
+    def setUp(self):
+        super(EvacuateTestV268, self).setUp()
+        self.admin_req = fakes.HTTPRequest.blank('', use_admin_context=True,
+                                                 version='2.68')
+        self.req = fakes.HTTPRequest.blank('', version='2.68')
+
+    def test_evacuate_with_valid_instance(self):
+        admin_pass = 'MyNewPass'
+        res = self._get_evacuate_response({'host': 'my-host',
+                                           'adminPass': admin_pass})
+
+        self.assertIsNone(res)
+
+    @mock.patch.object(compute_api.API, 'evacuate')
+    def test_evacuate_instance_with_forced_host(self, mock_evacuate):
+        ex = self._check_evacuate_failure(self.validation_error,
+                                          {'host': 'my-host',
+                                           'force': 'true'})
+        self.assertIn('force', six.text_type(ex))
+
+    def test_forced_evacuate_with_no_host_provided(self):
+        # not applicable for v2.68, which removed the 'force' parameter
+        pass
