@@ -33,22 +33,21 @@ For postgres on Ubuntu this can be done with the following commands::
 """
 
 import glob
-# NOTE(dhellmann): Use stdlib logging instead of oslo.log because we
-# need to call methods on the logger that are not exposed through the
-# adapter provided by oslo.log.
-import logging
 import os
 
 from migrate import UniqueConstraint
 from migrate.versioning import repository
 import mock
-from oslo_db.sqlalchemy import test_base
+from oslo_db.sqlalchemy import enginefacade
+from oslo_db.sqlalchemy import test_fixtures
 from oslo_db.sqlalchemy import test_migrations
 from oslo_db.sqlalchemy import utils as oslodbutils
+from oslotest import timeout
 import sqlalchemy
 from sqlalchemy.engine import reflection
 import sqlalchemy.exc
 from sqlalchemy.sql import null
+import testtools
 
 from nova.db import migration
 from nova.db.sqlalchemy import migrate_repo
@@ -59,6 +58,13 @@ from nova import exception
 from nova import test
 from nova.tests import fixtures as nova_fixtures
 
+# TODO(sdague): no tests in the nova/tests tree should inherit from
+# base test classes in another library. This causes all kinds of havoc
+# in these doing things incorrectly for what we need in subunit
+# reporting. This is a long unwind, but should be done in the future
+# and any code needed out of oslo_db should be exported / accessed as
+# a fixture.
+
 
 class NovaMigrationsCheckers(test_migrations.ModelsMigrationsSync,
                              test_migrations.WalkVersionsMixin):
@@ -84,21 +90,17 @@ class NovaMigrationsCheckers(test_migrations.ModelsMigrationsSync,
         return self.engine
 
     def setUp(self):
-        super(NovaMigrationsCheckers, self).setUp()
-        # NOTE(viktors): We should reduce log output because it causes issues,
-        #                when we run tests with testr
-        migrate_log = logging.getLogger('migrate')
-        old_level = migrate_log.level
-        migrate_log.setLevel(logging.WARN)
-        self.addCleanup(migrate_log.setLevel, old_level)
+        # NOTE(sdague): the oslo_db base test case completely
+        # invalidates our logging setup, we actually have to do that
+        # before it is called to keep this from vomitting all over our
+        # test output.
+        self.useFixture(nova_fixtures.StandardLogging())
 
-        # NOTE(rpodolyaka): we need to repeat the functionality of the base
-        # test case a bit here as this gets overriden by oslotest base test
-        # case and nova base test case cleanup must be the last one (as it
-        # deletes attributes of test case instances)
-        self.useFixture(nova_fixtures.Timeout(
-            os.environ.get('OS_TEST_TIMEOUT', 0),
-            self.TIMEOUT_SCALING_FACTOR))
+        super(NovaMigrationsCheckers, self).setUp()
+        # The Timeout fixture picks up env.OS_TEST_TIMEOUT, defaulting to 0.
+        self.useFixture(timeout.Timeout(
+            scaling_factor=self.TIMEOUT_SCALING_FACTOR))
+        self.engine = enginefacade.writer.get_engine()
 
     def assertColumnExists(self, engine, table_name, column):
         self.assertTrue(oslodbutils.column_exists(engine, table_name, column),
@@ -166,12 +168,22 @@ class NovaMigrationsCheckers(test_migrations.ModelsMigrationsSync,
             272,  # NOOP migration due to revert
         ]
 
-        havana_placeholders = range(217, 227)
-        icehouse_placeholders = range(235, 244)
-        juno_placeholders = range(255, 265)
-        kilo_placeholders = range(281, 291)
-        liberty_placeholders = range(303, 313)
-        mitaka_placeholders = range(320, 330)
+        havana_placeholders = list(range(217, 227))
+        icehouse_placeholders = list(range(235, 244))
+        juno_placeholders = list(range(255, 265))
+        kilo_placeholders = list(range(281, 291))
+        liberty_placeholders = list(range(303, 313))
+        mitaka_placeholders = list(range(320, 330))
+        newton_placeholders = list(range(335, 345))
+        ocata_placeholders = list(range(348, 358))
+        pike_placeholders = list(range(363, 373))
+        queens_placeholders = list(range(379, 389))
+        # We forgot to add the rocky placeholder. We've also switched to 5
+        # placeholders per cycle since the rate of DB changes has dropped
+        # significantly
+        stein_placeholders = list(range(392, 397))
+        train_placeholders = list(range(403, 408))
+        ussuri_placeholders = list(range(408, 413))
 
         return (special +
                 havana_placeholders +
@@ -179,7 +191,14 @@ class NovaMigrationsCheckers(test_migrations.ModelsMigrationsSync,
                 juno_placeholders +
                 kilo_placeholders +
                 liberty_placeholders +
-                mitaka_placeholders)
+                mitaka_placeholders +
+                newton_placeholders +
+                ocata_placeholders +
+                pike_placeholders +
+                queens_placeholders +
+                stein_placeholders +
+                train_placeholders +
+                ussuri_placeholders)
 
     def migrate_up(self, version, with_data=False):
         if with_data:
@@ -210,6 +229,11 @@ class NovaMigrationsCheckers(test_migrations.ModelsMigrationsSync,
             # 292 drops completely orphaned tables with no users, so
             # it can be done without affecting anything.
             292,
+
+            # 346 Drops column scheduled_at from instances table since it
+            # is no longer used. The field value is always NULL so
+            # it does not affect anything.
+            346,
         ]
         # Reviewers: DO NOT ALLOW THINGS TO BE ADDED HERE
 
@@ -765,7 +789,10 @@ class NovaMigrationsCheckers(test_migrations.ModelsMigrationsSync,
         def removed_column(element):
             # Define a whitelist of columns that would be removed from the
             # DB at a later release.
-            column_whitelist = {'instances': ['scheduled_at']}
+            # NOTE(Luyao) The vpmems column was added to the schema in train,
+            # and removed from the model in train.
+            column_whitelist = {'instances': ['internal_id'],
+                                'instance_extra': ['vpmems']}
 
             if element[0] != 'remove_column':
                 return False
@@ -916,16 +943,131 @@ class NovaMigrationsCheckers(test_migrations.ModelsMigrationsSync,
         self.assertColumnExists(engine, 'shadow_instance_extra',
                                         'device_metadata')
 
+    def _check_345(self, engine, data):
+        # NOTE(danms): Just a sanity-check migration
+        pass
+
+    def _check_346(self, engine, data):
+        self.assertColumnNotExists(engine, 'instances', 'scheduled_at')
+        self.assertColumnNotExists(engine, 'shadow_instances', 'scheduled_at')
+
+    def _check_347(self, engine, data):
+        self.assertIndexMembers(engine, 'instances',
+                                'instances_project_id_idx',
+                                ['project_id'])
+        self.assertIndexMembers(engine, 'instances',
+                                'instances_updated_at_project_id_idx',
+                                ['updated_at', 'project_id'])
+
+    def _check_358(self, engine, data):
+        self.assertColumnExists(engine, 'block_device_mapping',
+                                'attachment_id')
+
+    def _check_359(self, engine, data):
+        self.assertColumnExists(engine, 'services', 'uuid')
+        self.assertIndexMembers(engine, 'services', 'services_uuid_idx',
+                                ['uuid'])
+
+    def _check_360(self, engine, data):
+        self.assertColumnExists(engine, 'compute_nodes', 'mapped')
+        self.assertColumnExists(engine, 'shadow_compute_nodes', 'mapped')
+
+    def _check_361(self, engine, data):
+        self.assertIndexMembers(engine, 'compute_nodes',
+                                'compute_nodes_uuid_idx', ['uuid'])
+
+    def _check_362(self, engine, data):
+        self.assertColumnExists(engine, 'pci_devices', 'uuid')
+
+    def _check_373(self, engine, data):
+        self.assertColumnExists(engine, 'migrations', 'uuid')
+
+    def _check_374(self, engine, data):
+        self.assertColumnExists(engine, 'block_device_mapping', 'uuid')
+        self.assertColumnExists(engine, 'shadow_block_device_mapping', 'uuid')
+
+        inspector = reflection.Inspector.from_engine(engine)
+        constraints = inspector.get_unique_constraints('block_device_mapping')
+        constraint_names = [constraint['name'] for constraint in constraints]
+        self.assertIn('uniq_block_device_mapping0uuid', constraint_names)
+
+    def _check_375(self, engine, data):
+        self.assertColumnExists(engine, 'console_auth_tokens',
+                                'access_url_base')
+
+    def _check_376(self, engine, data):
+        self.assertIndexMembers(
+            engine, 'console_auth_tokens',
+            'console_auth_tokens_token_hash_instance_uuid_idx',
+            ['token_hash', 'instance_uuid'])
+
+    def _check_377(self, engine, data):
+        self.assertIndexMembers(engine, 'migrations',
+                                'migrations_updated_at_idx', ['updated_at'])
+
+    def _check_378(self, engine, data):
+        self.assertIndexMembers(
+            engine, 'instance_actions',
+            'instance_actions_instance_uuid_updated_at_idx',
+            ['instance_uuid', 'updated_at'])
+
+    def _check_389(self, engine, data):
+        self.assertIndexMembers(engine, 'aggregate_metadata',
+                                'aggregate_metadata_value_idx',
+                                ['value'])
+
+    def _check_390(self, engine, data):
+        self.assertColumnExists(engine, 'instance_extra', 'trusted_certs')
+        self.assertColumnExists(engine, 'shadow_instance_extra',
+                                'trusted_certs')
+
+    def _check_391(self, engine, data):
+        self.assertColumnExists(engine, 'block_device_mapping', 'volume_type')
+        self.assertColumnExists(engine, 'shadow_block_device_mapping',
+                                'volume_type')
+
+    def _check_397(self, engine, data):
+        for prefix in ('', 'shadow_'):
+            self.assertColumnExists(
+                engine, '%smigrations' % prefix, 'cross_cell_move')
+
+    def _check_398(self, engine, data):
+        self.assertColumnExists(engine, 'instance_extra', 'vpmems')
+        self.assertColumnExists(engine, 'shadow_instance_extra', 'vpmems')
+
+    def _check_399(self, engine, data):
+        for prefix in ('', 'shadow_'):
+            self.assertColumnExists(
+                engine, '%sinstances' % prefix, 'hidden')
+
+    def _check_400(self, engine, data):
+        # NOTE(mriedem): This is a dummy migration that just does a consistency
+        # check. The actual test for 400 is in TestServicesUUIDCheck.
+        pass
+
+    def _check_401(self, engine, data):
+        for prefix in ('', 'shadow_'):
+            self.assertColumnExists(
+                engine, '%smigrations' % prefix, 'user_id')
+            self.assertColumnExists(
+                engine, '%smigrations' % prefix, 'project_id')
+
+    def _check_402(self, engine, data):
+        self.assertColumnExists(engine, 'instance_extra', 'resources')
+        self.assertColumnExists(engine, 'shadow_instance_extra', 'resources')
+
 
 class TestNovaMigrationsSQLite(NovaMigrationsCheckers,
-                               test_base.DbTestCase,
-                               test.NoDBTestCase):
+                               test_fixtures.OpportunisticDBTestMixin,
+                               testtools.TestCase):
     pass
 
 
 class TestNovaMigrationsMySQL(NovaMigrationsCheckers,
-                              test_base.MySQLOpportunisticTestCase,
-                              test.NoDBTestCase):
+                              test_fixtures.OpportunisticDBTestMixin,
+                              testtools.TestCase):
+    FIXTURE = test_fixtures.MySQLOpportunisticFixture
+
     def test_innodb_tables(self):
         with mock.patch.object(sa_migration, 'get_engine',
                                return_value=self.migrate_engine):
@@ -950,32 +1092,33 @@ class TestNovaMigrationsMySQL(NovaMigrationsCheckers,
 
 
 class TestNovaMigrationsPostgreSQL(NovaMigrationsCheckers,
-                                   test_base.PostgreSQLOpportunisticTestCase,
-                                   test.NoDBTestCase):
-    pass
+                                   test_fixtures.OpportunisticDBTestMixin,
+                                   testtools.TestCase):
+    FIXTURE = test_fixtures.PostgresqlOpportunisticFixture
 
 
 class ProjectTestCase(test.NoDBTestCase):
 
     def test_no_migrations_have_downgrade(self):
         topdir = os.path.normpath(os.path.dirname(__file__) + '/../../../')
-        py_glob = os.path.join(topdir, "nova", "db", "sqlalchemy",
-                               "migrate_repo", "versions", "*.py")
-
+        # Walk both the nova_api and nova (cell) database migrations.
         includes_downgrade = []
-        for path in glob.iglob(py_glob):
-            has_upgrade = False
-            has_downgrade = False
-            with open(path, "r") as f:
-                for line in f:
-                    if 'def upgrade(' in line:
-                        has_upgrade = True
-                    if 'def downgrade(' in line:
-                        has_downgrade = True
+        for subdir in ('api_migrations', ''):
+            py_glob = os.path.join(topdir, "db", "sqlalchemy", subdir,
+                                   "migrate_repo", "versions", "*.py")
+            for path in glob.iglob(py_glob):
+                has_upgrade = False
+                has_downgrade = False
+                with open(path, "r") as f:
+                    for line in f:
+                        if 'def upgrade(' in line:
+                            has_upgrade = True
+                        if 'def downgrade(' in line:
+                            has_downgrade = True
 
-                if has_upgrade and has_downgrade:
-                    fname = os.path.basename(path)
-                    includes_downgrade.append(fname)
+                    if has_upgrade and has_downgrade:
+                        fname = os.path.basename(path)
+                        includes_downgrade.append(fname)
 
         helpful_msg = ("The following migrations have a downgrade "
                        "which is not supported:"
