@@ -13,15 +13,18 @@
 #    under the License.
 
 import datetime
+import os
 
 import iso8601
+import mock
+from oslo_serialization import jsonutils
 from oslo_versionedobjects import exception as ovo_exc
-import six
 
+from nova import exception
 from nova.network import model as network_model
 from nova.objects import fields
-from nova import signature_utils
 from nova import test
+from nova.tests.unit import fake_instance
 from nova import utils
 
 
@@ -60,6 +63,14 @@ class FakeEnumAlt(fields.Enum):
                                           **kwargs)
 
 
+class FakeAddress(fields.AddressBase):
+    PATTERN = '[a-z]+[0-9]+'
+
+
+class FakeAddressField(fields.AutoTypedField):
+    AUTO_TYPE = FakeAddress()
+
+
 class FakeEnumField(fields.BaseEnumField):
     AUTO_TYPE = FakeEnum()
 
@@ -108,8 +119,6 @@ class TestString(TestField):
         super(TestString, self).setUp()
         self.field = fields.StringField()
         self.coerce_good_values = [('foo', 'foo'), (1, '1'), (True, 'True')]
-        if six.PY2:
-            self.coerce_good_values.append((int(1), '1'))
         self.coerce_bad_values = [None]
         self.to_primitive_values = self.coerce_good_values[0:1]
         self.from_primitive_values = self.coerce_good_values[0:1]
@@ -149,8 +158,6 @@ class TestEnum(TestField):
         self.field = fields.EnumField(
             valid_values=['foo', 'bar', 1, 1, True])
         self.coerce_good_values = [('foo', 'foo'), (1, '1'), (True, 'True')]
-        if six.PY2:
-            self.coerce_good_values.append((int(1), '1'))
         self.coerce_bad_values = ['boo', 2, False]
         self.to_primitive_values = self.coerce_good_values[0:1]
         self.from_primitive_values = self.coerce_good_values[0:1]
@@ -177,77 +184,147 @@ class TestEnum(TestField):
                           fields.EnumField, [])
 
 
-class TestImageSignatureTypes(TestField):
-    # Ensure that the object definition is updated
-    # in step with the signature_utils module
-    def setUp(self):
-        super(TestImageSignatureTypes, self).setUp()
-        self.hash_field = fields.ImageSignatureHashType()
-        self.key_type_field = fields.ImageSignatureKeyType()
+class TestArchitecture(TestField):
+    @mock.patch.object(os, 'uname')
+    def test_host(self, mock_uname):
+        mock_uname.return_value = (
+            'Linux',
+            'localhost.localdomain',
+            '3.14.8-200.fc20.x86_64',
+            '#1 SMP Mon Jun 16 21:57:53 UTC 2014',
+            'i686'
+        )
 
-    def test_hashes(self):
-        for hash_name in list(signature_utils.HASH_METHODS.keys()):
-            self.assertIn(hash_name, self.hash_field.ALL)
+        self.assertEqual(fields.Architecture.I686,
+                         fields.Architecture.from_host())
 
-    def test_key_types(self):
-        key_type_dict = signature_utils.SignatureKeyType._REGISTERED_TYPES
-        key_types = list(key_type_dict.keys())
-        for key_type in key_types:
-            self.assertIn(key_type, self.key_type_field.ALL)
+    def test_valid_string(self):
+        self.assertTrue(fields.Architecture.is_valid('x86_64'))
+
+    def test_valid_constant(self):
+        self.assertTrue(fields.Architecture.is_valid(
+            fields.Architecture.X86_64))
+
+    def test_valid_bogus(self):
+        self.assertFalse(fields.Architecture.is_valid('x86_64wibble'))
+
+    def test_canonicalize_i386(self):
+        self.assertEqual(fields.Architecture.I686,
+                         fields.Architecture.canonicalize('i386'))
+
+    def test_canonicalize_amd64(self):
+        self.assertEqual(fields.Architecture.X86_64,
+                         fields.Architecture.canonicalize('amd64'))
+
+    def test_canonicalize_case(self):
+        self.assertEqual(fields.Architecture.X86_64,
+                         fields.Architecture.canonicalize('X86_64'))
+
+    def test_canonicalize_compat_xen1(self):
+        self.assertEqual(fields.Architecture.I686,
+                         fields.Architecture.canonicalize('x86_32'))
+
+    def test_canonicalize_compat_xen2(self):
+        self.assertEqual(fields.Architecture.I686,
+                         fields.Architecture.canonicalize('x86_32p'))
+
+    def test_canonicalize_invalid(self):
+        self.assertRaises(exception.InvalidArchitectureName,
+                          fields.Architecture.canonicalize,
+                          'x86_64wibble')
 
 
-class TestResourceClass(TestField):
-    def setUp(self):
-        super(TestResourceClass, self).setUp()
-        self.field = fields.ResourceClassField()
-        self.coerce_good_values = [
-            ('VCPU', 'VCPU'),
-            ('MEMORY_MB', 'MEMORY_MB'),
-            ('DISK_GB', 'DISK_GB'),
-            ('PCI_DEVICE', 'PCI_DEVICE'),
-            ('SRIOV_NET_VF', 'SRIOV_NET_VF'),
-            ('NUMA_SOCKET', 'NUMA_SOCKET'),
-            ('NUMA_CORE', 'NUMA_CORE'),
-            ('NUMA_THREAD', 'NUMA_THREAD'),
-            ('NUMA_MEMORY_MB', 'NUMA_MEMORY_MB'),
-            ('IPV4_ADDRESS', 'IPV4_ADDRESS'),
-        ]
-        self.expected_indexes = [
-            ('VCPU', 0),
-            ('MEMORY_MB', 1),
-            ('DISK_GB', 2),
-            ('PCI_DEVICE', 3),
-            ('SRIOV_NET_VF', 4),
-            ('NUMA_SOCKET', 5),
-            ('NUMA_CORE', 6),
-            ('NUMA_THREAD', 7),
-            ('NUMA_MEMORY_MB', 8),
-            ('IPV4_ADDRESS', 9),
-        ]
-        self.coerce_bad_values = ['acme']
-        self.to_primitive_values = self.coerce_good_values[0:1]
-        self.from_primitive_values = self.coerce_good_values[0:1]
+class TestHVType(TestField):
+    def test_valid_string(self):
+        self.assertTrue(fields.HVType.is_valid('vmware'))
 
-    def test_stringify(self):
-        self.assertEqual("'VCPU'", self.field.stringify(
-            fields.ResourceClass.VCPU))
+    def test_valid_constant(self):
+        self.assertTrue(fields.HVType.is_valid(fields.HVType.QEMU))
 
-    def test_stringify_invalid(self):
-        self.assertRaises(ValueError, self.field.stringify, 'cow')
+    def test_valid_docker(self):
+        self.assertTrue(fields.HVType.is_valid('docker'))
 
-    def test_index(self):
-        for name, index in self.expected_indexes:
-            self.assertEqual(index, self.field.index(name))
+    def test_valid_lxd(self):
+        self.assertTrue(fields.HVType.is_valid('lxd'))
 
-    def test_index_invalid(self):
-        self.assertRaises(ValueError, self.field.index, 'cow')
+    def test_valid_vz(self):
+        self.assertTrue(fields.HVType.is_valid(
+            fields.HVType.VIRTUOZZO))
 
-    def test_from_index(self):
-        for name, index in self.expected_indexes:
-            self.assertEqual(name, self.field.from_index(index))
+    def test_valid_bogus(self):
+        self.assertFalse(fields.HVType.is_valid('acmehypervisor'))
 
-    def test_from_index_invalid(self):
-        self.assertRaises(IndexError, self.field.from_index, 999)
+    def test_canonicalize_none(self):
+        self.assertIsNone(fields.HVType.canonicalize(None))
+
+    def test_canonicalize_case(self):
+        self.assertEqual(fields.HVType.QEMU,
+                         fields.HVType.canonicalize('QeMu'))
+
+    def test_canonicalize_xapi(self):
+        self.assertEqual(fields.HVType.XEN,
+                         fields.HVType.canonicalize('xapi'))
+
+    def test_canonicalize_invalid(self):
+        self.assertRaises(exception.InvalidHypervisorVirtType,
+                          fields.HVType.canonicalize,
+                          'wibble')
+
+
+class TestVMMode(TestField):
+    def _fake_object(self, updates):
+        return fake_instance.fake_instance_obj(None, **updates)
+
+    def test_case(self):
+        inst = self._fake_object(dict(vm_mode='HVM'))
+        mode = fields.VMMode.get_from_instance(inst)
+        self.assertEqual(mode, 'hvm')
+
+    def test_legacy_pv(self):
+        inst = self._fake_object(dict(vm_mode='pv'))
+        mode = fields.VMMode.get_from_instance(inst)
+        self.assertEqual(mode, 'xen')
+
+    def test_legacy_hv(self):
+        inst = self._fake_object(dict(vm_mode='hv'))
+        mode = fields.VMMode.get_from_instance(inst)
+        self.assertEqual(mode, 'hvm')
+
+    def test_bogus(self):
+        inst = self._fake_object(dict(vm_mode='wibble'))
+        self.assertRaises(exception.Invalid,
+                          fields.VMMode.get_from_instance,
+                          inst)
+
+    def test_good(self):
+        inst = self._fake_object(dict(vm_mode='hvm'))
+        mode = fields.VMMode.get_from_instance(inst)
+        self.assertEqual(mode, 'hvm')
+
+    def test_canonicalize_pv_compat(self):
+        mode = fields.VMMode.canonicalize('pv')
+        self.assertEqual(fields.VMMode.XEN, mode)
+
+    def test_canonicalize_hv_compat(self):
+        mode = fields.VMMode.canonicalize('hv')
+        self.assertEqual(fields.VMMode.HVM, mode)
+
+    def test_canonicalize_baremetal_compat(self):
+        mode = fields.VMMode.canonicalize('baremetal')
+        self.assertEqual(fields.VMMode.HVM, mode)
+
+    def test_canonicalize_hvm(self):
+        mode = fields.VMMode.canonicalize('hvm')
+        self.assertEqual(fields.VMMode.HVM, mode)
+
+    def test_canonicalize_none(self):
+        mode = fields.VMMode.canonicalize(None)
+        self.assertIsNone(mode)
+
+    def test_canonicalize_invalid(self):
+        self.assertRaises(exception.InvalidVirtualMachineMode,
+                          fields.VMMode.canonicalize,
+                          'invalid')
 
 
 class TestInteger(TestField):
@@ -263,7 +340,7 @@ class TestInteger(TestField):
 class TestNonNegativeInteger(TestInteger):
     def setUp(self):
         super(TestNonNegativeInteger, self).setUp()
-        self.field = fields.Field(fields.NonNegativeInteger())
+        self.field = fields.NonNegativeIntegerField()
         self.coerce_bad_values.extend(['-2', '4.2'])
 
 
@@ -280,7 +357,7 @@ class TestFloat(TestField):
 class TestNonNegativeFloat(TestFloat):
     def setUp(self):
         super(TestNonNegativeFloat, self).setUp()
-        self.field = fields.Field(fields.NonNegativeFloat())
+        self.field = fields.NonNegativeFloatField()
         self.coerce_bad_values.extend(['-4.2'])
 
 
@@ -298,7 +375,7 @@ class TestBoolean(TestField):
 class TestDateTime(TestField):
     def setUp(self):
         super(TestDateTime, self).setUp()
-        self.dt = datetime.datetime(1955, 11, 5, tzinfo=iso8601.iso8601.Utc())
+        self.dt = datetime.datetime(1955, 11, 5, tzinfo=iso8601.UTC)
         self.field = fields.DateTimeField()
         self.coerce_good_values = [(self.dt, self.dt),
                                    (utils.isotime(self.dt), self.dt)]
@@ -311,7 +388,7 @@ class TestDateTime(TestField):
             '1955-11-05T18:00:00Z',
             self.field.stringify(
                 datetime.datetime(1955, 11, 5, 18, 0, 0,
-                                  tzinfo=iso8601.iso8601.Utc())))
+                                  tzinfo=iso8601.UTC)))
 
 
 class TestDict(TestField):
@@ -483,6 +560,18 @@ class TestNetworkModel(TestField):
                          self.field.stringify(networkinfo))
 
 
+class TestNetworkVIFModel(TestField):
+    def setUp(self):
+        super(TestNetworkVIFModel, self).setUp()
+        model = network_model.VIF('6c197bc7-820c-40d5-8aff-7116b993e793')
+        primitive = jsonutils.dumps(model)
+        self.field = fields.Field(fields.NetworkVIFModel())
+        self.coerce_good_values = [(model, model), (primitive, model)]
+        self.coerce_bad_values = [[], 'foo']
+        self.to_primitive_values = [(model, primitive)]
+        self.from_primitive_values = [(primitive, model)]
+
+
 class TestNotificationPriority(TestField):
     def setUp(self):
         super(TestNotificationPriority, self).setUp()
@@ -538,28 +627,6 @@ class TestNotificationAction(TestField):
         self.assertRaises(ValueError, self.field.stringify, 'magic')
 
 
-class TestPCIAddress(TestField):
-    def setUp(self):
-        super(TestPCIAddress, self).setUp()
-        self.field = fields.Field(fields.PCIAddressField())
-        self.coerce_good_values = [('0000:00:02.0', '0000:00:02.0')]
-        self.coerce_bad_values = [
-            '000:00:02.0',
-            '0000:0:02.0',
-            '0000:00:2.0',
-            '0000:00:02.',
-            '-000:00:02.0',
-            '0000:0-:02.0',
-            '0000:00:-2.0',
-            '0000:00:02.-',
-            '000000:02.0',
-            '0000:0:02.0',
-            '0000:00:020',
-        ]
-        self.to_primitive_values = self.coerce_good_values
-        self.from_primitive_values = self.coerce_good_values
-
-
 class TestUSBAddress(TestField):
     def setUp(self):
         super(TestUSBAddress, self).setUp()
@@ -605,6 +672,20 @@ class TestIDEAddress(TestField):
         self.from_primitive_values = self.coerce_good_values
 
 
+class TestXenAddress(TestField):
+    def setUp(self):
+        super(TestXenAddress, self).setUp()
+        self.field = fields.Field(fields.XenAddressField())
+        self.coerce_good_values = [('000100', '000100'),
+                                   ('768', '768')]
+        self.coerce_bad_values = [
+            '1',
+            '00100',
+        ]
+        self.to_primitive_values = self.coerce_good_values
+        self.from_primitive_values = self.coerce_good_values
+
+
 class TestSecureBoot(TestField):
     def setUp(self):
         super(TestSecureBoot, self).setUp()
@@ -621,3 +702,21 @@ class TestSecureBoot(TestField):
 
     def test_stringify_invalid(self):
         self.assertRaises(ValueError, self.field.stringify, 'enabled')
+
+
+class TestSchemaGeneration(test.NoDBTestCase):
+    def test_address_base_get_schema(self):
+        field = FakeAddressField()
+        expected = {'type': ['string'], 'pattern': '[a-z]+[0-9]+',
+                    'readonly': False}
+        self.assertEqual(expected, field.get_schema())
+
+
+class TestNotificationSource(test.NoDBTestCase):
+    def test_get_source_by_binary(self):
+        self.assertEqual('nova-api',
+                         fields.NotificationSource.get_source_by_binary(
+                             'nova-osapi_compute'))
+        self.assertEqual('nova-metadata',
+                         fields.NotificationSource.get_source_by_binary(
+                             'nova-metadata'))
