@@ -12,16 +12,15 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import collections
 import contextlib
 import copy
 import datetime
-import inspect
 import os
 import pprint
 
 import fixtures
 import mock
-from oslo_log import log
 from oslo_utils import timeutils
 from oslo_versionedobjects import base as ovo_base
 from oslo_versionedobjects import exception as ovo_exc
@@ -40,9 +39,6 @@ from nova.tests.unit import fake_notifier
 from nova import utils
 
 
-LOG = log.getLogger(__name__)
-
-
 class MyOwnedObject(base.NovaPersistentObject, base.NovaObject):
     VERSION = '1.0'
     fields = {'baz': fields.IntegerField()}
@@ -119,7 +115,7 @@ class RandomMixInWithNoFields(object):
 
 
 @base.NovaObjectRegistry.register_if(False)
-class TestSubclassedObject(RandomMixInWithNoFields, MyObj):
+class SubclassedObject(RandomMixInWithNoFields, MyObj):
     fields = {'new_field': fields.StringField()}
 
 
@@ -172,26 +168,6 @@ class TestObjToPrimitive(test.NoDBTestCase):
                          base.obj_to_primitive(obj))
 
 
-class TestObjMakeList(test.NoDBTestCase):
-
-    def test_obj_make_list(self):
-        class MyList(base.ObjectListBase, base.NovaObject):
-            fields = {
-                'objects': fields.ListOfObjectsField('MyObj'),
-            }
-
-        db_objs = [{'foo': 1, 'bar': 'baz', 'missing': 'banana'},
-                   {'foo': 2, 'bar': 'bat', 'missing': 'apple'},
-                   ]
-        mylist = base.obj_make_list('ctxt', MyList(), MyObj, db_objs)
-        self.assertEqual(2, len(mylist))
-        self.assertEqual('ctxt', mylist._context)
-        for index, item in enumerate(mylist):
-            self.assertEqual(db_objs[index]['foo'], item.foo)
-            self.assertEqual(db_objs[index]['bar'], item.bar)
-            self.assertEqual(db_objs[index]['missing'], item.missing)
-
-
 def compare_obj(test, obj, db_obj, subs=None, allow_missing=None,
                 comparators=None):
     """Compare a NovaObject and a dict-like database object.
@@ -571,23 +547,6 @@ class _TestObject(object):
         self.assertFalse(obj.obj_attr_is_set('bar'))
         self.assertRaises(AttributeError, obj.obj_attr_is_set, 'bang')
 
-    def test_obj_reset_changes_recursive(self):
-        obj = MyObj(rel_object=MyOwnedObject(baz=123),
-                    rel_objects=[MyOwnedObject(baz=456)])
-        self.assertEqual(set(['rel_object', 'rel_objects']),
-                         obj.obj_what_changed())
-        obj.obj_reset_changes()
-        self.assertEqual(set(['rel_object']), obj.obj_what_changed())
-        self.assertEqual(set(['baz']), obj.rel_object.obj_what_changed())
-        self.assertEqual(set(['baz']), obj.rel_objects[0].obj_what_changed())
-        obj.obj_reset_changes(recursive=True, fields=['foo'])
-        self.assertEqual(set(['rel_object']), obj.obj_what_changed())
-        self.assertEqual(set(['baz']), obj.rel_object.obj_what_changed())
-        self.assertEqual(set(['baz']), obj.rel_objects[0].obj_what_changed())
-        obj.obj_reset_changes(recursive=True)
-        self.assertEqual(set([]), obj.rel_object.obj_what_changed())
-        self.assertEqual(set([]), obj.obj_what_changed())
-
     def test_get(self):
         obj = MyObj(foo=1)
         # Foo has value, should not get the default
@@ -612,34 +571,13 @@ class _TestObject(object):
                          'rel_objects', 'mutable_default'] +
                         list(base_fields))
         myobj3_fields = ['new_field']
-        self.assertTrue(issubclass(TestSubclassedObject, MyObj))
+        self.assertTrue(issubclass(SubclassedObject, MyObj))
         self.assertEqual(len(myobj_fields), len(MyObj.fields))
         self.assertEqual(set(myobj_fields), set(MyObj.fields.keys()))
         self.assertEqual(len(myobj_fields) + len(myobj3_fields),
-                         len(TestSubclassedObject.fields))
+                         len(SubclassedObject.fields))
         self.assertEqual(set(myobj_fields) | set(myobj3_fields),
-                         set(TestSubclassedObject.fields.keys()))
-
-    def test_obj_as_admin(self):
-        obj = MyObj(context=self.context)
-
-        def fake(*args, **kwargs):
-            self.assertTrue(obj._context.is_admin)
-
-        with mock.patch.object(obj, 'obj_reset_changes') as mock_fn:
-            mock_fn.side_effect = fake
-            with obj.obj_as_admin():
-                obj.save()
-            self.assertTrue(mock_fn.called)
-
-        self.assertFalse(obj._context.is_admin)
-
-    def test_obj_as_admin_orphaned(self):
-        def testme():
-            obj = MyObj()
-            with obj.obj_as_admin():
-                pass
-        self.assertRaises(exception.OrphanedObjectError, testme)
+                         set(SubclassedObject.fields.keys()))
 
     def test_obj_alternate_context(self):
         obj = MyObj(context=self.context)
@@ -943,7 +881,7 @@ class TestObjectSerializer(_BaseTestCase):
             }
 
         @base.NovaObjectRegistry.register  # noqa
-        class Parent(base.NovaObject):
+        class Parent(base.NovaObject):  # noqa
             VERSION = '1.1'
 
             fields = {
@@ -1038,7 +976,7 @@ class TestArgsSerializer(test.NoDBTestCase):
         super(TestArgsSerializer, self).setUp()
         self.now = timeutils.utcnow()
         self.str_now = utils.strtime(self.now)
-        self.unicode_str = u'\xF0\x9F\x92\xA9'
+        self.exc = exception.NotFound()
 
     @base.serialize_args
     def _test_serialize_args(self, *args, **kwargs):
@@ -1047,14 +985,26 @@ class TestArgsSerializer(test.NoDBTestCase):
             self.assertEqual(expected_args[index], val)
 
         expected_kwargs = {'a': 'untouched', 'b': self.str_now,
-                           'c': self.str_now, 'exc_val': self.unicode_str}
-        for key, val in six.iteritems(kwargs):
+                           'c': self.str_now}
+
+        nonnova = kwargs.pop('nonnova', None)
+        if nonnova:
+            expected_kwargs['exc_val'] = 'TestingException'
+        else:
+            expected_kwargs['exc_val'] = self.exc.format_message()
+        for key, val in kwargs.items():
             self.assertEqual(expected_kwargs[key], val)
 
     def test_serialize_args(self):
         self._test_serialize_args('untouched', self.now, self.now,
                                   a='untouched', b=self.now, c=self.now,
-                                  exc_val=self.unicode_str)
+                                  exc_val=self.exc)
+
+    def test_serialize_args_non_nova_exception(self):
+        self._test_serialize_args('untouched', self.now, self.now,
+                                  a='untouched', b=self.now, c=self.now,
+                                  exc_val=test.TestingException('foo'),
+                                  nonnova=True)
 
 
 class TestRegistry(test.NoDBTestCase):
@@ -1098,114 +1048,115 @@ object_data = {
     'Agent': '1.0-c0c092abaceb6f51efe5d82175f15eba',
     'AgentList': '1.0-5a7380d02c3aaf2a32fc8115ae7ca98c',
     'Aggregate': '1.3-f315cb68906307ca2d1cca84d4753585',
-    'AggregateList': '1.2-fb6e19f3c3a3186b04eceb98b5dadbfa',
-    'Allocation': '1.0-864506325f1822f4e4805b56faf51bbe',
-    'AllocationList': '1.1-e43fe4a9c9cbbda7438b0e48332f099e',
+    'AggregateList': '1.3-3ea55a050354e72ef3306adefa553957',
     'BandwidthUsage': '1.2-c6e4c779c7f40f2407e3d70022e3cd1c',
     'BandwidthUsageList': '1.2-5fe7475ada6fe62413cbfcc06ec70746',
-    'BlockDeviceMapping': '1.17-5e094927f1251770dcada6ab05adfcdb',
+    'BlockDeviceMapping': '1.20-45a6ad666ddf14bbbedece2293af77e2',
     'BlockDeviceMappingList': '1.17-1e568eecb91d06d4112db9fd656de235',
-    'BuildRequest': '1.2-532d95a88c5fd33e85878e408e5d6e8d',
+    'BuildRequest': '1.3-077dee42bed93f8a5b62be77657b7152',
     'BuildRequestList': '1.0-cd95608eccb89fbc702c8b52f38ec738',
-    'CellMapping': '1.0-7f1a7e85a22bbb7559fc730ab658b9bd',
-    'CellMappingList': '1.0-4ee0d9efdfd681fed822da88376e04d2',
-    'ComputeNode': '1.16-2436e5b836fa0306a3c4e6d9e5ddacec',
-    'ComputeNodeList': '1.15-4ec4ea3ed297edbd25c33e2aaf797cca',
-    'DNSDomain': '1.0-7b0b2dab778454b6a7b6c66afe163a1a',
-    'DNSDomainList': '1.0-4ee0d9efdfd681fed822da88376e04d2',
-    'Destination': '1.0-4c59dd1288b2e7adbda6051a2de59183',
+    'CellMapping': '1.1-5d652928000a5bc369d79d5bde7e497d',
+    'CellMappingList': '1.1-496ef79bb2ab41041fff8bcb57996352',
+    'ComputeNode': '1.19-af6bd29a6c3b225da436a0d8487096f2',
+    'ComputeNodeList': '1.17-52f3b0962b1c86b98590144463ebb192',
+    'ConsoleAuthToken': '1.1-8da320fb065080eb4d3c2e5c59f8bf52',
+    'CpuDiagnostics': '1.0-d256f2e442d1b837735fd17dfe8e3d47',
+    'Destination': '1.4-3b440d29459e2c98987ad5b25ad1cb2c',
     'DeviceBus': '1.0-77509ea1ea0dd750d5864b9bd87d3f9d',
     'DeviceMetadata': '1.0-04eb8fd218a49cbc3b1e54b774d179f7',
+    'Diagnostics': '1.0-38ad3e9b1a59306253fc03f97936db95',
+    'DiskDiagnostics': '1.0-dfd0892b5924af1a585f3fed8c9899ca',
     'DiskMetadata': '1.0-e7a0f1ccccf10d26a76b28e7492f3788',
     'EC2Ids': '1.0-474ee1094c7ec16f8ce657595d8c49d9',
     'EC2InstanceMapping': '1.0-a4556eb5c5e94c045fe84f49cf71644f',
-    'EC2SnapshotMapping': '1.0-47e7ddabe1af966dce0cfd0ed6cd7cd1',
-    'EC2VolumeMapping': '1.0-5b713751d6f97bad620f3378a521020d',
-    'FixedIP': '1.14-53e1c10b539f1a82fe83b1af4720efae',
-    'FixedIPList': '1.14-87a39361c8f08f059004d6b15103cdfd',
-    'Flavor': '1.1-b6bb7a730a79d720344accefafacf7ee',
+    'Flavor': '1.2-4ce99b41327bb230262e5a8f45ff0ce3',
     'FlavorList': '1.1-52b5928600e7ca973aa4fc1e46f3934c',
-    'FloatingIP': '1.10-52a67d52d85eb8b3f324a5b7935a335b',
-    'FloatingIPList': '1.11-7f2ba670714e1b7bab462ab3290f7159',
+    'HVSpec': '1.2-de06bcec472a2f04966b855a49c46b41',
     'HostMapping': '1.0-1a3390a696792a552ab7bd31a77ba9ac',
-    'HyperVLiveMigrateData': '1.1-9987a3cec31a81abac6fba7cc722e43f',
-    'HVSpec': '1.2-db672e73304da86139086d003f3977e7',
+    'HostMappingList': '1.1-18ac2bfb8c1eb5545bed856da58a79bc',
+    'HyperVLiveMigrateData': '1.4-e265780e6acfa631476c8170e8d6fce0',
     'IDEDeviceBus': '1.0-29d4c9f27ac44197f01b6ac1b7e16502',
     'ImageMeta': '1.8-642d1b2eb3e880a367f37d72dd76162d',
-    'ImageMetaProps': '1.15-d45133ec8d2d4a6456338fb0ffd0e5c2',
-    'Instance': '2.3-4f98ab23f4b0a25fabb1040c8f5edecc',
-    'InstanceAction': '1.1-f9f293e526b66fca0d05c3b3a2d13914',
-    'InstanceActionEvent': '1.1-e56a64fa4710e43ef7af2ad9d6028b33',
+    'ImageMetaProps': '1.27-f3f17d5e35146a0dbb56420ffc4f3990',
+    'Instance': '2.7-d187aec68cad2e4d8b8a03a68e4739ce',
+    'InstanceAction': '1.2-9a5abc87fdd3af46f45731960651efb5',
+    'InstanceActionEvent': '1.4-5b1f361bd81989f8bb2c20bb7e8a4cb4',
     'InstanceActionEventList': '1.1-13d92fb953030cdbfee56481756e02be',
-    'InstanceActionList': '1.0-4a53826625cc280e15fae64a575e0879',
+    'InstanceActionList': '1.1-a2b2fb6006b47c27076d3a1d48baa759',
     'InstanceDeviceMetadata': '1.0-74d78dd36aa32d26d2769a1b57caf186',
-    'InstanceExternalEvent': '1.1-6e446ceaae5f475ead255946dd443417',
+    'InstanceExternalEvent': '1.4-06c2dfcf2d2813c24cd37ee728524f1a',
     'InstanceFault': '1.2-7ef01f16f1084ad1304a513d6d410a38',
-    'InstanceFaultList': '1.1-f8ec07cbe3b60f5f07a8b7a06311ac0d',
-    'InstanceGroup': '1.10-1a0c8c7447dc7ecb9da53849430c4a5f',
-    'InstanceGroupList': '1.7-be18078220513316abd0ae1b2d916873',
+    'InstanceFaultList': '1.2-6bb72de2872fe49ded5eb937a93f2451',
+    'InstanceGroup': '1.11-852ac511d30913ee88f3c3a869a8f30a',
+    'InstanceGroupList': '1.8-90f8f1a445552bb3bbc9fa1ae7da27d4',
     'InstanceInfoCache': '1.5-cd8b96fefe0fc8d4d337243ba0bf0e1e',
-    'InstanceList': '2.1-e64b9f623db6370b22ec910461f06a52',
-    'InstanceMapping': '1.0-65de80c491f54d19374703c0753c4d47',
-    'InstanceMappingList': '1.0-9e982e3de1613b9ada85e35f69b23d47',
-    'InstanceNUMACell': '1.3-6991a20992c5faa57fae71a45b40241b',
-    'InstanceNUMATopology': '1.2-d944a7d6c21e1c773ffdf09c6d025954',
-    'InstancePCIRequest': '1.1-b1d75ebc716cb12906d9d513890092bf',
+    'InstanceList': '2.6-238f125650c25d6d12722340d726f723',
+    'InstanceMapping': '1.2-3bd375e65c8eb9c45498d2f87b882e03',
+    'InstanceMappingList': '1.3-d34b6ebb076d542ae0f8b440534118da',
+    'InstanceNUMACell': '1.6-25d9120d83a18356f4146f2a6fe2cc8d',
+    'InstanceNUMATopology': '1.3-ec0030cb0402a49c96da7051c037082a',
+    'InstancePCIRequest': '1.3-f6d324f1c337fad4f34892ed5f484c9a',
     'InstancePCIRequests': '1.1-65e38083177726d806684cb1cc0136d2',
-    'Inventory': '1.0-84131c00c84a27ee6930d01b329c9a9d',
-    'InventoryList': '1.0-de53f0fd078c27cc1d43400f4e8bcef8',
-    'LibvirtLiveMigrateBDMInfo': '1.0-252aabb723ca79d5469fa56f64b57811',
-    'LibvirtLiveMigrateData': '1.3-2795e5646ee21e8c7f1c3e64fb6c80a3',
     'KeyPair': '1.4-1244e8d1b103cc69d038ed78ab3a8cc6',
     'KeyPairList': '1.3-94aad3ac5c938eef4b5e83da0212f506',
-    'Migration': '1.4-17979b9f2ae7f28d97043a220b2a8350',
-    'MigrationContext': '1.1-9fb17b0b521370957a884636499df52d',
-    'MigrationList': '1.3-55595bfc1a299a5962614d0821a3567e',
+    'LibvirtLiveMigrateBDMInfo': '1.1-5f4a68873560b6f834b74e7861d71aaf',
+    'LibvirtLiveMigrateData': '1.10-348cf70ea44d3b985f45f64725d6f6a7',
+    'LibvirtLiveMigrateNUMAInfo': '1.0-0e777677f3459d0ed1634eabbdb6c22f',
+    'MemoryDiagnostics': '1.0-2c995ae0f2223bb0f8e523c5cc0b83da',
+    'Migration': '1.7-bd45b232fd7c95cd79ae9187e10ef582',
+    'MigrationContext': '1.2-89f10a83999f852a489962ae37d8a026',
+    'MigrationList': '1.5-36793f8d65bae421bd5564d09a4de7be',
     'MonitorMetric': '1.1-53b1db7c4ae2c531db79761e7acc52ba',
     'MonitorMetricList': '1.1-15ecf022a68ddbb8c2a6739cfc9f8f5e',
-    'NUMACell': '1.2-74fc993ac5c83005e76e34e8487f1c05',
+    'NUMACell': '1.4-7695303e820fa855d76954be2eb2680e',
     'NUMAPagesTopology': '1.1-edab9fa2dc43c117a38d600be54b4542',
     'NUMATopology': '1.2-c63fad38be73b6afd04715c9c1b29220',
-    'NUMATopologyLimits': '1.0-9463e0edd40f64765ae518a539b9dfd2',
-    'Network': '1.2-a977ab383aa462a479b2fae8211a5dde',
-    'NetworkInterfaceMetadata': '1.0-99a9574d086feb5ad45cd04a34855647',
-    'NetworkList': '1.2-69eca910d8fa035dfecd8ba10877ee59',
+    'NUMATopologyLimits': '1.1-4235c5da7a76c7e36075f0cd2f5cf922',
+    'NetworkInterfaceMetadata': '1.2-6f3d480b40fe339067b1c0dd4d656716',
+    'NetworkMetadata': '1.0-2cb8d21b34f87b0261d3e1d1ae5cf218',
     'NetworkRequest': '1.2-af1ff2d986999fbb79377712794d82aa',
     'NetworkRequestList': '1.1-15ecf022a68ddbb8c2a6739cfc9f8f5e',
-    'PciDevice': '1.5-0d5abe5c91645b8469eb2a93fc53f932',
+    'NicDiagnostics': '1.0-895e9ad50e0f56d5258585e3e066aea5',
     'PCIDeviceBus': '1.0-2b891cb77e42961044689f3dc2718995',
+    'PciDevice': '1.6-25ca0542a22bc25386a72c0065a79c01',
     'PciDeviceList': '1.3-52ff14355491c8c580bdc0ba34c26210',
     'PciDevicePool': '1.1-3f5ddc3ff7bfa14da7f6c7e9904cc000',
     'PciDevicePoolList': '1.1-15ecf022a68ddbb8c2a6739cfc9f8f5e',
-    'Quotas': '1.2-1fe4cd50593aaf5d36a6dc5ab3f98fb3',
-    'QuotasNoOp': '1.2-e041ddeb7dc8188ca71706f78aad41c1',
-    'RequestSpec': '1.6-c1cb516acdf120d367a42d343ed695b5',
-    'ResourceProvider': '1.1-7bbcd5ea1c51782692f55489ab08dea6',
-    'ResourceProviderList': '1.0-82bd48d8d0f7913bbe7266f3835c81bf',
+    'PowerVMLiveMigrateData': '1.4-a745f4eda16b45e1bc5686a0c498f27e',
+    'Quotas': '1.3-3b2b91371f60e788035778fc5f87797d',
+    'QuotasNoOp': '1.3-d1593cf969c81846bc8192255ea95cce',
+    'RequestGroup': '1.3-0458d350a8ec9d0673f9be5640a990ce',
+    'RequestLevelParams': '1.0-1e5c8c18bd44cd233c8b32509c99d06f',
+    'RequestSpec': '1.13-e1aa38b2bf3f8547474ee9e4c0aa2745',
+    'Resource': '1.0-d8a2abbb380da583b995fd118f6a8953',
+    'ResourceList': '1.0-4a53826625cc280e15fae64a575e0879',
+    'ResourceMetadata': '1.0-77509ea1ea0dd750d5864b9bd87d3f9d',
     'S3ImageMapping': '1.0-7dd7366a890d82660ed121de9092276e',
+    'SCSIDeviceBus': '1.0-61c1e89a00901069ab1cf2991681533b',
     'SchedulerLimits': '1.0-249c4bd8e62a9b327b7026b7f19cc641',
     'SchedulerRetries': '1.1-3c9c8b16143ebbb6ad7030e999d14cc0',
-    'SCSIDeviceBus': '1.0-61c1e89a00901069ab1cf2991681533b',
-    'SecurityGroup': '1.1-0e1b9ba42fe85c13c1437f8b74bdb976',
-    'SecurityGroupList': '1.0-dc8bbea01ba09a2edb6e5233eae85cbc',
-    'SecurityGroupRule': '1.1-ae1da17b79970012e8536f88cb3c6b29',
-    'SecurityGroupRuleList': '1.2-0005c47fcd0fb78dd6d7fd32a1409f5b',
-    'Service': '1.20-0f9c0bf701e68640b78638fd09e2cddc',
+    'SecurityGroup': '1.2-86d67d8d3ab0c971e1dc86e02f9524a8',
+    'SecurityGroupList': '1.1-c655ed13298e630f4d398152f7d08d71',
+    'Selection': '1.1-548e3c2f04da2a61ceaf9c4e1589f264',
+    'Service': '1.22-8a740459ab9bf258a19c8fcb875c2d9a',
     'ServiceList': '1.19-5325bce13eebcbf22edc9678285270cc',
-    'TaskLog': '1.0-78b0534366f29aa3eebb01860fbe18fe',
-    'TaskLogList': '1.0-cc8cce1af8a283b9d28b55fcd682e777',
     'Tag': '1.1-8b8d7d5b48887651a0e01241672e2963',
     'TagList': '1.1-55231bdb671ecf7641d6a2e9109b5d8e',
-    'Usage': '1.0-b78f18c3577a38e7a033e46a9725b09b',
-    'UsageList': '1.0-de53f0fd078c27cc1d43400f4e8bcef8',
+    'TaskLog': '1.0-78b0534366f29aa3eebb01860fbe18fe',
+    'TaskLogList': '1.0-cc8cce1af8a283b9d28b55fcd682e777',
+    'TrustedCerts': '1.0-dcf528851e0f868c77ee47e90563cda7',
     'USBDeviceBus': '1.0-e4c7dd6032e46cd74b027df5eb2d4750',
-    'VirtCPUFeature': '1.0-3310718d8c72309259a6e39bdefe83ee',
-    'VirtCPUModel': '1.0-6a5cc9f322729fc70ddc6733bacd57d3',
+    'VIFMigrateData': '1.0-cb15282b25a039ab35046ed705eb931d',
+    'VMwareLiveMigrateData': '1.0-a3cc858a2bf1d3806d6f57cfaa1fb98a',
+    'VirtCPUFeature': '1.0-ea2464bdd09084bd388e5f61d5d4fc86',
+    'VirtCPUModel': '1.0-5e1864af9227f698326203d7249796b5',
     'VirtCPUTopology': '1.0-fc694de72e20298f7c6bab1083fd4563',
     'VirtualInterface': '1.3-efd3ca8ebcc5ce65fff5a25f31754c54',
     'VirtualInterfaceList': '1.0-9750e2074437b3077e46359102779fc6',
     'VolumeUsage': '1.0-6c8190c46ce1469bb3286a1f21c2e475',
-    'XenapiLiveMigrateData': '1.0-5f982bec68f066e194cd9ce53a24ac4c',
+    'XenDeviceBus': '1.0-272a4f899b24e31e42b2b9a7ed7e9194',
+    # TODO(efried): re-alphabetize this
+    'LibvirtVPMEMDevice': '1.0-17ffaf47585199eeb9a2b83d6bde069f',
 }
 
 
@@ -1223,10 +1174,16 @@ def get_nova_objects():
     nova_classes = {}
     for name in all_classes:
         objclasses = all_classes[name]
-        if (objclasses[0].OBJ_PROJECT_NAMESPACE !=
-            base.NovaObject.OBJ_PROJECT_NAMESPACE):
-            continue
-        nova_classes[name] = objclasses
+        # NOTE(danms): All object registries that inherit from the
+        # base VersionedObjectRegistry share a common list of classes.
+        # That means even things like os_vif objects will be in our
+        # registry, and for any of them that share the same name
+        # (i.e. Network), we need to keep ours and exclude theirs.
+        our_ns = [cls for cls in objclasses
+                  if (cls.OBJ_PROJECT_NAMESPACE ==
+                      base.NovaObject.OBJ_PROJECT_NAMESPACE)]
+        if our_ns:
+            nova_classes[name] = our_ns
     return nova_classes
 
 
@@ -1268,8 +1225,7 @@ class TestObjectVersions(test.NoDBTestCase):
         init_args = {}
         init_kwargs = {}
 
-        checker = fixture.ObjectVersionChecker(
-            base.NovaObjectRegistry.obj_classes())
+        checker = fixture.ObjectVersionChecker(get_nova_objects())
         checker.test_compatibility_routines(use_manifest=True,
                                             init_args=init_args,
                                             init_kwargs=init_kwargs)
@@ -1365,9 +1321,46 @@ class TestObjEqualPrims(_BaseTestCase):
 
 class TestObjMethodOverrides(test.NoDBTestCase):
     def test_obj_reset_changes(self):
-        args = inspect.getargspec(base.NovaObject.obj_reset_changes)
+        args = utils.getargspec(base.NovaObject.obj_reset_changes)
         obj_classes = base.NovaObjectRegistry.obj_classes()
         for obj_name in obj_classes:
             obj_class = obj_classes[obj_name][0]
             self.assertEqual(args,
-                    inspect.getargspec(obj_class.obj_reset_changes))
+                utils.getargspec(obj_class.obj_reset_changes))
+
+
+class TestObjectsDefaultingOnInit(test.NoDBTestCase):
+    def test_init_behavior_policy(self):
+        all_objects = get_nova_objects()
+
+        violations = collections.defaultdict(list)
+
+        # NOTE(danms): Do not add things to this list!
+        #
+        # There is one known exception to this init policy, and that
+        # is the Service object because of the special behavior of the
+        # version field. We *want* to counteract the usual non-clobber
+        # behavior of that field specifically. See the comments in
+        # Service.__init__ for more details. This will likely never
+        # apply to any other non-ephemeral object, so this list should
+        # never grow.
+        exceptions = [objects.Service]
+
+        for name, objclasses in all_objects.items():
+            for objcls in objclasses:
+                if objcls in exceptions:
+                    continue
+
+                key = '%s-%s' % (name, objcls.VERSION)
+                obj = objcls()
+                if isinstance(obj, base.NovaEphemeralObject):
+                    # Skip ephemeral objects, which are allowed to
+                    # set fields at init time
+                    continue
+                for field in objcls.fields:
+                    if field in obj:
+                        violations[key].append(field)
+
+        self.assertEqual({}, violations,
+                         'Some non-ephemeral objects set fields during '
+                         'initialization; This is not allowed.')
