@@ -25,6 +25,43 @@ from nova.api.validation import parameter_types
 from nova.api.validation import validators
 from nova import exception
 from nova import test
+from nova.tests.unit.api.openstack import fakes
+
+
+query_schema = {
+    'type': 'object',
+    'properties': {
+        'foo': parameter_types.single_param({'type': 'string',
+                                             'format': 'uuid'}),
+        'foos': parameter_types.multi_params({'type': 'string'})
+    },
+    'patternProperties': {
+        "^_": parameter_types.multi_params({'type': 'string'})},
+    'additionalProperties': True
+}
+
+
+class FakeQueryParametersController(object):
+
+    @validation.query_schema(query_schema, '2.3')
+    def get(self, req):
+        return list(set(req.GET.keys()))
+
+
+class RegexFormatFakeController(object):
+
+    schema = {
+        'type': 'object',
+        'properties': {
+            'foo': {
+                'format': 'regex',
+            },
+        },
+    }
+
+    @validation.schema(request_body_schema=schema)
+    def post(self, req, body):
+        return 'Validation succeeded.'
 
 
 class FakeRequest(object):
@@ -37,15 +74,6 @@ class FakeRequest(object):
 
 
 class ValidationRegex(test.NoDBTestCase):
-    def test_cell_names(self):
-        cellre = re.compile(parameter_types.valid_cell_name_regex.regex)
-        self.assertTrue(cellre.search('foo'))
-        self.assertFalse(cellre.search('foo.bar'))
-        self.assertFalse(cellre.search('foo@bar'))
-        self.assertFalse(cellre.search('foo!bar'))
-        self.assertFalse(cellre.search(' foo!bar'))
-        self.assertFalse(cellre.search('\nfoo!bar'))
-
     def test_build_regex_range(self):
         # this is much easier to think about if we only use the ascii
         # subset because it's a printable range we can think
@@ -57,7 +85,9 @@ class ValidationRegex(test.NoDBTestCase):
         self.useFixture(fixtures.MonkeyPatch(
             'nova.api.validation.parameter_types._get_all_chars',
             _get_all_chars))
-
+        # note that since we use only the ascii range in the tests
+        # we have to clear the cache to recompute them.
+        parameter_types._reset_cache()
         r = parameter_types._build_regex_range(ws=False)
         self.assertEqual(r, re.escape('!') + '-' + re.escape('~'))
 
@@ -94,14 +124,30 @@ class ValidationRegex(test.NoDBTestCase):
 
 class APIValidationTestCase(test.NoDBTestCase):
 
+    post_schema = None
+
+    def setUp(self):
+        super(APIValidationTestCase, self).setUp()
+        self.post = None
+
+        if self.post_schema is not None:
+            @validation.schema(request_body_schema=self.post_schema)
+            def post(req, body):
+                return 'Validation succeeded.'
+
+            self.post = post
+
     def check_validation_error(self, method, body, expected_detail, req=None):
         if not req:
             req = FakeRequest()
         try:
-            method(body=body, req=req,)
+            method(body=body, req=req)
         except exception.ValidationError as ex:
             self.assertEqual(400, ex.kwargs['code'])
-            if not re.match(expected_detail, ex.kwargs['detail']):
+            if isinstance(expected_detail, list):
+                self.assertIn(ex.kwargs['detail'], expected_detail,
+                              'Exception details did not match expected')
+            elif not re.match(expected_detail, ex.kwargs['detail']):
                 self.assertEqual(expected_detail, ex.kwargs['detail'],
                                  'Exception details did not match expected')
         except Exception as ex:
@@ -112,29 +158,27 @@ class APIValidationTestCase(test.NoDBTestCase):
 
 class FormatCheckerTestCase(test.NoDBTestCase):
 
-    def test_format_checker_failed(self):
+    def _format_checker(self, format, value, error_message):
         format_checker = validators.FormatChecker()
         exc = self.assertRaises(jsonschema_exc.FormatError,
-                                format_checker.check, "   ", "name")
+                                format_checker.check, value, format)
         self.assertIsInstance(exc.cause, exception.InvalidName)
-        self.assertEqual("An invalid 'name' value was provided. The name must "
-                         "be: printable characters. "
-                         "Can not start or end with whitespace.",
+        self.assertEqual(error_message,
                          exc.cause.format_message())
 
-    def test_format_checker_failed_with_non_string(self):
-        checks = ["name", "name_with_leading_trailing_spaces",
-                  "cell_name", "cell_name_with_leading_trailing_spaces"]
-        format_checker = validators.FormatChecker()
+    def test_format_checker_failed_with_non_string_name(self):
+        error_message = ("An invalid 'name' value was provided. The name must "
+                         "be: printable characters. "
+                         "Can not start or end with whitespace.")
+        self._format_checker("name", "   ", error_message)
+        self._format_checker("name", None, error_message)
 
-        for check in checks:
-            exc = self.assertRaises(jsonschema_exc.FormatError,
-                                    format_checker.check, None, "name")
-            self.assertIsInstance(exc.cause, exception.InvalidName)
-            self.assertEqual("An invalid 'name' value was provided. The name "
-                             "must be: printable characters. "
-                             "Can not start or end with whitespace.",
-                             exc.cause.format_message())
+    def test_format_checker_failed_name_with_leading_trailing_spaces(self):
+        error_message = ("An invalid 'name' value was provided. "
+                         "The name must be: printable characters with at "
+                         "least one non space character")
+        self._format_checker("name_with_leading_trailing_spaces",
+                             None, error_message)
 
 
 class MicroversionsSchemaTestCase(APIValidationTestCase):
@@ -202,24 +246,84 @@ class MicroversionsSchemaTestCase(APIValidationTestCase):
                                     expected_detail=detail, req=req)
 
 
-class RequiredDisableTestCase(APIValidationTestCase):
+class QueryParamsSchemaTestCase(test.NoDBTestCase):
 
     def setUp(self):
-        super(RequiredDisableTestCase, self).setUp()
-        schema = {
-            'type': 'object',
-            'properties': {
-                'foo': {
-                    'type': 'integer',
-                },
+        super(QueryParamsSchemaTestCase, self).setUp()
+        self.controller = FakeQueryParametersController()
+
+    def test_validate_request(self):
+        req = fakes.HTTPRequest.blank("/tests?foo=%s" % fakes.FAKE_UUID)
+        req.api_version_request = api_version.APIVersionRequest("2.3")
+        self.assertEqual(['foo'], self.controller.get(req))
+
+    def test_validate_request_failed(self):
+        # parameter 'foo' expect a UUID
+        req = fakes.HTTPRequest.blank("/tests?foo=abc")
+        req.api_version_request = api_version.APIVersionRequest("2.3")
+        ex = self.assertRaises(exception.ValidationError, self.controller.get,
+                               req)
+        self.assertEqual("Invalid input for query parameters foo. Value: "
+                         "abc. 'abc' is not a 'uuid'", str(ex))
+
+    def test_validate_request_with_multiple_values(self):
+        req = fakes.HTTPRequest.blank("/tests?foos=abc")
+        req.api_version_request = api_version.APIVersionRequest("2.3")
+        self.assertEqual(['foos'], self.controller.get(req))
+        req = fakes.HTTPRequest.blank("/tests?foos=abc&foos=def")
+        self.assertEqual(['foos'], self.controller.get(req))
+
+    def test_validate_request_with_multiple_values_fails(self):
+        req = fakes.HTTPRequest.blank(
+            "/tests?foo=%s&foo=%s" % (fakes.FAKE_UUID, fakes.FAKE_UUID))
+        req.api_version_request = api_version.APIVersionRequest("2.3")
+        self.assertRaises(exception.ValidationError, self.controller.get, req)
+
+    def test_validate_request_unicode_decode_failure(self):
+        req = fakes.HTTPRequest.blank("/tests?foo=%88")
+        req.api_version_request = api_version.APIVersionRequest("2.1")
+        ex = self.assertRaises(
+            exception.ValidationError, self.controller.get, req)
+        self.assertIn("Query string is not UTF-8 encoded", six.text_type(ex))
+
+    def test_strip_out_additional_properties(self):
+        req = fakes.HTTPRequest.blank(
+            "/tests?foos=abc&foo=%s&bar=123&-bar=456" % fakes.FAKE_UUID)
+        req.api_version_request = api_version.APIVersionRequest("2.3")
+        res = self.controller.get(req)
+        res.sort()
+        self.assertEqual(['foo', 'foos'], res)
+
+    def test_no_strip_out_additional_properties_when_not_match_version(self):
+        req = fakes.HTTPRequest.blank(
+            "/tests?foos=abc&foo=%s&bar=123&bar=456" % fakes.FAKE_UUID)
+        # The JSON-schema matches to the API version 2.3 and above. Request
+        # with version 2.1 to ensure there isn't no strip out for additional
+        # parameters when schema didn't match the request version.
+        req.api_version_request = api_version.APIVersionRequest("2.1")
+        res = self.controller.get(req)
+        res.sort()
+        self.assertEqual(['bar', 'foo', 'foos'], res)
+
+    def test_strip_out_correct_pattern_retained(self):
+        req = fakes.HTTPRequest.blank(
+            "/tests?foos=abc&foo=%s&bar=123&_foo_=456" % fakes.FAKE_UUID)
+        req.api_version_request = api_version.APIVersionRequest("2.3")
+        res = self.controller.get(req)
+        res.sort()
+        self.assertEqual(['_foo_', 'foo', 'foos'], res)
+
+
+class RequiredDisableTestCase(APIValidationTestCase):
+
+    post_schema = {
+        'type': 'object',
+        'properties': {
+            'foo': {
+                'type': 'integer',
             },
-        }
-
-        @validation.schema(request_body_schema=schema)
-        def post(req, body):
-            return 'Validation succeeded.'
-
-        self.post = post
+        },
+    }
 
     def test_validate_required_disable(self):
         self.assertEqual(self.post(body={'foo': 1}, req=FakeRequest()),
@@ -230,23 +334,15 @@ class RequiredDisableTestCase(APIValidationTestCase):
 
 class RequiredEnableTestCase(APIValidationTestCase):
 
-    def setUp(self):
-        super(RequiredEnableTestCase, self).setUp()
-        schema = {
-            'type': 'object',
-            'properties': {
-                'foo': {
-                    'type': 'integer',
-                },
+    post_schema = {
+        'type': 'object',
+        'properties': {
+            'foo': {
+                'type': 'integer',
             },
-            'required': ['foo']
-        }
-
-        @validation.schema(request_body_schema=schema)
-        def post(req, body):
-            return 'Validation succeeded.'
-
-        self.post = post
+        },
+        'required': ['foo']
+    }
 
     def test_validate_required_enable(self):
         self.assertEqual(self.post(body={'foo': 1},
@@ -260,23 +356,15 @@ class RequiredEnableTestCase(APIValidationTestCase):
 
 class AdditionalPropertiesEnableTestCase(APIValidationTestCase):
 
-    def setUp(self):
-        super(AdditionalPropertiesEnableTestCase, self).setUp()
-        schema = {
-            'type': 'object',
-            'properties': {
-                'foo': {
-                    'type': 'integer',
-                },
+    post_schema = {
+        'type': 'object',
+        'properties': {
+            'foo': {
+                'type': 'integer',
             },
-            'required': ['foo'],
-        }
-
-        @validation.schema(request_body_schema=schema)
-        def post(req, body):
-            return 'Validation succeeded.'
-
-        self.post = post
+        },
+        'required': ['foo'],
+    }
 
     def test_validate_additionalProperties_enable(self):
         self.assertEqual(self.post(body={'foo': 1}, req=FakeRequest()),
@@ -288,24 +376,16 @@ class AdditionalPropertiesEnableTestCase(APIValidationTestCase):
 
 class AdditionalPropertiesDisableTestCase(APIValidationTestCase):
 
-    def setUp(self):
-        super(AdditionalPropertiesDisableTestCase, self).setUp()
-        schema = {
-            'type': 'object',
-            'properties': {
-                'foo': {
-                    'type': 'integer',
-                },
+    post_schema = {
+        'type': 'object',
+        'properties': {
+            'foo': {
+                'type': 'integer',
             },
-            'required': ['foo'],
-            'additionalProperties': False,
-        }
-
-        @validation.schema(request_body_schema=schema)
-        def post(req, body):
-            return 'Validation succeeded.'
-
-        self.post = post
+        },
+        'required': ['foo'],
+        'additionalProperties': False,
+    }
 
     def test_validate_additionalProperties_disable(self):
         self.assertEqual(self.post(body={'foo': 1}, req=FakeRequest()),
@@ -319,64 +399,61 @@ class AdditionalPropertiesDisableTestCase(APIValidationTestCase):
 
 class PatternPropertiesTestCase(APIValidationTestCase):
 
-    def setUp(self):
-        super(PatternPropertiesTestCase, self).setUp()
-        schema = {
-            'patternProperties': {
-                '^[a-zA-Z0-9]{1,10}$': {
-                    'type': 'string'
-                },
+    post_schema = {
+        'patternProperties': {
+            '^[a-zA-Z0-9]{1,10}$': {
+                'type': 'string'
             },
-            'additionalProperties': False,
-        }
-
-        @validation.schema(request_body_schema=schema)
-        def post(req, body):
-            return 'Validation succeeded.'
-
-        self.post = post
+        },
+        'additionalProperties': False,
+    }
 
     def test_validate_patternProperties(self):
         self.assertEqual('Validation succeeded.',
                          self.post(body={'foo': 'bar'}, req=FakeRequest()))
 
     def test_validate_patternProperties_fails(self):
-        detail = "Additional properties are not allowed ('__' was unexpected)"
+        details = [
+            "Additional properties are not allowed ('__' was unexpected)",
+            "'__' does not match any of the regexes: '^[a-zA-Z0-9]{1,10}$'"
+        ]
         self.check_validation_error(self.post, body={'__': 'bar'},
-                                    expected_detail=detail)
+                                    expected_detail=details)
 
-        detail = "Additional properties are not allowed ('' was unexpected)"
+        details = [
+            "'' does not match any of the regexes: '^[a-zA-Z0-9]{1,10}$'",
+            "Additional properties are not allowed ('' was unexpected)"
+        ]
         self.check_validation_error(self.post, body={'': 'bar'},
-                                    expected_detail=detail)
+                                    expected_detail=details)
 
-        detail = ("Additional properties are not allowed ('0123456789a' was"
-                  " unexpected)")
+        details = [
+            ("'0123456789a' does not match any of the regexes: "
+                  "'^[a-zA-Z0-9]{1,10}$'"),
+            ("Additional properties are not allowed ('0123456789a' was"
+             " unexpected)")
+        ]
         self.check_validation_error(self.post, body={'0123456789a': 'bar'},
-                                    expected_detail=detail)
+                                    expected_detail=details)
 
-        detail = "expected string or buffer"
-        self.check_validation_error(self.post, body={None: 'bar'},
-                                    expected_detail=detail)
+        # Note(jrosenboom): This is referencing an internal python error
+        # string, which is no stable interface. We need a patch in the
+        # jsonschema library in order to fix this properly.
+        self.check_validation_error(
+                self.post, body={None: 'bar'},
+                expected_detail="expected string or bytes-like object")
 
 
 class StringTestCase(APIValidationTestCase):
 
-    def setUp(self):
-        super(StringTestCase, self).setUp()
-        schema = {
-            'type': 'object',
-            'properties': {
-                'foo': {
-                    'type': 'string',
-                },
+    post_schema = {
+        'type': 'object',
+        'properties': {
+            'foo': {
+                'type': 'string',
             },
-        }
-
-        @validation.schema(request_body_schema=schema)
-        def post(req, body):
-            return 'Validation succeeded.'
-
-        self.post = post
+        },
+    }
 
     def test_validate_string(self):
         self.assertEqual(self.post(body={'foo': 'abc'}, req=FakeRequest()),
@@ -405,24 +482,16 @@ class StringTestCase(APIValidationTestCase):
 
 class StringLengthTestCase(APIValidationTestCase):
 
-    def setUp(self):
-        super(StringLengthTestCase, self).setUp()
-        schema = {
-            'type': 'object',
-            'properties': {
-                'foo': {
-                    'type': 'string',
-                    'minLength': 1,
-                    'maxLength': 10,
-                },
+    post_schema = {
+        'type': 'object',
+        'properties': {
+            'foo': {
+                'type': 'string',
+                'minLength': 1,
+                'maxLength': 10,
             },
-        }
-
-        @validation.schema(request_body_schema=schema)
-        def post(req, body):
-            return 'Validation succeeded.'
-
-        self.post = post
+        },
+    }
 
     def test_validate_string_length(self):
         self.assertEqual(self.post(body={'foo': '0'}, req=FakeRequest()),
@@ -445,23 +514,15 @@ class StringLengthTestCase(APIValidationTestCase):
 
 class IntegerTestCase(APIValidationTestCase):
 
-    def setUp(self):
-        super(IntegerTestCase, self).setUp()
-        schema = {
-            'type': 'object',
-            'properties': {
-                'foo': {
-                    'type': ['integer', 'string'],
-                    'pattern': '^[0-9]+$',
-                },
+    post_schema = {
+        'type': 'object',
+        'properties': {
+            'foo': {
+                'type': ['integer', 'string'],
+                'pattern': '^[0-9]+$',
             },
-        }
-
-        @validation.schema(request_body_schema=schema)
-        def post(req, body):
-            return 'Validation succeeded.'
-
-        self.post = post
+        },
+    }
 
     def test_validate_integer(self):
         self.assertEqual(self.post(body={'foo': 1}, req=FakeRequest()),
@@ -501,25 +562,17 @@ class IntegerTestCase(APIValidationTestCase):
 
 class IntegerRangeTestCase(APIValidationTestCase):
 
-    def setUp(self):
-        super(IntegerRangeTestCase, self).setUp()
-        schema = {
-            'type': 'object',
-            'properties': {
-                'foo': {
-                    'type': ['integer', 'string'],
-                    'pattern': '^[0-9]+$',
-                    'minimum': 1,
-                    'maximum': 10,
-                },
+    post_schema = {
+        'type': 'object',
+        'properties': {
+            'foo': {
+                'type': ['integer', 'string'],
+                'pattern': '^[0-9]+$',
+                'minimum': 1,
+                'maximum': 10,
             },
-        }
-
-        @validation.schema(request_body_schema=schema)
-        def post(req, body):
-            return 'Validation succeeded.'
-
-        self.post = post
+        },
+    }
 
     def test_validate_integer_range(self):
         self.assertEqual(self.post(body={'foo': 1}, req=FakeRequest()),
@@ -553,20 +606,12 @@ class IntegerRangeTestCase(APIValidationTestCase):
 
 class BooleanTestCase(APIValidationTestCase):
 
-    def setUp(self):
-        super(BooleanTestCase, self).setUp()
-        schema = {
-            'type': 'object',
-            'properties': {
-                'foo': parameter_types.boolean,
-            },
-        }
-
-        @validation.schema(request_body_schema=schema)
-        def post(req, body):
-            return 'Validation succeeded.'
-
-        self.post = post
+    post_schema = {
+        'type': 'object',
+        'properties': {
+            'foo': parameter_types.boolean,
+        },
+    }
 
     def test_validate_boolean(self):
         self.assertEqual('Validation succeeded.',
@@ -601,20 +646,12 @@ class BooleanTestCase(APIValidationTestCase):
 
 class HostnameTestCase(APIValidationTestCase):
 
-    def setUp(self):
-        super(HostnameTestCase, self).setUp()
-        schema = {
-            'type': 'object',
-            'properties': {
-                'foo': parameter_types.hostname,
-            },
-        }
-
-        @validation.schema(request_body_schema=schema)
-        def post(req, body):
-            return 'Validation succeeded.'
-
-        self.post = post
+    post_schema = {
+        'type': 'object',
+        'properties': {
+            'foo': parameter_types.hostname,
+        },
+    }
 
     def test_validate_hostname(self):
         self.assertEqual('Validation succeeded.',
@@ -647,20 +684,12 @@ class HostnameTestCase(APIValidationTestCase):
 
 class HostnameIPaddressTestCase(APIValidationTestCase):
 
-    def setUp(self):
-        super(HostnameIPaddressTestCase, self).setUp()
-        schema = {
-            'type': 'object',
-            'properties': {
-                'foo': parameter_types.hostname_or_ip_address,
-            },
-        }
-
-        @validation.schema(request_body_schema=schema)
-        def post(req, body):
-            return 'Validation succeeded.'
-
-        self.post = post
+    post_schema = {
+        'type': 'object',
+        'properties': {
+            'foo': parameter_types.hostname_or_ip_address,
+        },
+    }
 
     def test_validate_hostname_or_ip_address(self):
         self.assertEqual('Validation succeeded.',
@@ -697,138 +726,14 @@ class HostnameIPaddressTestCase(APIValidationTestCase):
                                     expected_detail=detail)
 
 
-class CellNameTestCase(APIValidationTestCase):
-
-    def setUp(self):
-        super(CellNameTestCase, self).setUp()
-        schema = {
-            'type': 'object',
-            'properties': {
-                'foo': parameter_types.cell_name,
-            },
-        }
-
-        @validation.schema(request_body_schema=schema)
-        def post(req, body):
-            return 'Validation succeeded.'
-
-        self.post = post
-
-    def test_validate_name(self):
-        self.assertEqual('Validation succeeded.',
-                         self.post(body={'foo': 'abc'},
-                                   req=FakeRequest()))
-        self.assertEqual('Validation succeeded.',
-                         self.post(body={'foo': 'my server'},
-                                   req=FakeRequest()))
-        self.assertEqual('Validation succeeded.',
-                         self.post(body={'foo': u'\u0434'}, req=FakeRequest()))
-        self.assertEqual('Validation succeeded.',
-                         self.post(body={'foo': u'\u0434\u2006\ufffd'},
-                                   req=FakeRequest()))
-
-    def test_validate_name_fails(self):
-        error = ("An invalid 'name' value was provided. The name must be: "
-                 "printable characters except !, ., @. "
-                 "Can not start or end with whitespace.")
-
-        should_fail = (' ',
-                       ' server',
-                       'server ',
-                       u'a\xa0',  # trailing unicode space
-                       u'\uffff',  # non-printable unicode
-                       'abc!def',
-                       'abc.def',
-                       'abc@def')
-
-        for item in should_fail:
-            self.check_validation_error(self.post, body={'foo': item},
-                                    expected_detail=error)
-
-        # four-byte unicode, if supported by this python build
-        try:
-            self.check_validation_error(self.post, body={'foo': u'\U00010000'},
-                                        expected_detail=error)
-        except ValueError:
-            pass
-
-
-class CellNameLeadingTrailingSpacesTestCase(APIValidationTestCase):
-
-    def setUp(self):
-        super(CellNameLeadingTrailingSpacesTestCase, self).setUp()
-        schema = {
-            'type': 'object',
-            'properties': {
-                'foo': parameter_types.cell_name_leading_trailing_spaces,
-            },
-        }
-
-        @validation.schema(request_body_schema=schema)
-        def post(req, body):
-            return 'Validation succeeded.'
-
-        self.post = post
-
-    def test_validate_name(self):
-        self.assertEqual('Validation succeeded.',
-                         self.post(body={'foo': 'abc'},
-                                   req=FakeRequest()))
-        self.assertEqual('Validation succeeded.',
-                         self.post(body={'foo': 'my server'},
-                                   req=FakeRequest()))
-        self.assertEqual('Validation succeeded.',
-                         self.post(body={'foo': u'\u0434'}, req=FakeRequest()))
-        self.assertEqual('Validation succeeded.',
-                         self.post(body={'foo': u'\u0434\u2006\ufffd'},
-                                   req=FakeRequest()))
-        self.assertEqual('Validation succeeded.',
-                         self.post(body={'foo': '  my server'},
-                                   req=FakeRequest()))
-        self.assertEqual('Validation succeeded.',
-                         self.post(body={'foo': 'my server  '},
-                                   req=FakeRequest()))
-
-    def test_validate_name_fails(self):
-        error = ("An invalid 'name' value was provided. The name must be: "
-                 "printable characters except !, ., @, "
-                 "with at least one non space character")
-
-        should_fail = (
-            ' ',
-            u'\uffff',  # non-printable unicode
-            'abc!def',
-            'abc.def',
-            'abc@def')
-
-        for item in should_fail:
-            self.check_validation_error(self.post, body={'foo': item},
-                                    expected_detail=error)
-
-        # four-byte unicode, if supported by this python build
-        try:
-            self.check_validation_error(self.post, body={'foo': u'\U00010000'},
-                                        expected_detail=error)
-        except ValueError:
-            pass
-
-
 class NameTestCase(APIValidationTestCase):
 
-    def setUp(self):
-        super(NameTestCase, self).setUp()
-        schema = {
-            'type': 'object',
-            'properties': {
-                'foo': parameter_types.name,
-            },
-        }
-
-        @validation.schema(request_body_schema=schema)
-        def post(req, body):
-            return 'Validation succeeded.'
-
-        self.post = post
+    post_schema = {
+        'type': 'object',
+        'properties': {
+            'foo': parameter_types.name,
+        },
+    }
 
     def test_validate_name(self):
         self.assertEqual('Validation succeeded.',
@@ -871,20 +776,12 @@ class NameTestCase(APIValidationTestCase):
 
 class NameWithLeadingTrailingSpacesTestCase(APIValidationTestCase):
 
-    def setUp(self):
-        super(NameWithLeadingTrailingSpacesTestCase, self).setUp()
-        schema = {
-            'type': 'object',
-            'properties': {
-                'foo': parameter_types.name_with_leading_trailing_spaces,
-            },
-        }
-
-        @validation.schema(request_body_schema=schema)
-        def post(req, body):
-            return 'Validation succeeded.'
-
-        self.post = post
+    post_schema = {
+        'type': 'object',
+        'properties': {
+            'foo': parameter_types.name_with_leading_trailing_spaces,
+        },
+    }
 
     def test_validate_name(self):
         self.assertEqual('Validation succeeded.',
@@ -938,20 +835,12 @@ class NameWithLeadingTrailingSpacesTestCase(APIValidationTestCase):
 
 class NoneTypeTestCase(APIValidationTestCase):
 
-    def setUp(self):
-        super(NoneTypeTestCase, self).setUp()
-        schema = {
-            'type': 'object',
-            'properties': {
-                'foo': parameter_types.none
-            }
+    post_schema = {
+        'type': 'object',
+        'properties': {
+            'foo': parameter_types.none
         }
-
-        @validation.schema(request_body_schema=schema)
-        def post(req, body):
-            return 'Validation succeeded.'
-
-        self.post = post
+    }
 
     def test_validate_none(self):
         self.assertEqual('Validation succeeded.',
@@ -977,22 +866,51 @@ class NoneTypeTestCase(APIValidationTestCase):
                                     expected_detail=detail)
 
 
+class NameOrNoneTestCase(APIValidationTestCase):
+
+    post_schema = {
+        'type': 'object',
+        'properties': {
+            'foo': parameter_types.name_or_none
+        }
+    }
+
+    def test_valid(self):
+        self.assertEqual('Validation succeeded.',
+                         self.post(body={'foo': None},
+                                   req=FakeRequest()))
+        self.assertEqual('Validation succeeded.',
+                         self.post(body={'foo': '1'},
+                                   req=FakeRequest()))
+
+    def test_validate_fails(self):
+        detail = ("Invalid input for field/attribute foo. Value: 1234. 1234 "
+                  "is not valid under any of the given schemas")
+        self.check_validation_error(self.post, body={'foo': 1234},
+                                    expected_detail=detail)
+
+        detail = ("Invalid input for field/attribute foo. Value: . '' "
+                  "is not valid under any of the given schemas")
+        self.check_validation_error(self.post, body={'foo': ''},
+                                    expected_detail=detail)
+
+        too_long_name = 256 * "k"
+        detail = ("Invalid input for field/attribute foo. Value: %s. "
+                  "'%s' is not valid under any of the "
+                  "given schemas") % (too_long_name, too_long_name)
+        self.check_validation_error(self.post,
+                                    body={'foo': too_long_name},
+                                    expected_detail=detail)
+
+
 class TcpUdpPortTestCase(APIValidationTestCase):
 
-    def setUp(self):
-        super(TcpUdpPortTestCase, self).setUp()
-        schema = {
-            'type': 'object',
-            'properties': {
-                'foo': parameter_types.tcp_udp_port,
-            },
-        }
-
-        @validation.schema(request_body_schema=schema)
-        def post(req, body):
-            return 'Validation succeeded.'
-
-        self.post = post
+    post_schema = {
+        'type': 'object',
+        'properties': {
+            'foo': parameter_types.tcp_udp_port,
+        },
+    }
 
     def test_validate_tcp_udp_port(self):
         self.assertEqual('Validation succeeded.',
@@ -1014,23 +932,15 @@ class TcpUdpPortTestCase(APIValidationTestCase):
 
 class CidrFormatTestCase(APIValidationTestCase):
 
-    def setUp(self):
-        super(CidrFormatTestCase, self).setUp()
-        schema = {
-            'type': 'object',
-            'properties': {
-                'foo': {
-                    'type': 'string',
-                    'format': 'cidr',
-                },
+    post_schema = {
+        'type': 'object',
+        'properties': {
+            'foo': {
+                'type': 'string',
+                'format': 'cidr',
             },
-        }
-
-        @validation.schema(request_body_schema=schema)
-        def post(req, body):
-            return 'Validation succeeded.'
-
-        self.post = post
+        },
+    }
 
     def test_validate_cidr(self):
         self.assertEqual('Validation succeeded.',
@@ -1066,23 +976,15 @@ class CidrFormatTestCase(APIValidationTestCase):
 
 class DatetimeTestCase(APIValidationTestCase):
 
-    def setUp(self):
-        super(DatetimeTestCase, self).setUp()
-        schema = {
-            'type': 'object',
-            'properties': {
-                'foo': {
-                    'type': 'string',
-                    'format': 'date-time',
-                },
+    post_schema = {
+        'type': 'object',
+        'properties': {
+            'foo': {
+                'type': 'string',
+                'format': 'date-time',
             },
-        }
-
-        @validation.schema(request_body_schema=schema)
-        def post(req, body):
-            return 'Validation succeeded.'
-
-        self.post = post
+        },
+    }
 
     def test_validate_datetime(self):
         self.assertEqual('Validation succeeded.',
@@ -1112,23 +1014,15 @@ class DatetimeTestCase(APIValidationTestCase):
 
 class UuidTestCase(APIValidationTestCase):
 
-    def setUp(self):
-        super(UuidTestCase, self).setUp()
-        schema = {
-            'type': 'object',
-            'properties': {
-                'foo': {
-                    'type': 'string',
-                    'format': 'uuid',
-                },
+    post_schema = {
+        'type': 'object',
+        'properties': {
+            'foo': {
+                'type': 'string',
+                'format': 'uuid',
             },
-        }
-
-        @validation.schema(request_body_schema=schema)
-        def post(req, body):
-            return 'Validation succeeded.'
-
-        self.post = post
+        },
+    }
 
     def test_validate_uuid(self):
         self.assertEqual('Validation succeeded.',
@@ -1158,23 +1052,15 @@ class UuidTestCase(APIValidationTestCase):
 
 class UriTestCase(APIValidationTestCase):
 
-    def setUp(self):
-        super(UriTestCase, self).setUp()
-        schema = {
-            'type': 'object',
-            'properties': {
-                'foo': {
-                    'type': 'string',
-                    'format': 'uri',
-                },
+    post_schema = {
+        'type': 'object',
+        'properties': {
+            'foo': {
+                'type': 'string',
+                'format': 'uri',
             },
-        }
-
-        @validation.schema(request_body_schema=schema)
-        def post(req, body):
-            return 'Validation succeeded.'
-
-        self.post = post
+        },
+    }
 
     def test_validate_uri(self):
         self.assertEqual('Validation succeeded.',
@@ -1218,23 +1104,15 @@ class UriTestCase(APIValidationTestCase):
 
 class Ipv4TestCase(APIValidationTestCase):
 
-    def setUp(self):
-        super(Ipv4TestCase, self).setUp()
-        schema = {
-            'type': 'object',
-            'properties': {
-                'foo': {
-                    'type': 'string',
-                    'format': 'ipv4',
-                },
+    post_schema = {
+        'type': 'object',
+        'properties': {
+            'foo': {
+                'type': 'string',
+                'format': 'ipv4',
             },
-        }
-
-        @validation.schema(request_body_schema=schema)
-        def post(req, body):
-            return 'Validation succeeded.'
-
-        self.post = post
+        },
+    }
 
     def test_validate_ipv4(self):
         self.assertEqual('Validation succeeded.',
@@ -1264,23 +1142,15 @@ class Ipv4TestCase(APIValidationTestCase):
 
 class Ipv6TestCase(APIValidationTestCase):
 
-    def setUp(self):
-        super(Ipv6TestCase, self).setUp()
-        schema = {
-            'type': 'object',
-            'properties': {
-                'foo': {
-                    'type': 'string',
-                    'format': 'ipv6',
-                },
+    post_schema = {
+        'type': 'object',
+        'properties': {
+            'foo': {
+                'type': 'string',
+                'format': 'ipv6',
             },
-        }
-
-        @validation.schema(request_body_schema=schema)
-        def post(req, body):
-            return 'Validation succeeded.'
-
-        self.post = post
+        },
+    }
 
     def test_validate_ipv6(self):
         self.assertEqual('Validation succeeded.',
@@ -1308,23 +1178,15 @@ class Ipv6TestCase(APIValidationTestCase):
 
 class Base64TestCase(APIValidationTestCase):
 
-    def setUp(self):
-        super(APIValidationTestCase, self).setUp()
-        schema = {
-            'type': 'object',
-            'properties': {
-                'foo': {
-                    'type': 'string',
-                    'format': 'base64',
-                },
+    post_schema = {
+        'type': 'object',
+        'properties': {
+            'foo': {
+               'type': 'string',
+                'format': 'base64',
             },
-        }
-
-        @validation.schema(request_body_schema=schema)
-        def post(req, body):
-            return 'Validation succeeded.'
-
-        self.post = post
+        },
+    }
 
     def test_validate_base64(self):
         self.assertEqual('Validation succeeded.',
@@ -1338,3 +1200,24 @@ class Base64TestCase(APIValidationTestCase):
                   "Value: %s. '%s' is not a 'base64'") % (value, value)
         self.check_validation_error(self.post, body={'foo': value},
                                     expected_detail=detail)
+
+
+class RegexFormatTestCase(APIValidationTestCase):
+
+    def setUp(self):
+        super(RegexFormatTestCase, self).setUp()
+        self.controller = RegexFormatFakeController()
+
+    def test_validate_regex(self):
+        req = fakes.HTTPRequest.blank("")
+        self.assertEqual('Validation succeeded.',
+                         self.controller.post(req, body={'foo': u'Myserver'}))
+
+    def test_validate_regex_fails(self):
+        value = 1
+        req = fakes.HTTPRequest.blank("")
+        detail = ("Invalid input for field/attribute foo. "
+                  "Value: %s. %s is not a 'regex'") % (value, value)
+        self.check_validation_error(self.controller.post, req=req,
+                                    body={'foo': value},
+                                    expected_detail=detail)
