@@ -14,90 +14,41 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import sqlalchemy
-import sys
+import copy
+import datetime
+import io
 
 import fixtures as fx
+import futurist
 import mock
 from oslo_config import cfg
+from oslo_db import exception as db_exc
 from oslo_log import log as logging
+from oslo_utils.fixture import uuidsentinel as uuids
+from oslo_utils import timeutils
 from oslo_utils import uuidutils
+from oslotest import output
+import sqlalchemy
 import testtools
 
 from nova.compute import rpcapi as compute_rpcapi
 from nova import conductor
+from nova import context
 from nova.db.sqlalchemy import api as session
 from nova import exception
+from nova.network import neutron as neutron_api
+from nova import objects
 from nova.objects import base as obj_base
 from nova.objects import service as service_obj
+from nova import test
 from nova.tests import fixtures
 from nova.tests.unit import conf_fixture
+from nova.tests.unit import fake_instance
 from nova import utils
 
 CONF = cfg.CONF
 
 
-class TestConfFixture(testtools.TestCase):
-    """Test the Conf fixtures in Nova.
-
-    This is a basic test that this fixture works like we expect.
-
-    Expectations:
-
-    1. before using the fixture, a default value (api_paste_config)
-       comes through untouched.
-
-    2. before using the fixture, a known default value that we
-       override is correct.
-
-    3. after using the fixture a known value that we override is the
-       new value.
-
-    4. after using the fixture we can set a default value to something
-       random, and it will be reset once we are done.
-
-    There are 2 copies of this test so that you can verify they do the
-    right thing with:
-
-       tox -e py27 test_fixtures -- --concurrency=1
-
-    As regardless of run order, their initial asserts would be
-    impacted if the reset behavior isn't working correctly.
-
-    """
-    def _test_override(self):
-        self.assertEqual('api-paste.ini', CONF.wsgi.api_paste_config)
-        self.assertFalse(CONF.fake_network)
-        self.useFixture(conf_fixture.ConfFixture())
-        CONF.set_default('api_paste_config', 'foo', group='wsgi')
-        self.assertTrue(CONF.fake_network)
-
-    def test_override1(self):
-        self._test_override()
-
-    def test_override2(self):
-        self._test_override()
-
-
-class TestOutputStream(testtools.TestCase):
-    """Ensure Output Stream capture works as expected.
-
-    This has the added benefit of providing a code example of how you
-    can manipulate the output stream in your own tests.
-    """
-    def test_output(self):
-        self.useFixture(fx.EnvironmentVariable('OS_STDOUT_CAPTURE', '1'))
-        self.useFixture(fx.EnvironmentVariable('OS_STDERR_CAPTURE', '1'))
-
-        out = self.useFixture(fixtures.OutputStreamCapture())
-        sys.stdout.write("foo")
-        sys.stderr.write("bar")
-        self.assertEqual("foo", out.stdout)
-        self.assertEqual("bar", out.stderr)
-        # TODO(sdague): nuke the out and err buffers so it doesn't
-        # make it to testr
-
-
 class TestLogging(testtools.TestCase):
     def test_default_logging(self):
         stdlog = self.useFixture(fixtures.StandardLogging())
@@ -136,33 +87,12 @@ class TestLogging(testtools.TestCase):
         self.assertIn("at debug", stdlog.logger.output)
 
 
-class TestTimeout(testtools.TestCase):
-    """Tests for our timeout fixture.
-
-    Testing the actual timeout mechanism is beyond the scope of this
-    test, because it's a pretty clear pass through to fixtures'
-    timeout fixture, which tested in their tree.
-
-    """
-    def test_scaling(self):
-        # a bad scaling factor
-        self.assertRaises(ValueError, fixtures.Timeout, 1, 0.5)
-
-        # various things that should work.
-        timeout = fixtures.Timeout(10)
-        self.assertEqual(10, timeout.test_timeout)
-        timeout = fixtures.Timeout("10")
-        self.assertEqual(10, timeout.test_timeout)
-        timeout = fixtures.Timeout("10", 2)
-        self.assertEqual(20, timeout.test_timeout)
-
-
 class TestOSAPIFixture(testtools.TestCase):
     @mock.patch('nova.objects.Service.get_by_host_and_binary')
     @mock.patch('nova.objects.Service.create')
     def test_responds_to_version(self, mock_service_create, mock_get):
         """Ensure the OSAPI server responds to calls sensibly."""
-        self.useFixture(fixtures.OutputStreamCapture())
+        self.useFixture(output.CaptureOutput())
         self.useFixture(fixtures.StandardLogging())
         self.useFixture(conf_fixture.ConfFixture())
         self.useFixture(fixtures.RPCFixture('nova.test'))
@@ -286,13 +216,11 @@ class TestDatabaseFixture(testtools.TestCase):
         schema = "".join(line for line in conn.connection.iterdump())
         self.assertEqual("BEGIN TRANSACTION;COMMIT;", schema)
 
-
-class TestDatabaseAtVersionFixture(testtools.TestCase):
     def test_fixture_schema_version(self):
         self.useFixture(conf_fixture.ConfFixture())
 
         # In/after 317 aggregates did have uuid
-        self.useFixture(fixtures.DatabaseAtVersion(318))
+        self.useFixture(fixtures.Database(version=318))
         engine = session.get_engine()
         engine.connect()
         meta = sqlalchemy.MetaData(engine)
@@ -300,7 +228,7 @@ class TestDatabaseAtVersionFixture(testtools.TestCase):
         self.assertTrue(hasattr(aggregate.c, 'uuid'))
 
         # Before 317, aggregates had no uuid
-        self.useFixture(fixtures.DatabaseAtVersion(316))
+        self.useFixture(fixtures.Database(version=316))
         engine = session.get_engine()
         engine.connect()
         meta = sqlalchemy.MetaData(engine)
@@ -311,11 +239,12 @@ class TestDatabaseAtVersionFixture(testtools.TestCase):
     def test_fixture_after_database_fixture(self):
         self.useFixture(conf_fixture.ConfFixture())
         self.useFixture(fixtures.Database())
-        self.useFixture(fixtures.DatabaseAtVersion(318))
+        self.useFixture(fixtures.Database(version=318))
 
 
 class TestDefaultFlavorsFixture(testtools.TestCase):
-    def test_flavors(self):
+    @mock.patch("nova.objects.flavor.Flavor._send_notification")
+    def test_flavors(self, mock_send_notification):
         self.useFixture(conf_fixture.ConfFixture())
         self.useFixture(fixtures.Database())
         self.useFixture(fixtures.Database(database='api'))
@@ -330,7 +259,7 @@ class TestDefaultFlavorsFixture(testtools.TestCase):
 
         result = conn.execute("select * from flavors")
         rows = result.fetchall()
-        self.assertEqual(5, len(rows), "Rows %s" % rows)
+        self.assertEqual(6, len(rows), "Rows %s" % rows)
 
 
 class TestIndirectionAPIFixture(testtools.TestCase):
@@ -405,6 +334,17 @@ class TestSpawnIsSynchronousFixture(testtools.TestCase):
         self.assertEqual(1, len(call_count))
 
 
+class TestSynchronousThreadPoolExecutorFixture(testtools.TestCase):
+    def test_submit_passes_through(self):
+        self.useFixture(fixtures.SynchronousThreadPoolExecutorFixture())
+        tester = mock.MagicMock()
+        executor = futurist.GreenThreadPoolExecutor()
+        future = executor.submit(tester.function, 'foo', bar='bar')
+        tester.function.assert_called_once_with('foo', bar='bar')
+        result = future.result()
+        self.assertEqual(tester.function.return_value, result)
+
+
 class TestBannedDBSchemaOperations(testtools.TestCase):
     def test_column(self):
         column = sqlalchemy.Column()
@@ -453,3 +393,274 @@ class TestNoopConductorFixture(testtools.TestCase):
         self.useFixture(fixtures.NoopConductorFixture())
         conductor.API().wait_until_ready()
         self.assertFalse(mock_wait.called)
+
+
+class TestSingleCellSimpleFixture(testtools.TestCase):
+    def test_single_cell(self):
+        self.useFixture(fixtures.SingleCellSimple())
+        cml = objects.CellMappingList.get_all(None)
+        self.assertEqual(1, len(cml))
+
+    def test_target_cell(self):
+        self.useFixture(fixtures.SingleCellSimple())
+        with context.target_cell(mock.sentinel.context, None) as c:
+            self.assertIs(mock.sentinel.context, c)
+
+
+class TestWarningsFixture(test.TestCase):
+    def test_invalid_uuid_errors(self):
+        """Creating an oslo.versionedobject with an invalid UUID value for a
+        UUIDField should raise an exception.
+        """
+        valid_migration_kwargs = {
+                "created_at": timeutils.utcnow().replace(microsecond=0),
+                "updated_at": None,
+                "deleted_at": None,
+                "deleted": False,
+                "id": 123,
+                "uuid": uuids.migration,
+                "source_compute": "compute-source",
+                "dest_compute": "compute-dest",
+                "source_node": "node-source",
+                "dest_node": "node-dest",
+                "dest_host": "host-dest",
+                "old_instance_type_id": 42,
+                "new_instance_type_id": 84,
+                "instance_uuid": "fake-uuid",
+                "status": "migrating",
+                "migration_type": "resize",
+                "hidden": False,
+                "memory_total": 123456,
+                "memory_processed": 12345,
+                "memory_remaining": 111111,
+                "disk_total": 234567,
+                "disk_processed": 23456,
+                "disk_remaining": 211111,
+                }
+
+        # this shall not throw FutureWarning
+        objects.migration.Migration(**valid_migration_kwargs)
+
+        invalid_migration_kwargs = copy.deepcopy(valid_migration_kwargs)
+        invalid_migration_kwargs["uuid"] = "fake_id"
+        self.assertRaises(FutureWarning, objects.migration.Migration,
+                          **invalid_migration_kwargs)
+
+
+class TestDownCellFixture(test.TestCase):
+
+    def test_fixture(self):
+        # The test setup creates two cell mappings (cell0 and cell1) by
+        # default. Let's first list servers across all cells while they are
+        # "up" to make sure that works as expected. We'll create a single
+        # instance in cell1.
+        ctxt = context.get_admin_context()
+        cell1 = self.cell_mappings[test.CELL1_NAME]
+        with context.target_cell(ctxt, cell1) as cctxt:
+            inst = fake_instance.fake_instance_obj(cctxt)
+            if 'id' in inst:
+                delattr(inst, 'id')
+            inst.create()
+
+        # Now list all instances from all cells (should get one back).
+        results = context.scatter_gather_all_cells(
+            ctxt, objects.InstanceList.get_all)
+        self.assertEqual(2, len(results))
+        self.assertEqual(0, len(results[objects.CellMapping.CELL0_UUID]))
+        self.assertEqual(1, len(results[cell1.uuid]))
+
+        # Now do the same but with the DownCellFixture which should result
+        # in exception results from both cells.
+        with fixtures.DownCellFixture():
+            results = context.scatter_gather_all_cells(
+                ctxt, objects.InstanceList.get_all)
+            self.assertEqual(2, len(results))
+            for result in results.values():
+                self.assertIsInstance(result, db_exc.DBError)
+
+    def test_fixture_when_explicitly_passing_down_cell_mappings(self):
+        # The test setup creates two cell mappings (cell0 and cell1) by
+        # default. We'll create one instance per cell and pass cell0 as
+        # the down cell. We should thus get db_exc.DBError for cell0 and
+        # correct InstanceList object from cell1.
+        ctxt = context.get_admin_context()
+        cell0 = self.cell_mappings['cell0']
+        cell1 = self.cell_mappings['cell1']
+        with context.target_cell(ctxt, cell0) as cctxt:
+            inst1 = fake_instance.fake_instance_obj(cctxt)
+            if 'id' in inst1:
+                delattr(inst1, 'id')
+            inst1.create()
+        with context.target_cell(ctxt, cell1) as cctxt:
+            inst2 = fake_instance.fake_instance_obj(cctxt)
+            if 'id' in inst2:
+                delattr(inst2, 'id')
+            inst2.create()
+        with fixtures.DownCellFixture([cell0]):
+            results = context.scatter_gather_all_cells(
+                ctxt, objects.InstanceList.get_all)
+            self.assertEqual(2, len(results))
+            for cell_uuid, result in results.items():
+                if cell_uuid == cell0.uuid:
+                    self.assertIsInstance(result, db_exc.DBError)
+                else:
+                    self.assertIsInstance(result, objects.InstanceList)
+                    self.assertEqual(1, len(result))
+                    self.assertEqual(inst2.uuid, result[0].uuid)
+
+    def test_fixture_for_an_individual_down_cell_targeted_call(self):
+        # We have cell0 and cell1 by default in the setup. We try targeting
+        # both the cells. We should get a db error for the down cell and
+        # the correct result for the up cell.
+        ctxt = context.get_admin_context()
+        cell0 = self.cell_mappings['cell0']
+        cell1 = self.cell_mappings['cell1']
+        with context.target_cell(ctxt, cell0) as cctxt:
+            inst1 = fake_instance.fake_instance_obj(cctxt)
+            if 'id' in inst1:
+                delattr(inst1, 'id')
+            inst1.create()
+        with context.target_cell(ctxt, cell1) as cctxt:
+            inst2 = fake_instance.fake_instance_obj(cctxt)
+            if 'id' in inst2:
+                delattr(inst2, 'id')
+            inst2.create()
+
+        def dummy_tester(ctxt, cell_mapping, uuid):
+            with context.target_cell(ctxt, cell_mapping) as cctxt:
+                return objects.Instance.get_by_uuid(cctxt, uuid)
+
+        # Scenario A: We do not pass any down cells, fixture automatically
+        # assumes the targeted cell is down whether its cell0 or cell1.
+        with fixtures.DownCellFixture():
+            self.assertRaises(
+                db_exc.DBError, dummy_tester, ctxt, cell1, inst2.uuid)
+        # Scenario B: We pass cell0 as the down cell.
+        with fixtures.DownCellFixture([cell0]):
+            self.assertRaises(
+                db_exc.DBError, dummy_tester, ctxt, cell0, inst1.uuid)
+            # Scenario C: We get the correct result from the up cell
+            # when targeted.
+            result = dummy_tester(ctxt, cell1, inst2.uuid)
+            self.assertEqual(inst2.uuid, result.uuid)
+
+
+class TestNeutronFixture(test.NoDBTestCase):
+
+    def setUp(self):
+        super(TestNeutronFixture, self).setUp()
+        self.neutron = self.useFixture(fixtures.NeutronFixture(self))
+
+    def test_list_ports_with_resource_request_non_admin_client(self):
+        ctxt = context.get_context()
+        client = neutron_api.get_client(ctxt)
+        ports = client.list_ports(ctxt)['ports']
+        port_id = self.neutron.port_with_resource_request['id']
+        ports = [port for port in ports if port_id == port['id']]
+        self.assertIsNone(ports[0]['resource_request'])
+
+    def test_list_ports_with_resource_request_admin_client(self):
+        ctxt = context.get_admin_context()
+        client = neutron_api.get_client(ctxt)
+        ports = client.list_ports(ctxt)['ports']
+        port_id = self.neutron.port_with_resource_request['id']
+        ports = [port for port in ports if port_id == port['id']]
+        self.assertIsNotNone(ports[0]['resource_request'])
+
+
+class TestGlanceFixture(test.NoDBTestCase):
+    def setUp(self):
+        super().setUp()
+        self.image_service = self.useFixture(fixtures.GlanceFixture(self))
+        self.context = context.get_admin_context()
+
+    def test_detail(self):
+        res = self.image_service.detail(self.context)
+        for image in res:
+            self.assertEqual(
+                set(image.keys()),
+                {
+                    'id', 'name', 'created_at', 'updated_at', 'deleted_at',
+                    'deleted', 'status', 'is_public', 'properties',
+                    'disk_format', 'container_format', 'size', 'min_disk',
+                    'min_ram', 'protected', 'tags', 'visibility',
+                },
+            )
+            self.assertIsInstance(image['created_at'], datetime.datetime)
+            self.assertIsInstance(image['updated_at'], datetime.datetime)
+
+            if not (
+                isinstance(image['deleted_at'], datetime.datetime) or
+                image['deleted_at'] is None
+            ):
+                self.fail(
+                    "image's 'deleted_at' attribute was neither a datetime "
+                    "object nor None"
+                )
+
+            def check_is_bool(image, key):
+                val = image.get('deleted')
+                if not isinstance(val, bool):
+                    self.fail(
+                        "image's '%s' attribute wasn't a bool: %r" % (key, val)
+                    )
+
+            check_is_bool(image, 'deleted')
+            check_is_bool(image, 'is_public')
+
+    def test_show_raises_imagenotfound_for_invalid_id(self):
+        self.assertRaises(
+            exception.ImageNotFound,
+            self.image_service.show,
+            self.context, 'this image does not exist')
+
+    def test_create_adds_id(self):
+        index = self.image_service.detail(self.context)
+        image_count = len(index)
+
+        self.image_service.create(self.context, {})
+
+        index = self.image_service.detail(self.context)
+        self.assertEqual(len(index), image_count + 1)
+        self.assertTrue(index[0]['id'])
+
+    def test_create_keeps_id(self):
+        self.image_service.create(self.context, {'id': '34'})
+        self.image_service.show(self.context, '34')
+
+    def test_create_rejects_duplicate_ids(self):
+        self.image_service.create(self.context, {'id': '34'})
+        self.assertRaises(
+            exception.CouldNotUploadImage,
+            self.image_service.create,
+            self.context, {'id': '34'})
+
+        # Make sure there's still one left
+        self.image_service.show(self.context, '34')
+
+    def test_update(self):
+        self.image_service.create(
+            self.context, {'id': '34', 'foo': 'bar'})
+
+        self.image_service.update(
+            self.context, '34', {'id': '34', 'foo': 'baz'})
+
+        img = self.image_service.show(self.context, '34')
+        self.assertEqual(img['foo'], 'baz')
+
+    def test_delete(self):
+        self.image_service.create(self.context, {'id': '34', 'foo': 'bar'})
+        self.image_service.delete(self.context, '34')
+        self.assertRaises(
+            exception.NotFound,
+            self.image_service.show,
+            self.context, '34')
+
+    def test_create_then_get(self):
+        blob = 'some data'
+        s1 = io.StringIO(blob)
+        self.image_service.create(
+            self.context, {'id': '32', 'foo': 'bar'}, data=s1)
+        s2 = io.StringIO()
+        self.image_service.download(self.context, '32', data=s2)
+        self.assertEqual(s2.getvalue(), blob, 'Did not get blob back intact')
