@@ -15,7 +15,7 @@
 import textwrap
 
 import mock
-import pep8
+import pycodestyle
 
 from nova.hacking import checks
 from nova import test
@@ -23,10 +23,10 @@ from nova import test
 
 class HackingTestCase(test.NoDBTestCase):
     """This class tests the hacking checks in nova.hacking.checks by passing
-    strings to the check methods like the pep8/flake8 parser would. The parser
-    loops over each line in the file and then passes the parameters to the
-    check method. The parameter names in the check method dictate what type of
-    object is passed to the check method. The parameter types are::
+    strings to the check methods like the pycodestyle/flake8 parser would. The
+    parser loops over each line in the file and then passes the parameters to
+    the check method. The parameter names in the check method dictate what type
+    of object is passed to the check method. The parameter types are::
 
         logical_line: A processed line with the following modifications:
             - Multi-line statements converted to a single line.
@@ -43,7 +43,7 @@ class HackingTestCase(test.NoDBTestCase):
         indent_level: indentation (with tabs expanded to multiples of 8)
         previous_indent_level: indentation on previous line
         previous_logical: previous logical line
-        filename: Path of the file being run through pep8
+        filename: Path of the file being run through pycodestyle
 
     When running a test on a check method the return will be False/None if
     there is no violation in the sample input. If there is an error a tuple is
@@ -57,55 +57,27 @@ class HackingTestCase(test.NoDBTestCase):
 
         self.assertEqual(expect, checks.import_no_virt_driver_import_deps(
             "from nova.virt.libvirt import utils as libvirt_utils",
-            "./nova/virt/xenapi/driver.py"))
+            "./nova/virt/hyperv/driver.py"))
 
         self.assertEqual(expect, checks.import_no_virt_driver_import_deps(
             "import nova.virt.libvirt.utils as libvirt_utils",
-            "./nova/virt/xenapi/driver.py"))
+            "./nova/virt/hyperv/driver.py"))
 
         self.assertIsNone(checks.import_no_virt_driver_import_deps(
             "from nova.virt.libvirt import utils as libvirt_utils",
             "./nova/virt/libvirt/driver.py"))
 
-        self.assertIsNone(checks.import_no_virt_driver_import_deps(
-            "import nova.virt.firewall",
-            "./nova/virt/libvirt/firewall.py"))
-
     def test_virt_driver_config_vars(self):
         self.assertIsInstance(checks.import_no_virt_driver_config_deps(
             "CONF.import_opt('volume_drivers', "
             "'nova.virt.libvirt.driver', group='libvirt')",
-            "./nova/virt/xenapi/driver.py"), tuple)
+            "./nova/virt/hyperv/driver.py"), tuple)
 
         self.assertIsNone(checks.import_no_virt_driver_config_deps(
             "CONF.import_opt('volume_drivers', "
             "'nova.virt.libvirt.driver', group='libvirt')",
             "./nova/virt/libvirt/volume.py"))
 
-    def test_no_vi_headers(self):
-
-        lines = ['Line 1\n', 'Line 2\n', 'Line 3\n', 'Line 4\n', 'Line 5\n',
-                 'Line 6\n', 'Line 7\n', 'Line 8\n', 'Line 9\n', 'Line 10\n',
-                 'Line 11\n', 'Line 12\n', 'Line 13\n', 'Line14\n', 'Line15\n']
-
-        self.assertIsNone(checks.no_vi_headers(
-            "Test string foo", 1, lines))
-        self.assertEqual(len(list(checks.no_vi_headers(
-            "# vim: et tabstop=4 shiftwidth=4 softtabstop=4",
-            2, lines))), 2)
-        self.assertIsNone(checks.no_vi_headers(
-            "# vim: et tabstop=4 shiftwidth=4 softtabstop=4",
-            6, lines))
-        self.assertIsNone(checks.no_vi_headers(
-            "# vim: et tabstop=4 shiftwidth=4 softtabstop=4",
-            9, lines))
-        self.assertEqual(len(list(checks.no_vi_headers(
-            "# vim: et tabstop=4 shiftwidth=4 softtabstop=4",
-            14, lines))), 2)
-        self.assertIsNone(checks.no_vi_headers(
-            "Test end string for vi",
-            15, lines))
-
     def test_assert_true_instance(self):
         self.assertEqual(len(list(checks.assert_true_instance(
             "self.assertTrue(isinstance(e, "
@@ -158,19 +130,7 @@ class HackingTestCase(test.NoDBTestCase):
         self.assertEqual(len(list(checks.assert_equal_in(
             "self.assertEqual(False, any(a==1 for a in b))"))), 0)
 
-    def test_assert_equal_none(self):
-        self.assertEqual(len(list(checks.assert_equal_none(
-            "self.assertEqual(A, None)"))), 1)
-
-        self.assertEqual(len(list(checks.assert_equal_none(
-            "self.assertEqual(None, A)"))), 1)
-
-        self.assertEqual(
-            len(list(checks.assert_equal_none("self.assertIsNone()"))), 0)
-
     def test_assert_true_or_false_with_in_or_not_in(self):
-        self.assertEqual(len(list(checks.assert_equal_none(
-            "self.assertEqual(A, None)"))), 1)
         self.assertEqual(len(list(checks.assert_true_or_false_with_in(
             "self.assertTrue(A in B)"))), 1)
 
@@ -198,9 +158,6 @@ class HackingTestCase(test.NoDBTestCase):
         self.assertEqual(len(list(checks.assert_true_or_false_with_in(
             "self.assertTrue(A in 'some string with spaces')"))), 1)
 
-        self.assertEqual(len(list(checks.assert_true_or_false_with_in(
-            "self.assertTrue(A in 'some string with spaces')"))), 1)
-
         self.assertEqual(len(list(checks.assert_true_or_false_with_in(
             "self.assertTrue(A in ['1', '2', '3'])"))), 1)
 
@@ -216,15 +173,15 @@ class HackingTestCase(test.NoDBTestCase):
         self.assertEqual(len(list(checks.assert_true_or_false_with_in(
             "self.assertFalse(some in list1 and some2 in list2)"))), 0)
 
-    def test_no_translate_debug_logs(self):
-        self.assertEqual(len(list(checks.no_translate_debug_logs(
+    def test_no_translate_logs(self):
+        self.assertEqual(len(list(checks.no_translate_logs(
             "LOG.debug(_('foo'))", "nova/scheduler/foo.py"))), 1)
 
-        self.assertEqual(len(list(checks.no_translate_debug_logs(
+        self.assertEqual(len(list(checks.no_translate_logs(
             "LOG.debug('foo')", "nova/scheduler/foo.py"))), 0)
 
-        self.assertEqual(len(list(checks.no_translate_debug_logs(
-            "LOG.info(_('foo'))", "nova/scheduler/foo.py"))), 0)
+        self.assertEqual(len(list(checks.no_translate_logs(
+            "LOG.info(_('foo'))", "nova/scheduler/foo.py"))), 1)
 
     def test_no_setting_conf_directly_in_tests(self):
         self.assertEqual(len(list(checks.no_setting_conf_directly_in_tests(
@@ -247,32 +204,6 @@ class HackingTestCase(test.NoDBTestCase):
         self.assertEqual(len(list(checks.no_setting_conf_directly_in_tests(
             "CONF.option = 1", "nova/compute/foo.py"))), 0)
 
-    def test_log_translations(self):
-        logs = ['audit', 'error', 'info', 'warning', 'critical', 'warn',
-                'exception']
-        levels = ['_LI', '_LW', '_LE', '_LC']
-        debug = "LOG.debug('OK')"
-        self.assertEqual(
-            0, len(list(checks.validate_log_translations(debug, debug, 'f'))))
-        for log in logs:
-            bad = 'LOG.%s("Bad")' % log
-            self.assertEqual(1,
-                len(list(
-                    checks.validate_log_translations(bad, bad, 'f'))))
-            ok = "LOG.%s('OK')    # noqa" % log
-            self.assertEqual(0,
-                len(list(
-                    checks.validate_log_translations(ok, ok, 'f'))))
-            ok = "LOG.%s(variable)" % log
-            self.assertEqual(0,
-                len(list(
-                    checks.validate_log_translations(ok, ok, 'f'))))
-            for level in levels:
-                ok = "LOG.%s(%s('OK'))" % (log, level)
-                self.assertEqual(0,
-                    len(list(
-                        checks.validate_log_translations(ok, ok, 'f'))))
-
     def test_no_mutable_default_args(self):
         self.assertEqual(1, len(list(checks.no_mutable_default_args(
             "def get_info_from_bdm(virt_type, bdm, mapping=[])"))))
@@ -284,23 +215,17 @@ class HackingTestCase(test.NoDBTestCase):
             "defined, undefined = [], {}"))))
 
     def test_check_explicit_underscore_import(self):
-        self.assertEqual(len(list(checks.check_explicit_underscore_import(
-            "LOG.info(_('My info message'))",
-            "cinder/tests/other_files.py"))), 1)
         self.assertEqual(len(list(checks.check_explicit_underscore_import(
             "msg = _('My message')",
             "cinder/tests/other_files.py"))), 1)
         self.assertEqual(len(list(checks.check_explicit_underscore_import(
             "from cinder.i18n import _",
             "cinder/tests/other_files.py"))), 0)
-        self.assertEqual(len(list(checks.check_explicit_underscore_import(
-            "LOG.info(_('My info message'))",
-            "cinder/tests/other_files.py"))), 0)
         self.assertEqual(len(list(checks.check_explicit_underscore_import(
             "msg = _('My message')",
             "cinder/tests/other_files.py"))), 0)
         self.assertEqual(len(list(checks.check_explicit_underscore_import(
-            "from cinder.i18n import _, _LW",
+            "from cinder.i18n import _",
             "cinder/tests/other_files2.py"))), 0)
         self.assertEqual(len(list(checks.check_explicit_underscore_import(
             "msg = _('My message')",
@@ -321,29 +246,36 @@ class HackingTestCase(test.NoDBTestCase):
         for method in ('dump', 'dumps', 'load', 'loads'):
             self.assertEqual(
                 __get_msg(method),
-                list(checks.use_jsonutils("json.%s(" % method,
-                                     "./nova/virt/xenapi/driver.py")))
-            self.assertEqual(0,
-                len(list(checks.use_jsonutils("json.%s(" % method,
-                                     "./plugins/xenserver/script.py"))))
-            self.assertEqual(0,
-                len(list(checks.use_jsonutils("jsonx.%s(" % method,
-                                     "./nova/virt/xenapi/driver.py"))))
-        self.assertEqual(0,
-            len(list(checks.use_jsonutils("json.dumb",
-                                 "./nova/virt/xenapi/driver.py"))))
+                list(checks.use_jsonutils(
+                    "json.%s(" % method, "./nova/virt/libvirt/driver.py")),
+            )
+            self.assertEqual(
+                0,
+                len(list(checks.use_jsonutils(
+                    "jsonx.%s(" % method, "./nova/virt/libvirt/driver.py"))),
+            )
 
-    # We are patching pep8 so that only the check under test is actually
+        self.assertEqual(
+            0,
+            len(list(checks.use_jsonutils(
+                "json.dumb", "./nova/virt/libvirt/driver.py"))),
+        )
+
+    # We are patching pycodestyle so that only the check under test is actually
     # installed.
-    @mock.patch('pep8._checks',
+    @mock.patch('pycodestyle._checks',
                 {'physical_line': {}, 'logical_line': {}, 'tree': {}})
     def _run_check(self, code, checker, filename=None):
-        pep8.register_check(checker)
+        pycodestyle.register_check(checker)
 
-        lines = textwrap.dedent(code).strip().splitlines(True)
+        lines = textwrap.dedent(code).lstrip().splitlines(True)
 
-        checker = pep8.Checker(filename=filename, lines=lines)
-        checker.check_all()
+        checker = pycodestyle.Checker(filename=filename, lines=lines)
+        # NOTE(sdague): the standard reporter has printing to stdout
+        # as a normal part of check_all, which bleeds through to the
+        # test output stream in an unhelpful way. This blocks that printing.
+        with mock.patch('pycodestyle.StandardReport.get_file_results'):
+            checker.check_all()
         checker.report._deferred_print.sort()
         return checker.report._deferred_print
 
@@ -356,71 +288,6 @@ class HackingTestCase(test.NoDBTestCase):
     def _assert_has_no_errors(self, code, checker, filename=None):
         self._assert_has_errors(code, checker, filename=filename)
 
-    def test_str_unicode_exception(self):
-
-        checker = checks.CheckForStrUnicodeExc
-        code = """
-               def f(a, b):
-                   try:
-                       p = str(a) + str(b)
-                   except ValueError as e:
-                       p = str(e)
-                   return p
-               """
-        errors = [(5, 16, 'N325')]
-        self._assert_has_errors(code, checker, expected_errors=errors)
-
-        code = """
-               def f(a, b):
-                   try:
-                       p = unicode(a) + str(b)
-                   except ValueError as e:
-                       p = e
-                   return p
-               """
-        self._assert_has_no_errors(code, checker)
-
-        code = """
-               def f(a, b):
-                   try:
-                       p = str(a) + str(b)
-                   except ValueError as e:
-                       p = unicode(e)
-                   return p
-               """
-        errors = [(5, 20, 'N325')]
-        self._assert_has_errors(code, checker, expected_errors=errors)
-
-        code = """
-               def f(a, b):
-                   try:
-                       p = str(a) + str(b)
-                   except ValueError as e:
-                       try:
-                           p  = unicode(a) + unicode(b)
-                       except ValueError as ve:
-                           p = str(e) + str(ve)
-                       p = e
-                   return p
-               """
-        errors = [(8, 20, 'N325'), (8, 29, 'N325')]
-        self._assert_has_errors(code, checker, expected_errors=errors)
-
-        code = """
-               def f(a, b):
-                   try:
-                       p = str(a) + str(b)
-                   except ValueError as e:
-                       try:
-                           p  = unicode(a) + unicode(b)
-                       except ValueError as ve:
-                           p = str(e) + unicode(ve)
-                       p = str(e)
-                   return p
-               """
-        errors = [(8, 20, 'N325'), (8, 33, 'N325'), (9, 16, 'N325')]
-        self._assert_has_errors(code, checker, expected_errors=errors)
-
     def test_api_version_decorator_check(self):
         code = """
                @some_other_decorator
@@ -442,13 +309,22 @@ class HackingTestCase(test.NoDBTestCase):
                                 expected_errors=[(1, 0, "N335")])
 
     def test_api_version_decorator_check_no_errors(self):
-        code = """
-               class ControllerClass():
-                   @wsgi.api_version("2.5")
-                   def my_method():
-                       pass
-               """
-        self._assert_has_no_errors(code, checks.check_api_version_decorator)
+        codelist = [
+            """
+            class ControllerClass():
+                @wsgi.api_version("2.5")
+                def my_method():
+                    pass
+            """,
+            """
+            @some_other_decorator
+            @mock.patch('foo', return_value=api_versions.APIVersion("2.5"))
+            def my_method():
+                pass
+            """]
+        for code in codelist:
+            self._assert_has_no_errors(
+                code, checks.check_api_version_decorator)
 
     def test_trans_add(self):
 
@@ -459,23 +335,14 @@ class HackingTestCase(test.NoDBTestCase):
 
 
                _ = fake_tran
-               _LI = _
-               _LW = _
-               _LE = _
-               _LC = _
 
 
                def f(a, b):
                    msg = _('test') + 'add me'
-                   msg = _LI('test') + 'add me'
-                   msg = _LW('test') + 'add me'
-                   msg = _LE('test') + 'add me'
-                   msg = _LC('test') + 'add me'
                    msg = 'add to me' + _('test')
                    return msg
                """
-        errors = [(13, 10, 'N326'), (14, 10, 'N326'), (15, 10, 'N326'),
-                  (16, 10, 'N326'), (17, 10, 'N326'), (18, 24, 'N326')]
+        errors = [(9, 10, 'N326'), (10, 24, 'N326')]
         self._assert_has_errors(code, checker, expected_errors=errors)
 
         code = """
@@ -613,50 +480,35 @@ class HackingTestCase(test.NoDBTestCase):
         self._assert_has_no_errors(code,
                                    checks.check_config_option_in_central_place,
                                    filename="nova/conf/serial_console.py")
-        # option at a location which is not in scope right now
-        # TODO(markus_z): This is temporary until all config options are
-        # moved to /nova/conf
-        self._assert_has_no_errors(code,
-                                   checks.check_config_option_in_central_place,
-                                   filename="nova/dummy/non_existent.py")
         # option at the wrong place in the tree
         self._assert_has_errors(code,
                                 checks.check_config_option_in_central_place,
                                 filename="nova/cmd/serialproxy.py",
                                 expected_errors=errors)
 
+        # option at a location which is marked as an exception
+        # TODO(macsz) remove testing exceptions as they are removed from
+        # check_config_option_in_central_place
+        self._assert_has_no_errors(code,
+                                   checks.check_config_option_in_central_place,
+                                   filename="nova/cmd/manage.py")
+        self._assert_has_no_errors(code,
+                                   checks.check_config_option_in_central_place,
+                                   filename="nova/tests/dummy_test.py")
+
     def test_check_doubled_words(self):
         errors = [(1, 0, "N343")]
 
-        # Artificial break to stop pep8 detecting the test !
-        code = "This is the" + " the best comment"
+        # Explicit addition of line-ending here and below since this isn't a
+        # block comment and without it we trigger #1804062. Artificial break is
+        # necessary to stop flake8 detecting the test
+        code = "'This is the" + " the best comment'\n"
         self._assert_has_errors(code, checks.check_doubled_words,
                                 expected_errors=errors)
 
-        code = "This is the then best comment"
+        code = "'This is the then best comment'\n"
         self._assert_has_no_errors(code, checks.check_doubled_words)
 
-    def test_dict_iteritems(self):
-        self.assertEqual(1, len(list(checks.check_python3_no_iteritems(
-            "obj.iteritems()"))))
-
-        self.assertEqual(0, len(list(checks.check_python3_no_iteritems(
-            "six.iteritems(ob))"))))
-
-    def test_dict_iterkeys(self):
-        self.assertEqual(1, len(list(checks.check_python3_no_iterkeys(
-            "obj.iterkeys()"))))
-
-        self.assertEqual(0, len(list(checks.check_python3_no_iterkeys(
-            "six.iterkeys(ob))"))))
-
-    def test_dict_itervalues(self):
-        self.assertEqual(1, len(list(checks.check_python3_no_itervalues(
-            "obj.itervalues()"))))
-
-        self.assertEqual(0, len(list(checks.check_python3_no_itervalues(
-            "six.itervalues(ob))"))))
-
     def test_no_os_popen(self):
         code = """
                import os
@@ -749,3 +601,311 @@ class HackingTestCase(test.NoDBTestCase):
         foo.enforce()
         """
         self._assert_has_no_errors(code, checks.check_policy_enforce)
+
+    def test_log_context(self):
+        code = """
+                  LOG.info("Rebooting instance",
+                           context=context, instance=instance)
+               """
+        errors = [(1, 0, 'N353')]
+        self._assert_has_errors(code, checks.check_context_log,
+                                expected_errors=errors)
+        code = """
+                  LOG.info("Rebooting instance",
+                           context=admin_context, instance=instance)
+               """
+        errors = [(1, 0, 'N353')]
+        self._assert_has_errors(code, checks.check_context_log,
+                                expected_errors=errors)
+        code = """
+                  LOG.info("Rebooting instance", instance=instance)
+               """
+        self._assert_has_no_errors(code, checks.check_context_log)
+
+    def test_no_assert_equal_true_false(self):
+        code = """
+                  self.assertEqual(context_is_admin, True)
+                  self.assertEqual(context_is_admin, False)
+                  self.assertEqual(True, context_is_admin)
+                  self.assertEqual(False, context_is_admin)
+                  self.assertNotEqual(context_is_admin, True)
+                  self.assertNotEqual(context_is_admin, False)
+                  self.assertNotEqual(True, context_is_admin)
+                  self.assertNotEqual(False, context_is_admin)
+               """
+        errors = [(1, 0, 'N355'), (2, 0, 'N355'), (3, 0, 'N355'),
+                  (4, 0, 'N355'), (5, 0, 'N355'), (6, 0, 'N355'),
+                  (7, 0, 'N355'), (8, 0, 'N355')]
+        self._assert_has_errors(code, checks.no_assert_equal_true_false,
+                                expected_errors=errors)
+        code = """
+                  self.assertEqual(context_is_admin, stuff)
+                  self.assertNotEqual(context_is_admin, stuff)
+               """
+        self._assert_has_no_errors(code, checks.no_assert_equal_true_false)
+
+    def test_no_assert_true_false_is_not(self):
+        code = """
+                  self.assertTrue(test is None)
+                  self.assertTrue(False is my_variable)
+                  self.assertFalse(None is test)
+                  self.assertFalse(my_variable is False)
+               """
+        errors = [(1, 0, 'N356'), (2, 0, 'N356'), (3, 0, 'N356'),
+                  (4, 0, 'N356')]
+        self._assert_has_errors(code, checks.no_assert_true_false_is_not,
+                                expected_errors=errors)
+
+    def test_check_uuid4(self):
+        code = """
+                  fake_uuid = uuid.uuid4()
+                  hex_uuid = uuid.uuid4().hex
+               """
+        errors = [(1, 0, 'N357'), (2, 0, 'N357')]
+        self._assert_has_errors(code, checks.check_uuid4,
+                                expected_errors=errors)
+        code = """
+                  int_uuid = uuid.uuid4().int
+                  urn_uuid = uuid.uuid4().urn
+                  variant_uuid = uuid.uuid4().variant
+                  version_uuid = uuid.uuid4().version
+               """
+        self._assert_has_no_errors(code, checks.check_uuid4)
+
+    def test_return_followed_by_space(self):
+        code = """
+                  return(42)
+                  return(a, b)
+                  return(' some string ')
+               """
+        errors = [(1, 0, 'N358'), (2, 0, 'N358'), (3, 0, 'N358')]
+        self._assert_has_errors(code, checks.return_followed_by_space,
+                                expected_errors=errors)
+        code = """
+                  return
+                  return 42
+                  return (a, b)
+                  return a, b
+                  return ' some string '
+               """
+        self._assert_has_no_errors(code, checks.return_followed_by_space)
+
+    def test_no_redundant_import_alias(self):
+        code = """
+                  from x import y as y
+                  import x as x
+                  import x.y.z as z
+                  import x.y.z as y.z
+               """
+        errors = [(x + 1, 0, 'N359') for x in range(4)]
+        self._assert_has_errors(code, checks.no_redundant_import_alias,
+                                expected_errors=errors)
+        code = """
+                  from x import y
+                  import x
+                  from x.y import z
+                  from a import bcd as cd
+                  import ab.cd.efg as fg
+                  import ab.cd.efg as d.efg
+               """
+        self._assert_has_no_errors(code, checks.no_redundant_import_alias)
+
+    def test_yield_followed_by_space(self):
+        code = """
+                  yield(x, y)
+                  yield{"type": "test"}
+                  yield[a, b, c]
+                  yield"test"
+                  yield'test'
+               """
+        errors = [(x + 1, 0, 'N360') for x in range(5)]
+        self._assert_has_errors(code, checks.yield_followed_by_space,
+                                expected_errors=errors)
+        code = """
+                  yield x
+                  yield (x, y)
+                  yield {"type": "test"}
+                  yield [a, b, c]
+                  yield "test"
+                  yield 'test'
+                  yieldx_func(a, b)
+               """
+        self._assert_has_no_errors(code, checks.yield_followed_by_space)
+
+    def test_assert_regexpmatches(self):
+        code = """
+                   self.assertRegexpMatches("Test", output)
+                   self.assertNotRegexpMatches("Notmatch", output)
+               """
+        errors = [(x + 1, 0, 'N361') for x in range(2)]
+        self._assert_has_errors(code, checks.assert_regexpmatches,
+                                expected_errors=errors)
+        code = """
+                   self.assertRegexpMatchesfoo("Test", output)
+                   self.assertNotRegexpMatchesbar("Notmatch", output)
+               """
+        self._assert_has_no_errors(code, checks.assert_regexpmatches)
+
+    def test_import_alias_privsep(self):
+        code = """
+                  from nova import privsep
+                  import nova.privsep as nova_privsep
+                  from nova.privsep import linux_net
+                  import nova.privsep.linux_net as privsep_linux_net
+               """
+        errors = [(x + 1, 0, 'N362') for x in range(4)]
+        bad_filenames = ('nova/foo/bar.py',
+                         'nova/foo/privsep.py',
+                         'nova/privsep_foo/bar.py')
+        for filename in bad_filenames:
+            self._assert_has_errors(
+                code, checks.privsep_imports_not_aliased,
+                expected_errors=errors,
+                filename=filename)
+        good_filenames = ('nova/privsep.py',
+                          'nova/privsep/__init__.py',
+                          'nova/privsep/foo.py')
+        for filename in good_filenames:
+            self._assert_has_no_errors(
+                code, checks.privsep_imports_not_aliased, filename=filename)
+        code = """
+                  import nova.privsep
+                  import nova.privsep.foo
+                  import nova.privsep.foo.bar
+                  import nova.foo.privsep
+                  import nova.foo.privsep.bar
+                  import nova.tests.unit.whatever
+               """
+        for filename in (good_filenames + bad_filenames):
+            self._assert_has_no_errors(
+                code, checks.privsep_imports_not_aliased, filename=filename)
+
+    def test_did_you_mean_tuple(self):
+        code = """
+                    if foo in (bar):
+                    if foo in ('bar'):
+                    if foo in (path.to.CONST_1234):
+                    if foo in (
+                        bar):
+               """
+        errors = [(x + 1, 0, 'N363') for x in range(4)]
+        self._assert_has_errors(
+            code, checks.did_you_mean_tuple, expected_errors=errors)
+        code = """
+                    def in(this_would_be_weird):
+                        # A match in (any) comment doesn't count
+                        if foo in (bar,)
+                            or foo in ('bar',)
+                            or foo in ("bar",)
+                            or foo in (set1 + set2)
+                            or foo in ("string continuations "
+                                "are probably okay")
+                            or foo in (method_call_should_this_work()):
+               """
+        self._assert_has_no_errors(code, checks.did_you_mean_tuple)
+
+    def test_nonexistent_assertion_methods_and_attributes(self):
+        code = """
+                   mock_sample.called_once()
+                   mock_sample.called_once_with(a, "TEST")
+                   mock_sample.has_calls([mock.call(x), mock.call(y)])
+                   mock_sample.mock_assert_called()
+                   mock_sample.mock_assert_called_once()
+                   mock_sample.mock_assert_called_with(a, "xxxx")
+                   mock_sample.mock_assert_called_once_with(a, b)
+                   mock_sample.mock_assert_any_call(1, 2, instance=instance)
+                   sample.mock_assert_has_calls([mock.call(x), mock.call(y)])
+                   mock_sample.mock_assert_not_called()
+                   mock_sample.asser_called()
+                   mock_sample.asser_called_once()
+                   mock_sample.asser_called_with(a, "xxxx")
+                   mock_sample.asser_called_once_with(a, b)
+                   mock_sample.asser_any_call(1, 2, instance=instance)
+                   mock_sample.asser_has_calls([mock.call(x), mock.call(y)])
+                   mock_sample.asser_not_called()
+                   mock_sample.asset_called()
+                   mock_sample.asset_called_once()
+                   mock_sample.asset_called_with(a, "xxxx")
+                   mock_sample.asset_called_once_with(a, b)
+                   mock_sample.asset_any_call(1, 2, instance=instance)
+                   mock_sample.asset_has_calls([mock.call(x), mock.call(y)])
+                   mock_sample.asset_not_called()
+                   mock_sample.asssert_called()
+                   mock_sample.asssert_called_once()
+                   mock_sample.asssert_called_with(a, "xxxx")
+                   mock_sample.asssert_called_once_with(a, b)
+                   mock_sample.asssert_any_call(1, 2, instance=instance)
+                   mock_sample.asssert_has_calls([mock.call(x), mock.call(y)])
+                   mock_sample.asssert_not_called()
+                   mock_sample.assset_called()
+                   mock_sample.assset_called_once()
+                   mock_sample.assset_called_with(a, "xxxx")
+                   mock_sample.assset_called_once_with(a, b)
+                   mock_sample.assset_any_call(1, 2, instance=instance)
+                   mock_sample.assset_has_calls([mock.call(x), mock.call(y)])
+                   mock_sample.assset_not_called()
+                   mock_sample.retrun_value = 100
+                   sample.call_method(mock_sample.retrun_value, 100)
+                   mock.Mock(retrun_value=100)
+               """
+        errors = [(x + 1, 0, 'N364') for x in range(41)]
+        # Check errors in 'nova/tests' directory.
+        self._assert_has_errors(
+            code, checks.nonexistent_assertion_methods_and_attributes,
+            expected_errors=errors, filename="nova/tests/unit/test_context.py")
+        # Check no errors in other than 'nova/tests' directory.
+        self._assert_has_no_errors(
+            code, checks.nonexistent_assertion_methods_and_attributes,
+            filename="nova/compute/api.py")
+
+        code = """
+                   mock_sample.assert_called()
+                   mock_sample.assert_called_once()
+                   mock_sample.assert_called_with(a, "xxxx")
+                   mock_sample.assert_called_once_with(a, b)
+                   mock_sample.assert_any_call(1, 2, instance=instance)
+                   mock_sample.assert_has_calls([mock.call(x), mock.call(y)])
+                   mock_sample.assert_not_called()
+                   mock_sample.return_value = 100
+                   sample.call_method(mock_sample.return_value, 100)
+                   mock.Mock(return_value=100)
+
+                   sample.has_other_calls([1, 2, 3, 4])
+                   sample.get_called_once("TEST")
+                   sample.check_called_once_with("TEST")
+                   mock_assert_method.assert_called()
+                   test.asset_has_all(test)
+                   test_retrun_value = 99
+                   test.retrun_values = 100
+               """
+        self._assert_has_no_errors(
+            code, checks.nonexistent_assertion_methods_and_attributes,
+            filename="nova/tests/unit/test_context.py")
+
+    def test_useless_assertion(self):
+        code = """
+                   self.assertIsNone(None, status)
+                   self.assertTrue(True, flag)
+                   self.assertTrue(10, count)
+                   self.assertTrue('active', status)
+                   self.assertTrue("building", status)
+               """
+        errors = [(x + 1, 0, 'N365') for x in range(5)]
+        # Check errors in 'nova/tests' directory.
+        self._assert_has_errors(
+            code, checks.useless_assertion,
+            expected_errors=errors, filename="nova/tests/unit/test_context.py")
+        # Check no errors in other than 'nova/tests' directory.
+        self._assert_has_no_errors(
+            code, checks.nonexistent_assertion_methods_and_attributes,
+            filename="nova/compute/api.py")
+        code = """
+                   self.assertIsNone(None_test_var, "Fails")
+                   self.assertTrue(True_test_var, 'Fails')
+                   self.assertTrue(var2, "Fails")
+                   self.assertTrue(test_class.is_active('active'), 'Fails')
+                   self.assertTrue(check_status("building"), 'Fails')
+               """
+        self._assert_has_no_errors(
+            code, checks.useless_assertion,
+            filename="nova/tests/unit/test_context.py")
