@@ -16,12 +16,14 @@
 """Tests for common notifications."""
 
 import copy
+import datetime
 
 import mock
-from oslo_context import context as o_context
+from oslo_config import cfg
 from oslo_context import fixture as o_fixture
+from oslo_utils.fixture import uuidsentinel as uuids
+from oslo_utils import timeutils
 
-from nova.compute import flavors
 from nova.compute import task_states
 from nova.compute import vm_states
 from nova import context
@@ -32,7 +34,9 @@ from nova.objects import base as obj_base
 from nova import test
 from nova.tests.unit import fake_network
 from nova.tests.unit import fake_notifier
-from nova.tests import uuidsentinel as uuids
+
+
+CONF = cfg.CONF
 
 
 class NotificationsTestCase(test.TestCase):
@@ -41,23 +45,13 @@ class NotificationsTestCase(test.TestCase):
         super(NotificationsTestCase, self).setUp()
         self.fixture = self.useFixture(o_fixture.ClearRequestContext())
 
-        self.net_info = fake_network.fake_get_instance_nw_info(self, 1,
-                                                               1)
-
-        def fake_get_nw_info(cls, ctxt, instance):
-            self.assertTrue(ctxt.is_admin)
-            return self.net_info
-
-        self.stub_out('nova.network.api.API.get_instance_nw_info',
-                fake_get_nw_info)
-        fake_network.set_stub_network_methods(self)
-
+        self.net_info = fake_network.fake_get_instance_nw_info(self)
         fake_notifier.stub_notifier(self)
         self.addCleanup(fake_notifier.reset)
 
-        self.flags(network_manager='nova.network.manager.FlatManager',
-                   notify_on_state_change="vm_and_task_state",
-                   host='testhost')
+        self.flags(host='testhost')
+        self.flags(notify_on_state_change="vm_and_task_state",
+                   group='notifications')
 
         self.flags(api_servers=['http://localhost:9292'], group='glance')
 
@@ -65,12 +59,15 @@ class NotificationsTestCase(test.TestCase):
         self.project_id = 'fake'
         self.context = context.RequestContext(self.user_id, self.project_id)
 
+        self.fake_time = datetime.datetime(2017, 2, 2, 16, 45, 0)
+        timeutils.set_time_override(self.fake_time)
+
         self.instance = self._wrapped_create()
 
         self.decorated_function_called = False
 
     def _wrapped_create(self, params=None):
-        instance_type = flavors.get_flavor_by_name('m1.tiny')
+        instance_type = objects.Flavor.get_by_name(self.context, 'm1.tiny')
         inst = objects.Instance(image_ref=uuids.image_ref,
                                 user_id=self.user_id,
                                 project_id=self.project_id,
@@ -90,101 +87,10 @@ class NotificationsTestCase(test.TestCase):
         inst.create()
         return inst
 
-    def test_send_api_fault_disabled(self):
-        self.flags(notify_api_faults=False)
-        notifications.send_api_fault("http://example.com/foo", 500, None)
-        self.assertEqual(0, len(fake_notifier.NOTIFICATIONS))
-
-    def test_send_api_fault(self):
-        self.flags(notify_api_faults=True)
-        exception = None
-        try:
-            # Get a real exception with a call stack.
-            raise test.TestingException("junk")
-        except test.TestingException as e:
-            exception = e
-
-        notifications.send_api_fault("http://example.com/foo", 500, exception)
-
-        self.assertEqual(1, len(fake_notifier.NOTIFICATIONS))
-        n = fake_notifier.NOTIFICATIONS[0]
-        self.assertEqual(n.priority, 'ERROR')
-        self.assertEqual(n.event_type, 'api.fault')
-        self.assertEqual(n.payload['url'], 'http://example.com/foo')
-        self.assertEqual(n.payload['status'], 500)
-        self.assertIsNotNone(n.payload['exception'])
-
-    def test_send_api_fault_fresh_context(self):
-        self.flags(notify_api_faults=True)
-        exception = None
-        try:
-            # Get a real exception with a call stack.
-            raise test.TestingException("junk")
-        except test.TestingException as e:
-            exception = e
-
-        ctxt = context.RequestContext(overwrite=True)
-        notifications.send_api_fault("http://example.com/foo", 500, exception)
-
-        self.assertEqual(1, len(fake_notifier.NOTIFICATIONS))
-        n = fake_notifier.NOTIFICATIONS[0]
-        self.assertEqual(n.priority, 'ERROR')
-        self.assertEqual(n.event_type, 'api.fault')
-        self.assertEqual(n.payload['url'], 'http://example.com/foo')
-        self.assertEqual(n.payload['status'], 500)
-        self.assertIsNotNone(n.payload['exception'])
-        self.assertEqual(ctxt, n.context)
-
-    def test_send_api_fault_fake_context(self):
-        self.flags(notify_api_faults=True)
-        exception = None
-        try:
-            # Get a real exception with a call stack.
-            raise test.TestingException("junk")
-        except test.TestingException as e:
-            exception = e
-
-        ctxt = o_context.get_current()
-        self.assertIsNotNone(ctxt)
-        notifications.send_api_fault("http://example.com/foo", 500, exception)
-
-        self.assertEqual(1, len(fake_notifier.NOTIFICATIONS))
-        n = fake_notifier.NOTIFICATIONS[0]
-        self.assertEqual(n.priority, 'ERROR')
-        self.assertEqual(n.event_type, 'api.fault')
-        self.assertEqual(n.payload['url'], 'http://example.com/foo')
-        self.assertEqual(n.payload['status'], 500)
-        self.assertIsNotNone(n.payload['exception'])
-        self.assertIsNotNone(n.context)
-        self.assertEqual(ctxt, n.context)
-
-    def test_send_api_fault_admin_context(self):
-        self.flags(notify_api_faults=True)
-        exception = None
-        try:
-            # Get a real exception with a call stack.
-            raise test.TestingException("junk")
-        except test.TestingException as e:
-            exception = e
-
-        self.fixture._remove_cached_context()
-        self.assertIsNone(o_context.get_current())
-        notifications.send_api_fault("http://example.com/foo", 500, exception)
-
-        self.assertEqual(1, len(fake_notifier.NOTIFICATIONS))
-        n = fake_notifier.NOTIFICATIONS[0]
-        self.assertEqual(n.priority, 'ERROR')
-        self.assertEqual(n.event_type, 'api.fault')
-        self.assertEqual(n.payload['url'], 'http://example.com/foo')
-        self.assertEqual(n.payload['status'], 500)
-        self.assertIsNotNone(n.payload['exception'])
-        self.assertIsNotNone(n.context)
-        self.assertTrue(n.context.is_admin)
-
     def test_notif_disabled(self):
 
         # test config disable of the notifications
-        self.flags(notify_on_state_change=None)
+        self.flags(notify_on_state_change=None, group='notifications')
 
         old = copy.copy(self.instance)
         self.instance.vm_state = vm_states.ACTIVE
@@ -205,7 +111,7 @@ class NotificationsTestCase(test.TestCase):
     def test_task_notif(self):
 
         # test config disable of just the task state notifications
-        self.flags(notify_on_state_change="vm_state")
+        self.flags(notify_on_state_change="vm_state", group='notifications')
 
         # we should not get a notification on task stgate chagne now
         old = copy.copy(self.instance)
@@ -224,7 +130,8 @@ class NotificationsTestCase(test.TestCase):
         self.assertEqual(0, len(fake_notifier.VERSIONED_NOTIFICATIONS))
 
         # ok now enable task state notifications and re-try
-        self.flags(notify_on_state_change="vm_and_task_state")
+        self.flags(notify_on_state_change="vm_and_task_state",
+                   group='notifications')
 
         notifications.send_update(self.context, old, self.instance)
         self.assertEqual(1, len(fake_notifier.NOTIFICATIONS))
@@ -306,7 +213,7 @@ class NotificationsTestCase(test.TestCase):
     @mock.patch.object(objects.BandwidthUsageList, 'get_by_uuids')
     def test_vm_update_with_states(self, mock_bandwidth_list):
         mock_bandwidth_list.return_value = [self.get_fake_bandwidth()]
-        fake_net_info = fake_network.fake_get_instance_nw_info(self, 1, 1)
+        fake_net_info = fake_network.fake_get_instance_nw_info(self)
         self.instance.info_cache.network_info = fake_net_info
 
         notifications.send_update_with_states(self.context, self.instance,
@@ -337,6 +244,11 @@ class NotificationsTestCase(test.TestCase):
         self.assertEqual(payload["display_name"], display_name)
         self.assertEqual(payload["hostname"], hostname)
         self.assertEqual(payload["node"], node)
+        self.assertEqual("2017-02-01T00:00:00.000000",
+                         payload["audit_period_beginning"])
+        self.assertEqual("2017-02-02T16:45:00.000000",
+                         payload["audit_period_ending"])
+
         payload = fake_notifier.VERSIONED_NOTIFICATIONS[0][
             'payload']['nova_object.data']
         state_update = payload['state_update']['nova_object.data']
@@ -375,9 +287,10 @@ class NotificationsTestCase(test.TestCase):
 
     @mock.patch.object(objects.BandwidthUsageList, 'get_by_uuids')
     def test_task_update_with_states(self, mock_bandwidth_list):
-        self.flags(notify_on_state_change="vm_and_task_state")
+        self.flags(notify_on_state_change="vm_and_task_state",
+                   group='notifications')
         mock_bandwidth_list.return_value = [self.get_fake_bandwidth()]
-        fake_net_info = fake_network.fake_get_instance_nw_info(self, 1, 1)
+        fake_net_info = fake_network.fake_get_instance_nw_info(self)
         self.instance.info_cache.network_info = fake_net_info
 
         notifications.send_update_with_states(self.context, self.instance,
@@ -404,16 +317,16 @@ class NotificationsTestCase(test.TestCase):
     def test_update_with_service_name(self):
         notifications.send_update_with_states(self.context, self.instance,
                 vm_states.BUILDING, vm_states.BUILDING, task_states.SPAWNING,
-                None, service="testservice")
+                None, service="nova-compute")
         self.assertEqual(1, len(fake_notifier.NOTIFICATIONS))
         self.assertEqual(1, len(fake_notifier.VERSIONED_NOTIFICATIONS))
 
         # service name should default to 'compute'
         notif = fake_notifier.NOTIFICATIONS[0]
-        self.assertEqual('testservice.testhost', notif.publisher_id)
+        self.assertEqual('nova-compute.testhost', notif.publisher_id)
 
         notif = fake_notifier.VERSIONED_NOTIFICATIONS[0]
-        self.assertEqual('testservice:testhost', notif['publisher_id'])
+        self.assertEqual('nova-compute:testhost', notif['publisher_id'])
 
     def test_update_with_host_name(self):
         notifications.send_update_with_states(self.context, self.instance,
@@ -431,20 +344,20 @@ class NotificationsTestCase(test.TestCase):
 
     def test_payload_has_fixed_ip_labels(self):
         info = notifications.info_from_instance(self.context, self.instance,
-                                                  self.net_info, None)
+                                                  self.net_info)
         self.assertIn("fixed_ips", info)
         self.assertEqual(info["fixed_ips"][0]["label"], "test1")
 
     def test_payload_has_vif_mac_address(self):
         info = notifications.info_from_instance(self.context, self.instance,
-                                                  self.net_info, None)
+                                                  self.net_info)
         self.assertIn("fixed_ips", info)
         self.assertEqual(self.net_info[0]['address'],
                          info["fixed_ips"][0]["vif_mac"])
 
     def test_payload_has_cell_name_empty(self):
         info = notifications.info_from_instance(self.context, self.instance,
-                                                  self.net_info, None)
+                                                  self.net_info)
         self.assertIn("cell_name", info)
         self.assertIsNone(self.instance.cell_name)
         self.assertEqual("", info["cell_name"])
@@ -452,13 +365,13 @@ class NotificationsTestCase(test.TestCase):
     def test_payload_has_cell_name(self):
         self.instance.cell_name = "cell1"
         info = notifications.info_from_instance(self.context, self.instance,
-                                                  self.net_info, None)
+                                                  self.net_info)
         self.assertIn("cell_name", info)
         self.assertEqual("cell1", info["cell_name"])
 
     def test_payload_has_progress_empty(self):
         info = notifications.info_from_instance(self.context, self.instance,
-                                                  self.net_info, None)
+                                                  self.net_info)
         self.assertIn("progress", info)
         self.assertIsNone(self.instance.progress)
         self.assertEqual("", info["progress"])
@@ -466,7 +379,7 @@ class NotificationsTestCase(test.TestCase):
     def test_payload_has_progress(self):
         self.instance.progress = 50
         info = notifications.info_from_instance(self.context, self.instance,
-                                                  self.net_info, None)
+                                                  self.net_info)
         self.assertIn("progress", info)
         self.assertEqual(50, info["progress"])
 
@@ -481,13 +394,27 @@ class NotificationsTestCase(test.TestCase):
         self.instance.flavor.memory_mb = 30
         self.instance.flavor.ephemeral_gb = 40
         info = notifications.info_from_instance(self.context, self.instance,
-                                                self.net_info, None)
+                                                self.net_info)
         self.assertEqual(10, info['vcpus'])
         self.assertEqual(20, info['root_gb'])
         self.assertEqual(30, info['memory_mb'])
         self.assertEqual(40, info['ephemeral_gb'])
         self.assertEqual(60, info['disk_gb'])
 
+    def test_payload_has_timestamp_fields(self):
+        time = datetime.datetime(2017, 2, 2, 16, 45, 0)
+        # do not define deleted_at to test that missing value is handled
+        # properly
+        self.instance.terminated_at = time
+        self.instance.launched_at = time
+
+        info = notifications.info_from_instance(self.context, self.instance,
+                                                self.net_info)
+
+        self.assertEqual('2017-02-02T16:45:00.000000', info['terminated_at'])
+        self.assertEqual('2017-02-02T16:45:00.000000', info['launched_at'])
+        self.assertEqual('', info['deleted_at'])
+
     def test_send_access_ip_update(self):
         notifications.send_update(self.context, self.instance, self.instance)
         self.assertEqual(1, len(fake_notifier.NOTIFICATIONS))
@@ -517,13 +444,38 @@ class NotificationsTestCase(test.TestCase):
             self.assertEqual(payload["old_display_name"], old_display_name)
             self.assertEqual(payload["display_name"], new_display_name)
 
+    def test_send_versioned_tags_update(self):
+        objects.TagList.create(self.context,
+                               self.instance.uuid, [u'tag1', u'tag2'])
+        notifications.send_update(self.context, self.instance, self.instance)
+        self.assertEqual(1, len(fake_notifier.VERSIONED_NOTIFICATIONS))
+
+        self.assertEqual([u'tag1', u'tag2'],
+                         fake_notifier.VERSIONED_NOTIFICATIONS[0]
+                         ['payload']['nova_object.data']['tags'])
+
+    def test_send_versioned_action_initiator_update(self):
+        notifications.send_update(self.context, self.instance, self.instance)
+        action_initiator_user = self.context.user_id
+        action_initiator_project = self.context.project_id
+        self.assertEqual(1, len(fake_notifier.VERSIONED_NOTIFICATIONS))
+
+        self.assertEqual(action_initiator_user,
+                         fake_notifier.VERSIONED_NOTIFICATIONS[0]
+                         ['payload']['nova_object.data']
+                         ['action_initiator_user'])
+        self.assertEqual(action_initiator_project,
+                         fake_notifier.VERSIONED_NOTIFICATIONS[0]
+                         ['payload']['nova_object.data']
+                         ['action_initiator_project'])
+
     def test_send_no_state_change(self):
         called = [False]
 
         def sending_no_state_change(context, instance, **kwargs):
             called[0] = True
         self.stub_out('nova.notifications.base.'
-                      '_send_instance_update_notification',
+                      'send_instance_update_notification',
                        sending_no_state_change)
         notifications.send_update(self.context, self.instance, self.instance)
         self.assertTrue(called[0])
@@ -532,7 +484,7 @@ class NotificationsTestCase(test.TestCase):
         def fail_sending(context, instance, **kwargs):
             raise Exception('failed to notify')
         self.stub_out('nova.notifications.base.'
-                      '_send_instance_update_notification',
+                      'send_instance_update_notification',
                        fail_sending)
 
         notifications.send_update(self.context, self.instance, self.instance)
@@ -544,7 +496,7 @@ class NotificationsTestCase(test.TestCase):
         # not logged as an error.
         notfound = exception.InstanceNotFound(instance_id=self.instance.uuid)
         with mock.patch.object(notifications,
-                               '_send_instance_update_notification',
+                               'send_instance_update_notification',
                                side_effect=notfound):
             notifications.send_update(
                 self.context, self.instance, self.instance)
@@ -558,7 +510,7 @@ class NotificationsTestCase(test.TestCase):
         # not logged as an error.
         notfound = exception.InstanceNotFound(instance_id=self.instance.uuid)
         with mock.patch.object(notifications,
-                               '_send_instance_update_notification',
+                               'send_instance_update_notification',
                                side_effect=notfound):
             notifications.send_update_with_states(
                 self.context, self.instance,
@@ -570,25 +522,6 @@ class NotificationsTestCase(test.TestCase):
     def _decorated_function(self, arg1, arg2):
         self.decorated_function_called = True
 
-    def test_notify_decorator(self):
-        func_name = self._decorated_function.__name__
-
-        # Decorated with notify_decorator like monkey_patch
-        self._decorated_function = notifications.notify_decorator(
-            func_name,
-            self._decorated_function)
-
-        ctxt = o_context.RequestContext()
-
-        self._decorated_function(1, ctxt)
-
-        self.assertEqual(1, len(fake_notifier.NOTIFICATIONS))
-        n = fake_notifier.NOTIFICATIONS[0]
-        self.assertEqual(n.priority, 'INFO')
-        self.assertEqual(n.event_type, func_name)
-        self.assertEqual(n.context, ctxt)
-        self.assertTrue(self.decorated_function_called)
-
 
 class NotificationsFormatTestCase(test.NoDBTestCase):
 
