@@ -17,17 +17,17 @@ import copy
 
 import fixtures
 import mock
+from oslo_utils.fixture import uuidsentinel as uuids
 
 from nova import block_device
-from nova.compute import arch
 from nova import context
 from nova import exception
 from nova import objects
+from nova.objects import fields as obj_fields
 from nova import test
+from nova.tests import fixtures as nova_fixtures
 from nova.tests.unit import fake_block_device
-import nova.tests.unit.image.fake
 from nova.tests.unit.virt import fakelibosinfo
-from nova.tests import uuidsentinel as uuids
 from nova.virt import block_device as driver_block_device
 from nova.virt import driver
 from nova.virt.libvirt import blockinfo
@@ -41,7 +41,7 @@ class LibvirtBlockInfoTest(test.NoDBTestCase):
         self.user_id = 'fake'
         self.project_id = 'fake'
         self.context = context.get_admin_context()
-        nova.tests.unit.image.fake.stub_out_image_service(self)
+        self.useFixture(nova_fixtures.GlanceFixture(self))
         self.test_instance = {
             'uuid': '32dfcb37-5af1-552b-357c-be8c3aa38310',
             'memory_kb': '1024000',
@@ -55,6 +55,7 @@ class LibvirtBlockInfoTest(test.NoDBTestCase):
             'ephemeral_gb': 20,
             'instance_type_id': 2,  # m1.tiny
             'config_drive': None,
+            'launched_at': None,
             'system_metadata': {},
         }
         self.test_image_meta = {
@@ -75,22 +76,25 @@ class LibvirtBlockInfoTest(test.NoDBTestCase):
         self.test_instance['old_flavor'] = None
         self.test_instance['new_flavor'] = None
 
-    def test_volume_in_mapping(self):
-        swap = {'device_name': '/dev/sdb',
-                'swap_size': 1}
+    def _test_block_device_info(self, with_eph=True, with_swap=True,
+                                with_bdms=True):
+        swap = {'device_name': '/dev/vdb', 'swap_size': 1}
         ephemerals = [{'device_type': 'disk', 'guest_format': 'ext4',
-                       'device_name': '/dev/sdc1', 'size': 10},
+                       'device_name': '/dev/vdc1', 'size': 10},
                       {'disk_bus': 'ide', 'guest_format': None,
-                       'device_name': '/dev/sdd', 'size': 10}]
+                       'device_name': '/dev/vdd', 'size': 10}]
         block_device_mapping = [{'mount_device': '/dev/sde',
                                  'device_path': 'fake_device'},
                                 {'mount_device': '/dev/sdf',
                                  'device_path': 'fake_device'}]
-        block_device_info = {
-                'root_device_name': '/dev/sda',
-                'swap': swap,
-                'ephemerals': ephemerals,
-                'block_device_mapping': block_device_mapping}
+        return {'root_device_name': '/dev/vda',
+                'swap': swap if with_swap else {},
+                'ephemerals': ephemerals if with_eph else [],
+                'block_device_mapping':
+                    block_device_mapping if with_bdms else []}
+
+    def test_volume_in_mapping(self):
+        block_device_info = self._test_block_device_info()
 
         def _assert_volume_in_mapping(device_name, true_or_false):
             self.assertEqual(
@@ -98,10 +102,10 @@ class LibvirtBlockInfoTest(test.NoDBTestCase):
                 block_device.volume_in_mapping(device_name,
                                                block_device_info))
 
-        _assert_volume_in_mapping('sda', False)
-        _assert_volume_in_mapping('sdb', True)
-        _assert_volume_in_mapping('sdc1', True)
-        _assert_volume_in_mapping('sdd', True)
+        _assert_volume_in_mapping('vda', False)
+        _assert_volume_in_mapping('vdb', True)
+        _assert_volume_in_mapping('vdc1', True)
+        _assert_volume_in_mapping('vdd', True)
         _assert_volume_in_mapping('sde', True)
         _assert_volume_in_mapping('sdf', True)
         _assert_volume_in_mapping('sdg', False)
@@ -124,10 +128,6 @@ class LibvirtBlockInfoTest(test.NoDBTestCase):
         dev = blockinfo.find_disk_dev_for_disk_bus(mapping, 'scsi')
         self.assertEqual('sdb', dev)
 
-        dev = blockinfo.find_disk_dev_for_disk_bus(mapping, 'scsi',
-                                                   last_device=True)
-        self.assertEqual('sdz', dev)
-
         dev = blockinfo.find_disk_dev_for_disk_bus(mapping, 'virtio')
         self.assertEqual('vda', dev)
 
@@ -164,9 +164,8 @@ class LibvirtBlockInfoTest(test.NoDBTestCase):
 
         mapping['disk.config'] = blockinfo.get_next_disk_info(mapping,
                                                               'ide',
-                                                              'cdrom',
-                                                              True)
-        self.assertEqual({'dev': 'hdd', 'bus': 'ide', 'type': 'cdrom'},
+                                                              'cdrom')
+        self.assertEqual({'dev': 'hda', 'bus': 'ide', 'type': 'cdrom'},
                          mapping['disk.config'])
 
     def test_get_next_disk_dev_boot_index(self):
@@ -215,20 +214,19 @@ class LibvirtBlockInfoTest(test.NoDBTestCase):
                                              "virtio", "ide",
                                              image_meta,
                                              block_device_info)
-
         expect = {
-            'disk': {'bus': 'scsi', 'dev': 'sda',
+            'disk': {'bus': 'virtio', 'dev': 'sda',
                      'type': 'disk', 'boot_index': '1'},
             'disk.local': {'bus': 'virtio', 'dev': 'vda', 'type': 'disk'},
-            'root': {'bus': 'scsi', 'dev': 'sda',
+            'root': {'bus': 'virtio', 'dev': 'sda',
                      'type': 'disk', 'boot_index': '1'}
-            }
+        }
         self.assertEqual(expect, mapping)
 
     def test_get_disk_mapping_rescue(self):
         # A simple disk mapping setup, but in rescue mode
 
-        instance_ref = objects.Instance()
+        instance_ref = objects.Instance(**self.test_instance)
         image_meta = objects.ImageMeta.from_dict(self.test_image_meta)
 
         mapping = blockinfo.get_disk_mapping("kvm", instance_ref,
@@ -245,6 +243,234 @@ class LibvirtBlockInfoTest(test.NoDBTestCase):
             }
         self.assertEqual(expect, mapping)
 
+    def test_get_disk_mapping_rescue_with_config(self):
+        # A simple disk mapping setup, but in rescue mode with a config drive
+
+        test_instance_with_config = self.test_instance
+        test_instance_with_config['config_drive'] = True
+        instance_ref = objects.Instance(**test_instance_with_config)
+        image_meta = objects.ImageMeta.from_dict(self.test_image_meta)
+
+        mapping = blockinfo.get_disk_mapping("kvm", instance_ref,
+                                             "virtio", "ide",
+                                             image_meta,
+                                             rescue=True)
+
+        expect_disk_config_rescue = {
+            'bus': 'ide', 'dev': 'hda', 'type': 'cdrom'}
+        if blockinfo.libvirt_utils.get_arch({}) == 'aarch64':
+            expect_disk_config_rescue['bus'] = 'scsi'
+            expect_disk_config_rescue['dev'] = 'sda'
+        expect = {
+            'disk.rescue': {'bus': 'virtio', 'dev': 'vda',
+                            'type': 'disk', 'boot_index': '1'},
+            'disk': {'bus': 'virtio', 'dev': 'vdb', 'type': 'disk'},
+            'disk.config.rescue': expect_disk_config_rescue,
+            'root': {'bus': 'virtio', 'dev': 'vda',
+                     'type': 'disk', 'boot_index': '1'},
+            }
+        self.assertEqual(expect, mapping)
+
+    def _test_get_disk_mapping_stable_rescue(
+            self, rescue_props, expected, block_device_info, with_local=False):
+        instance = objects.Instance(**self.test_instance)
+
+        # Make disk.local disks optional per test as found in
+        # nova.virt.libvirt.BlockInfo.get_default_ephemeral_info
+        instance.ephemeral_gb = '20' if with_local else None
+
+        image_meta = objects.ImageMeta.from_dict(self.test_image_meta)
+
+        rescue_image_meta = objects.ImageMeta.from_dict(self.test_image_meta)
+        rescue_props = objects.ImageMetaProps.from_dict(rescue_props)
+        rescue_image_meta.properties = rescue_props
+
+        mapping = blockinfo.get_disk_mapping("kvm", instance, "virtio", "ide",
+            image_meta, rescue=True, block_device_info=block_device_info,
+            rescue_image_meta=rescue_image_meta)
+
+        # Assert that the expected mapping is returned from get_disk_mapping
+        self.assertEqual(expected, mapping)
+
+    def test_get_disk_mapping_stable_rescue_virtio_disk(self):
+        """Assert the disk mapping when rescuing using a virtio disk"""
+        rescue_props = {'hw_rescue_bus': 'virtio'}
+        block_info = self._test_block_device_info(
+            with_eph=False, with_swap=False, with_bdms=False)
+        expected = {
+            'disk': {'boot_index': '1', 'bus': 'virtio', 'dev': 'vda',
+                     'type': 'disk'},
+            'disk.rescue': {'bus': 'virtio', 'dev': 'vdb', 'type': 'disk'},
+            'root': {'boot_index': '1', 'bus': 'virtio', 'dev': 'vda',
+                     'type': 'disk'}
+        }
+        self._test_get_disk_mapping_stable_rescue(
+            rescue_props, expected, block_info)
+
+    def test_get_disk_mapping_stable_rescue_ide_disk(self):
+        """Assert the disk mapping when rescuing using an IDE disk"""
+        rescue_props = {'hw_rescue_bus': 'ide'}
+        block_info = self._test_block_device_info(
+             with_eph=False, with_swap=False, with_bdms=False)
+        expected = {
+            'disk': {'boot_index': '1', 'bus': 'virtio', 'dev': 'vda',
+                     'type': 'disk'},
+            'disk.rescue': {'bus': 'ide', 'dev': 'hda', 'type': 'disk'},
+            'root': {'boot_index': '1', 'bus': 'virtio', 'dev': 'vda',
+                     'type': 'disk'}
+        }
+        self._test_get_disk_mapping_stable_rescue(
+            rescue_props, expected, block_info)
+
+    def test_get_disk_mapping_stable_rescue_usb_disk(self):
+        """Assert the disk mapping when rescuing using a USB disk"""
+        rescue_props = {'hw_rescue_bus': 'usb'}
+        block_info = self._test_block_device_info(
+            with_eph=False, with_swap=False, with_bdms=False)
+        expected = {
+            'disk': {'boot_index': '1', 'bus': 'virtio', 'dev': 'vda',
+                     'type': 'disk'},
+            'disk.rescue': {'bus': 'usb', 'dev': 'sda', 'type': 'disk'},
+            'root': {'boot_index': '1', 'bus': 'virtio', 'dev': 'vda',
+                     'type': 'disk'}
+        }
+        self._test_get_disk_mapping_stable_rescue(
+            rescue_props, expected, block_info)
+
+    def test_get_disk_mapping_stable_rescue_ide_cdrom(self):
+        """Assert the disk mapping when rescuing using an IDE cd-rom"""
+        rescue_props = {'hw_rescue_device': 'cdrom'}
+        block_info = self._test_block_device_info(
+            with_eph=False, with_swap=False, with_bdms=False)
+        expected = {
+            'disk': {'boot_index': '1', 'bus': 'virtio', 'dev': 'vda',
+                     'type': 'disk'},
+            'disk.rescue': {'bus': 'ide', 'dev': 'hda', 'type': 'cdrom'},
+            'root': {'boot_index': '1', 'bus': 'virtio', 'dev': 'vda',
+                     'type': 'disk'}
+        }
+        self._test_get_disk_mapping_stable_rescue(
+            rescue_props, expected, block_info)
+
+    def test_get_disk_mapping_stable_rescue_virtio_disk_with_local(self):
+        """Assert the disk mapping when rescuing using a virtio disk with
+           default ephemeral (local) disks also attached to the instance.
+        """
+        rescue_props = {'hw_rescue_bus': 'virtio'}
+        block_info = self._test_block_device_info(
+            with_eph=False, with_swap=False, with_bdms=False)
+        expected = {
+            'disk': {'boot_index': '1', 'bus': 'virtio', 'dev': 'vda',
+                     'type': 'disk'},
+            'disk.local': {'bus': 'virtio', 'dev': 'vdb', 'type': 'disk'},
+            'disk.rescue': {'bus': 'virtio', 'dev': 'vdc', 'type': 'disk'},
+            'root': {'boot_index': '1', 'bus': 'virtio', 'dev': 'vda',
+                     'type': 'disk'}
+        }
+        self._test_get_disk_mapping_stable_rescue(
+            rescue_props, expected, block_info, with_local=True)
+
+    def test_get_disk_mapping_stable_rescue_virtio_disk_with_eph(self):
+        """Assert the disk mapping when rescuing using a virtio disk with
+           ephemeral disks also attached to the instance.
+        """
+        rescue_props = {'hw_rescue_bus': 'virtio'}
+        block_info = self._test_block_device_info(
+            with_swap=False, with_bdms=False)
+        expected = {
+            'disk': {
+                'boot_index': '1', 'bus': 'virtio', 'dev': 'vda',
+                'type': 'disk'},
+            'disk.eph0': {
+                'bus': 'virtio', 'dev': 'vdc1', 'format': 'ext4',
+                'type': 'disk'},
+            'disk.eph1': {
+                'bus': 'ide', 'dev': 'vdd', 'type': 'disk'},
+            'disk.rescue': {
+                'bus': 'virtio', 'dev': 'vdb', 'type': 'disk'},
+            'root': {
+                'boot_index': '1', 'bus': 'virtio', 'dev': 'vda',
+                'type': 'disk'}
+        }
+        self._test_get_disk_mapping_stable_rescue(
+            rescue_props, expected, block_info, with_local=True)
+
+    def test_get_disk_mapping_stable_rescue_virtio_disk_with_swap(self):
+        """Assert the disk mapping when rescuing using a virtio disk with
+           swap attached to the instance.
+        """
+        rescue_props = {'hw_rescue_bus': 'virtio'}
+        block_info = self._test_block_device_info(
+            with_eph=False, with_bdms=False)
+        expected = {
+            'disk': {
+                'boot_index': '1', 'bus': 'virtio', 'dev': 'vda',
+                'type': 'disk'},
+            'disk.rescue': {
+                'bus': 'virtio', 'dev': 'vdc', 'type': 'disk'},
+            'disk.swap': {
+                'bus': 'virtio', 'dev': 'vdb', 'type': 'disk'},
+            'root': {
+                'boot_index': '1', 'bus': 'virtio', 'dev': 'vda',
+                'type': 'disk'}
+        }
+        self._test_get_disk_mapping_stable_rescue(
+            rescue_props, expected, block_info)
+
+    def test_get_disk_mapping_stable_rescue_virtio_disk_with_bdm(self):
+        """Assert the disk mapping when rescuing using a virtio disk with
+           volumes also attached to the instance.
+        """
+        rescue_props = {'hw_rescue_bus': 'virtio'}
+        block_info = self._test_block_device_info(
+            with_eph=False, with_swap=False)
+        expected = {
+            '/dev/sde': {
+                'bus': 'scsi', 'dev': 'sde', 'type': 'disk'},
+            '/dev/sdf': {
+                'bus': 'scsi', 'dev': 'sdf', 'type': 'disk'},
+            'disk': {
+                'boot_index': '1', 'bus': 'virtio', 'dev': 'vda',
+                'type': 'disk'},
+            'disk.rescue': {
+                'bus': 'virtio', 'dev': 'vdb', 'type': 'disk'},
+            'root': {
+                'boot_index': '1', 'bus': 'virtio', 'dev': 'vda',
+                'type': 'disk'}
+        }
+        self._test_get_disk_mapping_stable_rescue(
+            rescue_props, expected, block_info)
+
+    def test_get_disk_mapping_stable_rescue_virtio_disk_with_everything(self):
+        """Assert the disk mapping when rescuing using a virtio disk with
+           volumes, ephemerals and swap also attached to the instance.
+        """
+        rescue_props = {'hw_rescue_bus': 'virtio'}
+        block_info = self._test_block_device_info()
+        expected = {
+            '/dev/sde': {
+                'bus': 'scsi', 'dev': 'sde', 'type': 'disk'},
+            '/dev/sdf': {
+                'bus': 'scsi', 'dev': 'sdf', 'type': 'disk'},
+            'disk': {
+                'boot_index': '1', 'bus': 'virtio', 'dev': 'vda',
+                'type': 'disk'},
+            'disk.eph0': {
+                'bus': 'virtio', 'dev': 'vdc1', 'format': 'ext4',
+                'type': 'disk'},
+            'disk.eph1': {
+                'bus': 'ide', 'dev': 'vdd', 'type': 'disk'},
+            'disk.rescue': {
+                'bus': 'virtio', 'dev': 'vdc', 'type': 'disk'},
+            'disk.swap': {
+                'bus': 'virtio', 'dev': 'vdb', 'type': 'disk'},
+            'root': {
+                'boot_index': '1', 'bus': 'virtio', 'dev': 'vda',
+                'type': 'disk'}
+        }
+        self._test_get_disk_mapping_stable_rescue(
+            rescue_props, expected, block_info, with_local=True)
+
     def test_get_disk_mapping_lxc(self):
         # A simple disk mapping setup, but for lxc
 
@@ -355,17 +581,17 @@ class LibvirtBlockInfoTest(test.NoDBTestCase):
                                              "virtio", "ide",
                                              image_meta)
 
-        # The last device is selected for this. on x86 is the last ide
-        # device (hdd). Since power only support scsi, the last device
-        # is sdz
+        # Pick the first drive letter on the bus that is available
+        # as the config drive. Delete the last device hardcode as
+        # the config drive here.
 
-        bus_ppc = ("scsi", "sdz")
-        bus_aarch64 = ("scsi", "sdz")
+        bus_ppc = ("scsi", "sda")
+        bus_aarch64 = ("scsi", "sda")
         expect_bus = {"ppc": bus_ppc, "ppc64": bus_ppc,
                         "ppc64le": bus_ppc, "aarch64": bus_aarch64}
 
         bus, dev = expect_bus.get(blockinfo.libvirt_utils.get_arch({}),
-                                  ("ide", "hdd"))
+                                  ("ide", "hda"))
 
         expect = {
             'disk': {'bus': 'virtio', 'dev': 'vda',
@@ -394,13 +620,13 @@ class LibvirtBlockInfoTest(test.NoDBTestCase):
                                              "virtio", "ide",
                                              image_meta)
 
-        bus_ppc = ("scsi", "sdz")
-        bus_aarch64 = ("scsi", "sdz")
+        bus_ppc = ("scsi", "sda")
+        bus_aarch64 = ("scsi", "sda")
         expect_bus = {"ppc": bus_ppc, "ppc64": bus_ppc,
                         "ppc64le": bus_ppc, "aarch64": bus_aarch64}
 
         bus, dev = expect_bus.get(blockinfo.libvirt_utils.get_arch({}),
-                                  ("ide", "hdd"))
+                                  ("ide", "hda"))
 
         expect = {
             'disk': {'bus': 'virtio', 'dev': 'vda',
@@ -430,7 +656,7 @@ class LibvirtBlockInfoTest(test.NoDBTestCase):
             'disk': {'bus': 'virtio', 'dev': 'vda',
                      'type': 'disk', 'boot_index': '1'},
             'disk.local': {'bus': 'virtio', 'dev': 'vdb', 'type': 'disk'},
-            'disk.config': {'bus': 'virtio', 'dev': 'vdz', 'type': 'disk'},
+            'disk.config': {'bus': 'virtio', 'dev': 'vdc', 'type': 'disk'},
             'root': {'bus': 'virtio', 'dev': 'vda',
                      'type': 'disk', 'boot_index': '1'},
             }
@@ -703,23 +929,23 @@ class LibvirtBlockInfoTest(test.NoDBTestCase):
     def test_get_disk_bus(self):
         instance = objects.Instance(**self.test_instance)
         expected = (
-                (arch.X86_64, 'disk', 'virtio'),
-                (arch.X86_64, 'cdrom', 'ide'),
-                (arch.X86_64, 'floppy', 'fdc'),
-                (arch.PPC, 'disk', 'virtio'),
-                (arch.PPC, 'cdrom', 'scsi'),
-                (arch.PPC64, 'disk', 'virtio'),
-                (arch.PPC64, 'cdrom', 'scsi'),
-                (arch.PPCLE, 'disk', 'virtio'),
-                (arch.PPCLE, 'cdrom', 'scsi'),
-                (arch.PPC64LE, 'disk', 'virtio'),
-                (arch.PPC64LE, 'cdrom', 'scsi'),
-                (arch.S390, 'disk', 'virtio'),
-                (arch.S390, 'cdrom', 'scsi'),
-                (arch.S390X, 'disk', 'virtio'),
-                (arch.S390X, 'cdrom', 'scsi'),
-                (arch.AARCH64, 'disk', 'virtio'),
-                (arch.AARCH64, 'cdrom', 'scsi')
+                (obj_fields.Architecture.X86_64, 'disk', 'virtio'),
+                (obj_fields.Architecture.X86_64, 'cdrom', 'ide'),
+                (obj_fields.Architecture.X86_64, 'floppy', 'fdc'),
+                (obj_fields.Architecture.PPC, 'disk', 'virtio'),
+                (obj_fields.Architecture.PPC, 'cdrom', 'scsi'),
+                (obj_fields.Architecture.PPC64, 'disk', 'virtio'),
+                (obj_fields.Architecture.PPC64, 'cdrom', 'scsi'),
+                (obj_fields.Architecture.PPCLE, 'disk', 'virtio'),
+                (obj_fields.Architecture.PPCLE, 'cdrom', 'scsi'),
+                (obj_fields.Architecture.PPC64LE, 'disk', 'virtio'),
+                (obj_fields.Architecture.PPC64LE, 'cdrom', 'scsi'),
+                (obj_fields.Architecture.S390, 'disk', 'virtio'),
+                (obj_fields.Architecture.S390, 'cdrom', 'scsi'),
+                (obj_fields.Architecture.S390X, 'disk', 'virtio'),
+                (obj_fields.Architecture.S390X, 'cdrom', 'scsi'),
+                (obj_fields.Architecture.AARCH64, 'disk', 'virtio'),
+                (obj_fields.Architecture.AARCH64, 'cdrom', 'scsi')
                 )
         image_meta = objects.ImageMeta.from_dict(self.test_image_meta)
         for guestarch, dev, res in expected:
@@ -731,11 +957,15 @@ class LibvirtBlockInfoTest(test.NoDBTestCase):
                 self.assertEqual(res, bus)
 
         expected = (
-                ('scsi', None, 'disk', 'scsi'),
-                (None, 'scsi', 'cdrom', 'scsi'),
-                ('usb', None, 'disk', 'usb')
+                ('kvm', 'scsi', None, 'disk', 'scsi'),
+                ('kvm', None, 'scsi', 'cdrom', 'scsi'),
+                ('kvm', 'usb', None, 'disk', 'usb'),
+                ('parallels', 'scsi', None, 'disk', 'scsi'),
+                ('parallels', None, None, 'disk', 'scsi'),
+                ('parallels', None, 'ide', 'cdrom', 'ide'),
+                ('parallels', None, None, 'cdrom', 'ide')
                 )
-        for dbus, cbus, dev, res in expected:
+        for hv, dbus, cbus, dev, res in expected:
             props = {}
             if dbus is not None:
                 props['hw_disk_bus'] = dbus
@@ -744,7 +974,7 @@ class LibvirtBlockInfoTest(test.NoDBTestCase):
             image_meta = objects.ImageMeta.from_dict(
                 {'properties': props})
             bus = blockinfo.get_disk_bus_for_device_type(
-                instance, 'kvm', image_meta, device_type=dev)
+                instance, hv, image_meta, device_type=dev)
             self.assertEqual(res, bus)
 
         image_meta = objects.ImageMeta.from_dict(
@@ -788,6 +1018,31 @@ class LibvirtBlockInfoTest(test.NoDBTestCase):
         self.assertRaises(exception.NovaException,
                 blockinfo.get_disk_bus_for_disk_dev, 'inv', 'val')
 
+    @mock.patch('nova.virt.libvirt.utils.get_machine_type')
+    @mock.patch('nova.virt.libvirt.utils.get_arch')
+    def test_get_disk_bus_for_device_type_cdrom_with_q35_get_arch(self,
+            mock_get_arch, mock_get_machine_type):
+        instance = objects.Instance(**self.test_instance)
+        mock_get_machine_type.return_value = 'pc-q35-rhel8.0.0'
+        mock_get_arch.return_value = obj_fields.Architecture.X86_64
+        image_meta = {'properties': {}}
+        image_meta = objects.ImageMeta.from_dict(image_meta)
+        bus = blockinfo.get_disk_bus_for_device_type(instance, 'kvm',
+                                                     image_meta,
+                                                     device_type='cdrom')
+        self.assertEqual('sata', bus)
+
+    def test_get_disk_bus_for_device_type_cdrom_with_q35_image_meta(self):
+        instance = objects.Instance(**self.test_instance)
+        image_meta = {'properties': {
+            'hw_machine_type': 'pc-q35-rhel8.0.0',
+            'hw_architecture': obj_fields.Architecture.X86_64}}
+        image_meta = objects.ImageMeta.from_dict(image_meta)
+        bus = blockinfo.get_disk_bus_for_device_type(instance, 'kvm',
+                                                     image_meta,
+                                                     device_type='cdrom')
+        self.assertEqual('sata', bus)
+
     def test_get_config_drive_type_default(self):
         config_drive_type = blockinfo.get_config_drive_type()
         self.assertEqual('cdrom', config_drive_type)
@@ -821,7 +1076,9 @@ class LibvirtBlockInfoTest(test.NoDBTestCase):
                  'disk_bus': 'scsi',
                  'boot_index': 1,
                  'device_type': 'lame_type',
-                 'delete_on_termination': True}]
+                 'delete_on_termination': True},
+                {'disk_bus': 'sata', 'guest_format': None,
+                 'device_name': '/dev/sda', 'size': 3}]
         expected = [{'dev': 'vds', 'type': 'disk', 'bus': 'usb'},
                     {'dev': 'vdb', 'type': 'disk',
                      'bus': 'virtio', 'format': 'ext4'},
@@ -829,7 +1086,8 @@ class LibvirtBlockInfoTest(test.NoDBTestCase):
                     {'dev': 'sdr', 'type': 'cdrom',
                      'bus': 'scsi', 'boot_index': '1'},
                     {'dev': 'vdo', 'type': 'disk',
-                     'bus': 'scsi', 'boot_index': '2'}]
+                     'bus': 'scsi', 'boot_index': '2'},
+                    {'dev': 'sda', 'type': 'disk', 'bus': 'sata'}]
 
         image_meta = objects.ImageMeta.from_dict(self.test_image_meta)
         for bdm, expected in zip(bdms, expected):
@@ -904,38 +1162,29 @@ class LibvirtBlockInfoTest(test.NoDBTestCase):
 
     @mock.patch('nova.virt.libvirt.blockinfo.find_disk_dev_for_disk_bus',
                 return_value='vda')
-    @mock.patch('nova.virt.libvirt.blockinfo.get_disk_bus_for_disk_dev',
-                return_value='virtio')
-    def test_get_root_info_no_bdm(self, mock_get_bus, mock_find_dev):
+    def test_get_root_info_no_bdm(self, mock_find_dev):
         instance = objects.Instance(**self.test_instance)
         image_meta = objects.ImageMeta.from_dict(self.test_image_meta)
-        blockinfo.get_root_info(instance, 'kvm', image_meta, None,
+        info = blockinfo.get_root_info(instance, 'kvm', image_meta, None,
                                 'virtio', 'ide')
         mock_find_dev.assert_called_once_with({}, 'virtio')
 
-        blockinfo.get_root_info(instance, 'kvm', image_meta, None,
-                                'virtio', 'ide', root_device_name='/dev/vda')
-        mock_get_bus.assert_called_once_with('kvm', '/dev/vda')
+        self.assertEqual('virtio', info['bus'])
 
     @mock.patch('nova.virt.libvirt.blockinfo.find_disk_dev_for_disk_bus',
                 return_value='vda')
-    @mock.patch('nova.virt.libvirt.blockinfo.get_disk_bus_for_disk_dev',
-                return_value='virtio')
-    def test_get_root_info_no_bdm_empty_image_meta(self, mock_get_bus,
-                                                   mock_find_dev):
+    def test_get_root_info_no_bdm_empty_image_meta(self, mock_find_dev):
         # The evacuate operation passes image_ref=None to the compute node for
         # rebuild which then defaults image_meta to {}, so we don't have any
         # attributes in the ImageMeta object passed to get_root_info and we
         # need to make sure we don't try lazy-loading anything.
         instance = objects.Instance(**self.test_instance)
         image_meta = objects.ImageMeta.from_dict({})
-        blockinfo.get_root_info(instance, 'kvm', image_meta, None,
+        info = blockinfo.get_root_info(instance, 'kvm', image_meta, None,
                                 'virtio', 'ide')
         mock_find_dev.assert_called_once_with({}, 'virtio')
 
-        blockinfo.get_root_info(instance, 'kvm', image_meta, None,
-                                'virtio', 'ide', root_device_name='/dev/vda')
-        mock_get_bus.assert_called_once_with('kvm', '/dev/vda')
+        self.assertEqual('virtio', info['bus'])
 
     @mock.patch('nova.virt.libvirt.blockinfo.get_info_from_bdm')
     def test_get_root_info_bdm(self, mock_get_info):
@@ -966,6 +1215,16 @@ class LibvirtBlockInfoTest(test.NoDBTestCase):
                                                'disk_bus': 'scsi',
                                                'device_type': 'disk'},
                                               {}, 'virtio')
+        mock_get_info.reset_mock()
+        # xen with incompatible root_device_name/disk_bus combination
+        root_bdm['disk_bus'] = 'xen'
+        blockinfo.get_root_info(instance, 'xen', image_meta, root_bdm,
+                                'xen', 'ide', root_device_name='sda')
+        mock_get_info.assert_called_once_with(instance, 'xen', image_meta,
+                                              {'device_name': 'xvda',
+                                               'disk_bus': 'xen',
+                                               'device_type': 'disk'},
+                                              {}, 'xen')
 
     def test_get_boot_order_simple(self):
         disk_info = {
@@ -1021,6 +1280,40 @@ class LibvirtBlockInfoTest(test.NoDBTestCase):
         expected_order = ['hd', 'cdrom']
         self.assertEqual(expected_order, blockinfo.get_boot_order(disk_info))
 
+    def _get_rescue_image_meta(self, props_dict):
+        meta_dict = dict(self.test_image_meta)
+        meta_dict['properties'] = props_dict
+        return objects.ImageMeta.from_dict(meta_dict)
+
+    def test_get_rescue_device(self):
+        # Assert that all supported device types are returned correctly
+        for device in blockinfo.SUPPORTED_DEVICE_TYPES:
+            meta = self._get_rescue_image_meta({'hw_rescue_device': device})
+            self.assertEqual(device, blockinfo.get_rescue_device(meta))
+
+        # Assert that disk is returned if hw_rescue_device isn't set
+        meta = self._get_rescue_image_meta({'hw_rescue_bus': 'virtio'})
+        self.assertEqual('disk', blockinfo.get_rescue_device(meta))
+
+        # Assert that UnsupportedHardware is raised for unsupported devices
+        meta = self._get_rescue_image_meta({'hw_rescue_device': 'fs'})
+        self.assertRaises(exception.UnsupportedRescueDevice,
+                          blockinfo.get_rescue_device, meta)
+
+    def test_get_rescue_bus(self):
+        # Assert that all supported device bus types are returned. Stable
+        # device rescue is not supported by xen or lxc so ignore these.
+        for virt_type in ['qemu', 'kvm', 'uml', 'parallels']:
+            for bus in blockinfo.SUPPORTED_DEVICE_BUSES[virt_type]:
+                meta = self._get_rescue_image_meta({'hw_rescue_bus': bus})
+                self.assertEqual(bus, blockinfo.get_rescue_bus(None, virt_type,
+                                                               meta, None))
+
+        # Assert that UnsupportedHardware is raised for unsupported devices
+        meta = self._get_rescue_image_meta({'hw_rescue_bus': 'xen'})
+        self.assertRaises(exception.UnsupportedRescueBus,
+                          blockinfo.get_rescue_bus, None, 'kvm', meta, 'disk')
+
 
 class DefaultDeviceNamesTestCase(test.NoDBTestCase):
     def setUp(self):
