@@ -13,11 +13,13 @@
 #    under the License.
 
 from lxml import etree
+from oslo_utils import uuidutils
 import six
 
-from nova.compute import arch
+from nova.objects import fields as obj_fields
 from nova import test
 import nova.tests.unit.virt.libvirt.fakelibvirt as libvirt
+from nova.virt.libvirt import config as vconfig
 
 
 def get_vm_xml(name="testname", uuid=None, source_type='file',
@@ -51,8 +53,8 @@ def get_vm_xml(name="testname", uuid=None, source_type='file',
       <source %(interface_type)s='br100'/>
     </interface>
     <input type='mouse' bus='ps2'/>
-    <graphics type='vnc' port='5901' autoport='yes' keymap='en-us'/>
-    <graphics type='spice' port='5901' autoport='yes' keymap='en-us'/>
+    <graphics type='vnc' port='5901' autoport='yes'/>
+    <graphics type='spice' port='5901' autoport='yes'/>
   </devices>
 </domain>''' % {'name': name,
                 'uuid_tag': uuid_tag,
@@ -81,7 +83,7 @@ class FakeLibvirtTests(test.NoDBTestCase):
     def test_openAuth_accepts_None_uri_by_default(self):
         conn_method = self.get_openAuth_curry_func()
         conn = conn_method(None)
-        self.assertNotEqual(conn, None, "Connecting to fake libvirt failed")
+        self.assertIsNotNone(conn, "Connecting to fake libvirt failed")
 
     def test_openAuth_can_refuse_None_uri(self):
         conn_method = self.get_openAuth_curry_func()
@@ -96,15 +98,18 @@ class FakeLibvirtTests(test.NoDBTestCase):
     def test_getInfo(self):
         conn_method = self.get_openAuth_curry_func(readOnly=True)
         res = conn_method(None).getInfo()
-        self.assertIn(res[0], (arch.I686, arch.X86_64))
-        self.assertTrue(1024 <= res[1] <= 16384,
-                        "Memory unusually high or low.")
-        self.assertTrue(1 <= res[2] <= 32,
-                        "Active CPU count unusually high or low.")
-        self.assertTrue(800 <= res[3] <= 4500,
-                        "CPU speed unusually high or low.")
-        self.assertLessEqual(res[2], (res[5] * res[6]), "More active CPUs "
-                             "than num_sockets*cores_per_socket")
+        self.assertIn(res[0], (obj_fields.Architecture.I686,
+                               obj_fields.Architecture.X86_64))
+
+        self.assertLessEqual(1024, res[1], "Memory unusually high.")
+        self.assertGreaterEqual(16384, res[1], "Memory unusually low.")
+        self.assertLessEqual(1, res[2], "Active CPU count unusually high.")
+        self.assertGreaterEqual(32, res[2], "Active CPU count unusually low.")
+        self.assertLessEqual(800, res[3], "CPU speed unusually high.")
+        self.assertGreaterEqual(4500, res[3], "CPU speed unusually low.")
+        self.assertEqual(res[2], (res[4] * res[5] * res[6] * res[7]),
+                         "More active CPUs than num_nodes * num_sockets * "
+                         "cores_per_socket * threads_per_core")
 
     def test_createXML_detects_invalid_xml(self):
         self._test_XML_func_detects_invalid_xml('createXML', [0])
@@ -123,29 +128,32 @@ class FakeLibvirtTests(test.NoDBTestCase):
         raise self.failureException("Invalid XML didn't raise libvirtError")
 
     def test_defineXML_defines_domain(self):
+        uuid = uuidutils.generate_uuid()
         conn = self.get_openAuth_curry_func()('qemu:///system')
-        conn.defineXML(get_vm_xml())
-        dom = conn.lookupByName('testname')
+        conn.defineXML(get_vm_xml(uuid=uuid))
+        dom = conn.lookupByUUIDString(uuid)
         self.assertEqual('testname', dom.name())
         self.assertEqual(0, dom.isActive())
         dom.undefine()
         self.assertRaises(libvirt.libvirtError,
-                          conn.lookupByName,
-                          'testname')
+                          conn.lookupByUUIDString,
+                          uuid)
 
     def test_blockStats(self):
+        uuid = uuidutils.generate_uuid()
         conn = self.get_openAuth_curry_func()('qemu:///system')
-        conn.createXML(get_vm_xml(), 0)
-        dom = conn.lookupByName('testname')
+        conn.createXML(get_vm_xml(uuid=uuid), 0)
+        dom = conn.lookupByUUIDString(uuid)
         blockstats = dom.blockStats('vda')
         self.assertEqual(len(blockstats), 5)
         for x in blockstats:
             self.assertIn(type(x), six.integer_types)
 
     def test_attach_detach(self):
+        uuid = uuidutils.generate_uuid()
         conn = self.get_openAuth_curry_func()('qemu:///system')
-        conn.createXML(get_vm_xml(), 0)
-        dom = conn.lookupByName('testname')
+        conn.createXML(get_vm_xml(uuid=uuid), 0)
+        dom = conn.lookupByUUIDString(uuid)
         xml = '''<disk type='block'>
                    <driver name='qemu' type='raw'/>
                    <source dev='/dev/nbd0'/>
@@ -155,9 +163,10 @@ class FakeLibvirtTests(test.NoDBTestCase):
         self.assertTrue(dom.detachDevice(xml))
 
     def test_info(self):
+        uuid = uuidutils.generate_uuid()
         conn = self.get_openAuth_curry_func()('qemu:///system')
-        conn.createXML(get_vm_xml(), 0)
-        dom = conn.lookupByName('testname')
+        conn.createXML(get_vm_xml(uuid=uuid), 0)
+        dom = conn.lookupByUUIDString(uuid)
         info = dom.info()
         self.assertEqual(info[0], libvirt.VIR_DOMAIN_RUNNING)
         self.assertEqual(info[1], 128000)
@@ -166,40 +175,43 @@ class FakeLibvirtTests(test.NoDBTestCase):
         self.assertIn(type(info[4]), six.integer_types)
 
     def test_createXML_runs_domain(self):
+        uuid = uuidutils.generate_uuid()
         conn = self.get_openAuth_curry_func()('qemu:///system')
-        conn.createXML(get_vm_xml(), 0)
-        dom = conn.lookupByName('testname')
+        conn.createXML(get_vm_xml(uuid=uuid), 0)
+        dom = conn.lookupByUUIDString(uuid)
         self.assertEqual('testname', dom.name())
         self.assertEqual(1, dom.isActive())
         dom.destroy()
         try:
-            conn.lookupByName('testname')
+            conn.lookupByUUIDString(uuid)
         except libvirt.libvirtError as e:
             self.assertEqual(e.get_error_code(), libvirt.VIR_ERR_NO_DOMAIN)
             self.assertEqual(e.get_error_domain(), libvirt.VIR_FROM_QEMU)
             return
-        self.fail("lookupByName succeeded for destroyed non-defined VM")
+        self.fail("lookupByUUIDString succeeded for destroyed non-defined VM")
 
     def test_defineXML_remembers_uuid(self):
         conn = self.get_openAuth_curry_func()('qemu:///system')
         uuid = 'b21f957d-a72f-4b93-b5a5-45b1161abb02'
         conn.defineXML(get_vm_xml(uuid=uuid))
-        dom = conn.lookupByName('testname')
+        dom = conn.lookupByUUIDString(uuid)
         self.assertEqual(dom.UUIDString(), uuid)
 
     def test_createWithFlags(self):
+        uuid = uuidutils.generate_uuid()
         conn = self.get_openAuth_curry_func()('qemu:///system')
-        conn.defineXML(get_vm_xml())
-        dom = conn.lookupByName('testname')
+        conn.defineXML(get_vm_xml(uuid=uuid))
+        dom = conn.lookupByUUIDString(uuid)
         self.assertFalse(dom.isActive(), 'Defined domain was running.')
         dom.createWithFlags(0)
         self.assertTrue(dom.isActive(),
                         'Domain wasn\'t running after createWithFlags')
 
     def test_managedSave(self):
+        uuid = uuidutils.generate_uuid()
         conn = self.get_openAuth_curry_func()('qemu:///system')
-        conn.defineXML(get_vm_xml())
-        dom = conn.lookupByName('testname')
+        conn.defineXML(get_vm_xml(uuid=uuid))
+        dom = conn.lookupByUUIDString(uuid)
         self.assertFalse(dom.isActive(), 'Defined domain was running.')
         dom.createWithFlags(0)
         self.assertEqual(dom.hasManagedSaveImage(0), 0)
@@ -208,40 +220,21 @@ class FakeLibvirtTests(test.NoDBTestCase):
         dom.managedSaveRemove(0)
         self.assertEqual(dom.hasManagedSaveImage(0), 0)
 
-    def test_listDomainsId_and_lookupById(self):
-        conn = self.get_openAuth_curry_func()('qemu:///system')
-        self.assertEqual(conn.listDomainsID(), [])
-        conn.defineXML(get_vm_xml())
-        dom = conn.lookupByName('testname')
-        dom.createWithFlags(0)
-        self.assertEqual(len(conn.listDomainsID()), 1)
-
-        dom_id = conn.listDomainsID()[0]
-        self.assertEqual(conn.lookupByID(dom_id), dom)
-
-        dom_id = conn.listDomainsID()[0]
-        try:
-            conn.lookupByID(dom_id + 1)
-        except libvirt.libvirtError as e:
-            self.assertEqual(e.get_error_code(), libvirt.VIR_ERR_NO_DOMAIN)
-            self.assertEqual(e.get_error_domain(), libvirt.VIR_FROM_QEMU)
-            return
-        raise self.failureException("Looking up an invalid domain ID didn't "
-                                    "raise libvirtError")
-
     def test_define_and_retrieve(self):
+        uuid = uuidutils.generate_uuid()
         conn = self.get_openAuth_curry_func()('qemu:///system')
-        self.assertEqual(conn.listDomainsID(), [])
-        conn.defineXML(get_vm_xml())
-        dom = conn.lookupByName('testname')
+        self.assertEqual(conn.listAllDomains(), [])
+        conn.defineXML(get_vm_xml(uuid=uuid))
+        dom = conn.lookupByUUIDString(uuid)
         xml = dom.XMLDesc(0)
         etree.fromstring(xml)
 
     def _test_accepts_source_type(self, source_type):
+        uuid = uuidutils.generate_uuid()
         conn = self.get_openAuth_curry_func()('qemu:///system')
-        self.assertEqual(conn.listDomainsID(), [])
-        conn.defineXML(get_vm_xml(source_type=source_type))
-        dom = conn.lookupByName('testname')
+        self.assertEqual(conn.listAllDomains(), [])
+        conn.defineXML(get_vm_xml(source_type=source_type, uuid=uuid))
+        dom = conn.lookupByUUIDString(uuid)
         xml = dom.XMLDesc(0)
         tree = etree.fromstring(xml)
         elem = tree.find('./devices/disk/source')
@@ -260,10 +253,11 @@ class FakeLibvirtTests(test.NoDBTestCase):
         self._test_network_type_sticks('network')
 
     def _test_network_type_sticks(self, network_type):
+        uuid = uuidutils.generate_uuid()
         conn = self.get_openAuth_curry_func()('qemu:///system')
-        self.assertEqual(conn.listDomainsID(), [])
-        conn.defineXML(get_vm_xml(interface_type=network_type))
-        dom = conn.lookupByName('testname')
+        self.assertEqual(conn.listAllDomains(), [])
+        conn.defineXML(get_vm_xml(interface_type=network_type, uuid=uuid))
+        dom = conn.lookupByUUIDString(uuid)
         xml = dom.XMLDesc(0)
         tree = etree.fromstring(xml)
         elem = tree.find('./devices/interface')
@@ -283,24 +277,10 @@ class FakeLibvirtTests(test.NoDBTestCase):
         conn = self.get_openAuth_curry_func()('qemu:///system')
         etree.fromstring(conn.getCapabilities())
 
-    def test_nwfilter_define_undefine(self):
+    def test_getDomainCapabilities(self):
         conn = self.get_openAuth_curry_func()('qemu:///system')
-        # Will raise an exception if it's not valid XML
-        xml = '''<filter name='nova-instance-instance-789' chain='root'>
-                    <uuid>946878c6-3ad3-82b2-87f3-c709f3807f58</uuid>
-                 </filter>'''
-
-        conn.nwfilterDefineXML(xml)
-        nwfilter = conn.nwfilterLookupByName('nova-instance-instance-789')
-        nwfilter.undefine()
-        try:
-            conn.nwfilterLookupByName('nova-instance-instance-789320334')
-        except libvirt.libvirtError as e:
-            self.assertEqual(e.get_error_code(), libvirt.VIR_ERR_NO_NWFILTER)
-            self.assertEqual(e.get_error_domain(), libvirt.VIR_FROM_NWFILTER)
-            return
-        raise self.failureException("Invalid NWFilter name didn't"
-                                    " raise libvirtError")
+        etree.fromstring(conn.getDomainCapabilities(
+            '/usr/bin/qemu-kvm', 'x86_64', 'q35', 'kvm', 0))
 
     def test_compareCPU_compatible(self):
         conn = self.get_openAuth_curry_func()('qemu:///system')
@@ -367,8 +347,7 @@ class FakeLibvirtTests(test.NoDBTestCase):
                               conn.host_info.cpu_sockets,
                               conn.host_info.cpu_cores,
                               conn.host_info.cpu_threads)
-        self.assertEqual(conn.compareCPU(xml, 0),
-                         libvirt.VIR_CPU_COMPARE_INCOMPATIBLE)
+        self.assertRaises(libvirt.libvirtError, conn.compareCPU, xml, 0)
 
     def test_compareCPU_compatible_unspecified_model(self):
         conn = self.get_openAuth_curry_func()('qemu:///system')
@@ -411,9 +390,90 @@ class FakeLibvirtTests(test.NoDBTestCase):
   </cells>
 </topology>
 """
-        host_topology = libvirt.HostInfo._gen_numa_topology(
-                                               cpu_nodes=2, cpu_sockets=1,
-                                               cpu_cores=2, cpu_threads=2,
-                                               kb_mem=15740000)
+        host_topology = libvirt.NUMATopology(
+            cpu_nodes=2, cpu_sockets=1, cpu_cores=2, cpu_threads=2,
+            kb_mem=15740000)
         self.assertEqual(host_topology.to_xml(),
                          topology)
+
+    def test_pci_devices_generation(self):
+        def _cmp_pci_dev_addr(dev_xml, cmp_addr):
+            cfgdev = vconfig.LibvirtConfigNodeDevice()
+            cfgdev.parse_str(dev_xml)
+
+            address = "%04x:%02x:%02x.%1x" % (
+                cfgdev.pci_capability.domain,
+                cfgdev.pci_capability.bus,
+                cfgdev.pci_capability.slot,
+                cfgdev.pci_capability.function)
+            self.assertEqual(cmp_addr, address)
+
+        pf_xml = """<device>
+  <name>pci_0000_81_00_0</name>
+  <path>/sys/devices/pci0000:80/0000:80:01.0/0000:81:00.0</path>
+  <parent>pci_0000_80_01_0</parent>
+  <driver>
+    <name>ixgbe</name>
+  </driver>
+  <capability type='pci'>
+    <domain>0</domain>
+    <bus>129</bus>
+    <slot>0</slot>
+    <function>0</function>
+    <product id='0x1528'>Ethernet Controller 10-Gigabit X540-AT2</product>
+    <vendor id='0x8086'>Intel Corporation</vendor>
+    <capability type='virt_functions'>
+      <address domain='0x0000' bus='0x81' slot='0x0' function='0x1'/>
+    </capability>
+    <iommuGroup number='40'>
+      <address domain='0x0000' bus='0x81' slot='0x0' function='0x0'/>
+    </iommuGroup>
+    <numa node='0'/>
+    <pci-express>
+      <link validity='cap' port='0' speed='5' width='8'/>
+      <link validity='sta' speed='5' width='8'/>
+    </pci-express>
+  </capability>
+</device>"""
+        vf_xml = """<device>
+  <name>pci_0000_81_00_1</name>
+  <path>/sys/devices/pci0000:80/0000:80:01.0/0000:81:00.1</path>
+  <parent>pci_0000_81_00_0</parent>
+  <driver>
+    <name>ixgbevf</name>
+  </driver>
+  <capability type='pci'>
+    <domain>0</domain>
+    <bus>129</bus>
+    <slot>0</slot>
+    <function>1</function>
+    <product id='0x1515'>X540 Ethernet Controller Virtual Function</product>
+    <vendor id='0x8086'>Intel Corporation</vendor>
+    <capability type='phys_function'>
+      <address domain='0x0000' bus='0x81' slot='0x0' function='0x0'/>
+    </capability>
+    <iommuGroup number='41'>
+      <address domain='0x0000' bus='0x81' slot='0x0' function='0x1'/>
+    </iommuGroup>
+    <numa node='0'/>
+    <pci-express>
+      <link validity='cap' port='0' speed='5' width='8'/>
+      <link validity='sta' speed='5' width='8'/>
+    </pci-express>
+  </capability>
+</device>"""
+
+        # create fake pci devices
+        pci_info = libvirt.HostPCIDevicesInfo(num_pfs=1, num_vfs=1)
+
+        # generate xml for the created pci devices
+        gen_pf = pci_info.get_device_by_name('pci_0000_81_00_0')
+        gen_vf = pci_info.get_device_by_name('pci_0000_81_00_1')
+
+        self.assertXmlEqual(gen_pf.XMLDesc(0), pf_xml)
+        self.assertXmlEqual(gen_vf.XMLDesc(0), vf_xml)
+
+        # parse the generated xml with a libvirt config class and compare
+        # device address
+        _cmp_pci_dev_addr(pf_xml, '0000:81:00.0')
+        _cmp_pci_dev_addr(vf_xml, '0000:81:00.1')
