@@ -17,11 +17,11 @@
 import collections
 
 import mock
+from oslo_utils import units
 from oslo_utils import uuidutils
 from oslo_vmware import exceptions as vexc
 from oslo_vmware.objects import datastore as ds_obj
 from oslo_vmware import pbm
-from oslo_vmware import vim_util as vutil
 
 from nova import exception
 from nova.network import model as network_model
@@ -55,12 +55,12 @@ class VMwareVMUtilTestCase(test.NoDBTestCase):
 
     def _test_get_stats_from_cluster(self, connection_state="connected",
                                      maintenance_mode=False):
-        ManagedObjectRefs = [fake.ManagedObjectReference("host1",
-                                                         "HostSystem"),
-                             fake.ManagedObjectReference("host2",
-                                                         "HostSystem")]
+        ManagedObjectRefs = [fake.ManagedObjectReference("HostSystem",
+                                                         "host1"),
+                             fake.ManagedObjectReference("HostSystem",
+                                                         "host2")]
         hosts = fake._convert_to_array_of_mor(ManagedObjectRefs)
-        respool = fake.ManagedObjectReference("resgroup-11", "ResourcePool")
+        respool = fake.ManagedObjectReference("ResourcePool", "resgroup-11")
         prop_dict = {'host': hosts, 'resourcePool': respool}
 
         hardware = fake.DataObject()
@@ -68,21 +68,32 @@ class VMwareVMUtilTestCase(test.NoDBTestCase):
         hardware.numCpuThreads = 16
         hardware.vendor = "Intel"
         hardware.cpuModel = "Intel(R) Xeon(R)"
+        hardware.memorySize = 4 * units.Gi
 
         runtime_host_1 = fake.DataObject()
         runtime_host_1.connectionState = "connected"
         runtime_host_1.inMaintenanceMode = False
 
+        quickstats_1 = fake.DataObject()
+        quickstats_1.overallMemoryUsage = 512
+
+        quickstats_2 = fake.DataObject()
+        quickstats_2.overallMemoryUsage = 512
+
         runtime_host_2 = fake.DataObject()
         runtime_host_2.connectionState = connection_state
         runtime_host_2.inMaintenanceMode = maintenance_mode
 
-        prop_list_host_1 = [fake.Prop(name="hardware_summary", val=hardware),
-                            fake.Prop(name="runtime_summary",
-                                      val=runtime_host_1)]
-        prop_list_host_2 = [fake.Prop(name="hardware_summary", val=hardware),
-                            fake.Prop(name="runtime_summary",
-                                      val=runtime_host_2)]
+        prop_list_host_1 = [fake.Prop(name="summary.hardware", val=hardware),
+                            fake.Prop(name="summary.runtime",
+                                      val=runtime_host_1),
+                            fake.Prop(name="summary.quickStats",
+                                      val=quickstats_1)]
+        prop_list_host_2 = [fake.Prop(name="summary.hardware", val=hardware),
+                            fake.Prop(name="summary.runtime",
+                                      val=runtime_host_2),
+                            fake.Prop(name="summary.quickStats",
+                                      val=quickstats_2)]
 
         fake_objects = fake.FakeRetrieveResult()
         fake_objects.add_object(fake.ObjectContent("prop_list_host1",
@@ -90,29 +101,27 @@ class VMwareVMUtilTestCase(test.NoDBTestCase):
         fake_objects.add_object(fake.ObjectContent("prop_list_host1",
                                                    prop_list_host_2))
 
-        respool_resource_usage = fake.DataObject()
-        respool_resource_usage.maxUsage = 5368709120
-        respool_resource_usage.overallUsage = 2147483648
-
         def fake_call_method(*args):
             if "get_object_properties_dict" in args:
                 return prop_dict
             elif "get_properties_for_a_collection_of_objects" in args:
                 return fake_objects
             else:
-                return respool_resource_usage
+                raise Exception('unexpected method call')
 
         session = fake.FakeSession()
         with mock.patch.object(session, '_call_method', fake_call_method):
             result = vm_util.get_stats_from_cluster(session, "cluster1")
-            mem_info = {}
             if connection_state == "connected" and not maintenance_mode:
-                vcpus = 32
+                num_hosts = 2
             else:
-                vcpus = 16
-            mem_info['total'] = 5120
-            mem_info['free'] = 3072
-            expected_stats = {'vcpus': vcpus, 'mem': mem_info}
+                num_hosts = 1
+            expected_stats = {'cpu': {'vcpus': num_hosts * 16,
+                                      'max_vcpus_per_host': 16},
+                              'mem': {'total': num_hosts * 4096,
+                                      'free': num_hosts * 4096 -
+                                              num_hosts * 512,
+                                      'max_mem_mb_per_host': 4096}}
             self.assertEqual(expected_stats, result)
 
     def test_get_stats_from_cluster_hosts_connected_and_active(self):
@@ -171,6 +180,67 @@ class VMwareVMUtilTestCase(test.NoDBTestCase):
 
         self.assertEqual(expected, result)
 
+    def test_update_vif_spec_opaque_net(self):
+        fake_factory = fake.FakeFactory()
+        vif_info = {'network_name': 'br100',
+            'mac_address': '00:00:00:ca:fe:01',
+            'network_ref': {'type': 'OpaqueNetwork',
+                            'network-id': 'fake-network-id',
+                            'network-type': 'fake-net',
+                            'use-external-id': False},
+            'iface_id': 7,
+            'vif_model': 'VirtualE1000'}
+        device = fake_factory.create('ns0:VirtualDevice')
+        actual = vm_util.update_vif_spec(fake_factory, vif_info, device)
+        spec = fake_factory.create('ns0:VirtualDeviceConfigSpec')
+        spec.device = fake_factory.create('ns0:VirtualDevice')
+        spec.device.backing = fake_factory.create(
+            'ns0:VirtualEthernetCardOpaqueNetworkBackingInfo')
+        spec.device.backing.opaqueNetworkType = 'fake-net'
+        spec.device.backing.opaqueNetworkId = 'fake-network-id'
+        spec.operation = 'edit'
+        self.assertEqual(spec, actual)
+
+    def test_update_vif_spec_dvpg(self):
+        fake_factory = fake.FakeFactory()
+        vif_info = {'network_name': 'br100',
+            'mac_address': '00:00:00:ca:fe:01',
+            'network_ref': {'type': 'DistributedVirtualPortgroup',
+                            'dvsw': 'fake-network-id',
+                            'dvpg': 'fake-group'},
+            'iface_id': 7,
+            'vif_model': 'VirtualE1000'}
+        device = fake_factory.create('ns0:VirtualDevice')
+        actual = vm_util.update_vif_spec(fake_factory, vif_info, device)
+        spec = fake_factory.create('ns0:VirtualDeviceConfigSpec')
+        spec.device = fake_factory.create('ns0:VirtualDevice')
+        spec.device.backing = fake_factory.create(
+            'ns0:VirtualEthernetCardDistributedVirtualPortBackingInfo')
+        spec.device.backing.port = fake_factory.create(
+            'ns0:DistributedVirtualSwitchPortConnection')
+        spec.device.backing.port.portgroupKey = 'fake-group'
+        spec.device.backing.port.switchUuid = 'fake-network-id'
+        spec.operation = 'edit'
+        self.assertEqual(spec, actual)
+
+    def test_update_vif_spec_network(self):
+        fake_factory = fake.FakeFactory()
+        vif_info = {'network_name': 'br100',
+            'mac_address': '00:00:00:ca:fe:01',
+            'network_ref': {'type': 'Network',
+                            'name': 'net1'},
+            'iface_id': 7,
+            'vif_model': 'VirtualE1000'}
+        device = fake_factory.create('ns0:VirtualDevice')
+        actual = vm_util.update_vif_spec(fake_factory, vif_info, device)
+        spec = fake_factory.create('ns0:VirtualDeviceConfigSpec')
+        spec.device = fake_factory.create('ns0:VirtualDevice')
+        spec.device.backing = fake_factory.create(
+            'ns0:VirtualEthernetCardNetworkBackingInfo')
+        spec.device.backing.deviceName = 'br100'
+        spec.operation = 'edit'
+        self.assertEqual(spec, actual)
+
     def test_get_cdrom_attach_config_spec(self):
         fake_factory = fake.FakeFactory()
         datastore = fake.Datastore()
@@ -433,9 +503,9 @@ class VMwareVMUtilTestCase(test.NoDBTestCase):
         self.assertEqual(1, controller_spec.device.busNumber)
 
     def test_get_vnc_config_spec(self):
+        self.flags(vnc_keymap='en-ie', group='vmware')
         fake_factory = fake.FakeFactory()
-        result = vm_util.get_vnc_config_spec(fake_factory,
-                                             7)
+        result = vm_util.get_vnc_config_spec(fake_factory, 7)
         expected = fake_factory.create('ns0:VirtualMachineConfigSpec')
         expected.extraConfig = []
 
@@ -450,7 +520,7 @@ class VMwareVMUtilTestCase(test.NoDBTestCase):
         expected.extraConfig.append(remote_display_vnc_port)
 
         remote_display_vnc_keymap = fake_factory.create('ns0:OptionValue')
-        remote_display_vnc_keymap.value = 'en-us'
+        remote_display_vnc_keymap.value = 'en-ie'
         remote_display_vnc_keymap.key = 'RemoteDisplay.vnc.keyMap'
         expected.extraConfig.append(remote_display_vnc_keymap)
 
@@ -874,6 +944,48 @@ class VMwareVMUtilTestCase(test.NoDBTestCase):
 
         self.assertEqual(expected, result)
 
+    def test_get_vm_create_spec_with_firmware(self):
+        extra_specs = vm_util.ExtraSpecs(firmware='efi')
+        fake_factory = fake.FakeFactory()
+        result = vm_util.get_vm_create_spec(fake_factory,
+                                            self._instance,
+                                            'fake-datastore', [],
+                                            extra_specs)
+        expected = fake_factory.create('ns0:VirtualMachineConfigSpec')
+        expected.name = self._instance.uuid
+        expected.instanceUuid = self._instance.uuid
+        expected.deviceChange = []
+        expected.numCPUs = 2
+
+        expected.version = None
+        expected.memoryMB = 2048
+        expected.guestId = 'otherGuest'
+        expected.firmware = 'efi'
+        expected.extraConfig = []
+
+        extra_config = fake_factory.create("ns0:OptionValue")
+        extra_config.value = self._instance.uuid
+        extra_config.key = 'nvp.vm-uuid'
+        expected.extraConfig.append(extra_config)
+        extra_config = fake_factory.create("ns0:OptionValue")
+        extra_config.value = True
+        extra_config.key = 'disk.EnableUUID'
+        expected.extraConfig.append(extra_config)
+        expected.files = fake_factory.create('ns0:VirtualMachineFileInfo')
+        expected.files.vmPathName = '[fake-datastore]'
+
+        expected.managedBy = fake_factory.create('ns0:ManagedByInfo')
+        expected.managedBy.extensionKey = 'org.openstack.compute'
+        expected.managedBy.type = 'instance'
+
+        expected.tools = fake_factory.create('ns0:ToolsConfigInfo')
+        expected.tools.afterPowerOn = True
+        expected.tools.afterResume = True
+        expected.tools.beforeGuestReboot = True
+        expected.tools.beforeGuestShutdown = True
+        expected.tools.beforeGuestStandby = True
+        self.assertEqual(expected, result)
+
     def test_create_vm(self):
 
         def fake_call_method(module, method, *args, **kwargs):
@@ -1446,6 +1558,96 @@ class VMwareVMUtilTestCase(test.NoDBTestCase):
                                             profile_spec='fake_profile_spec')
         self.assertEqual(['fake_profile_spec'], create_spec.vmProfile)
 
+    def test_vm_create_spec_with_multi_vifs(self):
+        datastore = ds_obj.Datastore('fake-ds-ref', 'fake-ds-name')
+        extra_specs = vm_util.ExtraSpecs()
+        vif_info = {'network_name': 'br100',
+            'mac_address': '00:00:00:ca:fe:01',
+            'network_ref': {'type': 'DistributedVirtualPortgroup',
+                            'dvsw': 'fake-network-id1',
+                            'dvpg': 'fake-group'},
+            'iface_id': 7,
+            'vif_model': 'VirtualE1000'}
+        vif_info2 = {'network_name': 'br101',
+            'mac_address': '00:00:00:ca:fe:02',
+            'network_ref': {'type': 'DistributedVirtualPortgroup',
+                            'dvsw': 'fake-network-id2',
+                            'dvpg': 'fake-group'},
+            'iface_id': 7,
+            'vif_model': 'VirtualE1000'}
+        create_spec = vm_util.get_vm_create_spec(fake.FakeFactory(),
+                                            self._instance,
+                                            datastore.name,
+                                            [vif_info, vif_info2],
+                                            extra_specs)
+
+        port = 'ns0:DistributedVirtualSwitchPortConnection'
+        backing = 'ns0:VirtualEthernetCardDistributedVirtualPortBackingInfo'
+
+        device_changes = []
+        fake_factory = fake.FakeFactory()
+        device_change = fake_factory.create('ns0:VirtualDeviceConfigSpec')
+        device_change.operation = 'add'
+
+        device = fake_factory.create('ns0:VirtualE1000')
+        device.key = -47
+        device.macAddress = '00:00:00:ca:fe:01'
+        device.addressType = 'manual'
+        device.wakeOnLanEnabled = True
+
+        device.backing = fake_factory.create(backing)
+        device.backing.port = fake_factory.create(port)
+        device.backing.port.portgroupKey = 'fake-group'
+        device.backing.port.switchUuid = 'fake-network-id1'
+
+        device.resourceAllocation = fake_factory.create(
+            'ns0:VirtualEthernetCardResourceAllocation')
+        device.resourceAllocation.share = fake_factory.create(
+            'ns0:SharesInfo')
+        device.resourceAllocation.share.level = None
+        device.resourceAllocation.share.shares = None
+
+        connectable = fake_factory.create('ns0:VirtualDeviceConnectInfo')
+        connectable.allowGuestControl = True
+        connectable.connected = True
+        connectable.startConnected = True
+        device.connectable = connectable
+        device_change.device = device
+
+        device_changes.append(device_change)
+
+        device_change = fake_factory.create('ns0:VirtualDeviceConfigSpec')
+        device_change.operation = 'add'
+
+        device = fake_factory.create('ns0:VirtualE1000')
+        device.key = -48
+        device.macAddress = '00:00:00:ca:fe:02'
+        device.addressType = 'manual'
+        device.wakeOnLanEnabled = True
+
+        device.backing = fake_factory.create(backing)
+        device.backing.port = fake_factory.create(port)
+        device.backing.port.portgroupKey = 'fake-group'
+        device.backing.port.switchUuid = 'fake-network-id2'
+
+        device.resourceAllocation = fake_factory.create(
+            'ns0:VirtualEthernetCardResourceAllocation')
+        device.resourceAllocation.share = fake_factory.create(
+            'ns0:SharesInfo')
+        device.resourceAllocation.share.level = None
+        device.resourceAllocation.share.shares = None
+
+        connectable = fake_factory.create('ns0:VirtualDeviceConnectInfo')
+        connectable.allowGuestControl = True
+        connectable.connected = True
+        connectable.startConnected = True
+        device.connectable = connectable
+        device_change.device = device
+
+        device_changes.append(device_change)
+
+        self.assertEqual(device_changes, create_spec.deviceChange)
+
     @mock.patch.object(pbm, 'get_profile_id_by_name')
     def test_get_storage_profile_spec(self, mock_retrieve_profile_id):
         fake_profile_id = fake.DataObject()
@@ -1712,182 +1914,41 @@ class VMwareVMUtilTestCase(test.NoDBTestCase):
                 "get_object_property", vm_ref, "config.hardware.device")
             self.assertEqual(swap_disk, device)
 
-    def test_create_folder_with_empty_vmfolder(self):
-        """Test create_folder when the datacenter vmFolder is empty"""
+    def test_create_folder(self):
+        """Test create_folder when the folder doesn't exist"""
         child_folder = mock.sentinel.child_folder
         session = fake.FakeSession()
         with mock.patch.object(session, '_call_method',
-                               side_effect=[None, child_folder]):
+                               side_effect=[child_folder]):
             parent_folder = mock.sentinel.parent_folder
             parent_folder.value = 'parent-ref'
             child_name = 'child_folder'
             ret = vm_util.create_folder(session, parent_folder, child_name)
 
             self.assertEqual(child_folder, ret)
-            expected_calls = [mock.call(vutil, 'get_object_property',
-                                        parent_folder,
-                                       'childEntity'),
-                              mock.call(session.vim, 'CreateFolder',
-                                        parent_folder, name=child_name)]
-            self.assertEqual(expected_calls,
-                             session._call_method.call_args_list)
+            session._call_method.assert_called_once_with(session.vim,
+                                                         'CreateFolder',
+                                                         parent_folder,
+                                                         name=child_name)
 
-    def test_create_folder_not_present(self):
-        """Test create_folder when child not present."""
-        prop_val = mock.Mock()
-        prop_val.ManagedObjectReference = []
-        child_folder = mock.sentinel.child_folder
+    def test_create_folder_duplicate_name(self):
+        """Test create_folder when the folder already exists"""
         session = fake.FakeSession()
-        with mock.patch.object(session, '_call_method',
-                               side_effect=[prop_val, child_folder]):
-            child_name = 'child_folder'
-            parent_folder = mock.sentinel.parent_folder
-            parent_folder.value = 'parent-ref'
-            ret = vm_util.create_folder(session, parent_folder, child_name)
-            self.assertEqual(child_folder, ret)
-            expected_invoke_api = [mock.call(vutil, 'get_object_property',
-                                             parent_folder,
-                                             'childEntity'),
-                                   mock.call(session.vim, 'CreateFolder',
-                                             parent_folder, name=child_name)]
-            self.assertEqual(expected_invoke_api,
-                             session._call_method.mock_calls)
-
-    def test_create_folder_already_present(self):
-        """Test create_folder when child already present."""
-        parent_folder = mock.sentinel.parent_folder
-        child_name = 'child_folder'
-        prop_val = mock.Mock()
-        child_entity_1 = mock.Mock()
-        child_entity_1._type = 'Folder'
-        child_entity_1_name = 'SomeOtherName'
-        child_entity_2 = mock.Mock()
-        child_entity_2._type = 'Folder'
-        child_entity_2_name = 'AnotherName'
-        child_entity_3 = mock.Mock()
-        child_entity_3._type = 'Folder'
-        child_entity_3_name = child_name
-        prop_val.ManagedObjectReference = [child_entity_1, child_entity_2,
-                                           child_entity_3]
-        session = fake.FakeSession()
-        with mock.patch.object(session, '_call_method',
-                               side_effect=[prop_val,
-                                            child_entity_1_name,
-                                            child_entity_2_name,
-                                            child_entity_3_name]):
-            ret = vm_util.create_folder(session, parent_folder, child_name)
-            self.assertEqual(child_entity_3, ret)
-            expected_invoke_api = [mock.call(vutil, 'get_object_property',
-                                             parent_folder,
-                                             'childEntity'),
-                                   mock.call(vutil, 'get_object_property',
-                                             child_entity_1,
-                                             'name'),
-                                   mock.call(vutil, 'get_object_property',
-                                             child_entity_2,
-                                             'name'),
-                                   mock.call(vutil, 'get_object_property',
-                                             child_entity_3,
-                                             'name')]
-            self.assertEqual(expected_invoke_api,
-                             session._call_method.mock_calls)
-
-    def test_create_folder_with_duplicate_name(self):
-        parent_folder = mock.sentinel.parent_folder
-        parent_folder.value = 'parent-ref'
-        child_name = 'child_folder'
-
-        prop_val_1 = mock.Mock()
-        prop_val_1.ManagedObjectReference = []
-
-        child_entity_2 = mock.Mock()
-        child_entity_2._type = 'Folder'
-        prop_val_2 = mock.Mock()
-        prop_val_2.ManagedObjectReference = [child_entity_2]
-        child_entity_2_name = child_name
-
         details = {'object': 'folder-1'}
         duplicate_exception = vexc.DuplicateName(details=details)
-
-        session = fake.FakeSession()
         with mock.patch.object(session, '_call_method',
-                               side_effect=[prop_val_1,
-                                            duplicate_exception,
-                                            prop_val_2,
-                                            child_entity_2_name]):
+                               side_effect=[duplicate_exception]):
+            parent_folder = mock.sentinel.parent_folder
+            parent_folder.value = 'parent-ref'
+            child_name = 'child_folder'
             ret = vm_util.create_folder(session, parent_folder, child_name)
-            self.assertEqual(child_entity_2._type, ret._type)
-            expected_invoke_api = [mock.call(vutil, 'get_object_property',
-                                             parent_folder,
-                                             'childEntity'),
-                                   mock.call(session.vim, 'CreateFolder',
-                                             parent_folder, name=child_name)]
-            self.assertEqual(expected_invoke_api,
-                             session._call_method.mock_calls)
 
-    def test_get_folder_does_not_exist(self):
-        session = fake.FakeSession()
-        with mock.patch.object(session, '_call_method',
-                               return_value=None):
-            ret = vm_util._get_folder(session, 'fake-parent', 'fake-name')
-            self.assertIsNone(ret)
-            expected_invoke_api = [mock.call(vutil, 'get_object_property',
-                                             'fake-parent',
-                                             'childEntity')]
-            self.assertEqual(expected_invoke_api,
-                             session._call_method.mock_calls)
-
-    def test_get_folder_child_entry_not_folder(self):
-        child_entity = mock.Mock()
-        child_entity._type = 'NotFolder'
-        prop_val = mock.Mock()
-        prop_val.ManagedObjectReference = [child_entity]
-        session = fake.FakeSession()
-        with mock.patch.object(session, '_call_method',
-                               return_value=prop_val):
-            ret = vm_util._get_folder(session, 'fake-parent', 'fake-name')
-            self.assertIsNone(ret)
-            expected_invoke_api = [mock.call(vutil, 'get_object_property',
-                                             'fake-parent',
-                                             'childEntity')]
-            self.assertEqual(expected_invoke_api,
-                             session._call_method.mock_calls)
-
-    def test_get_folder_child_entry_not_matched(self):
-        child_entity = mock.Mock()
-        child_entity._type = 'Folder'
-        prop_val = mock.Mock()
-        prop_val.ManagedObjectReference = [child_entity]
-        session = fake.FakeSession()
-        with mock.patch.object(session, '_call_method',
-                               side_effect=[prop_val, 'fake-1-name']):
-            ret = vm_util._get_folder(session, 'fake-parent', 'fake-name')
-            self.assertIsNone(ret)
-            expected_invoke_api = [mock.call(vutil, 'get_object_property',
-                                             'fake-parent',
-                                             'childEntity'),
-                                   mock.call(vutil, 'get_object_property',
-                                             child_entity, 'name')]
-            self.assertEqual(expected_invoke_api,
-                             session._call_method.mock_calls)
-
-    def test_get_folder_child_entry_matched(self):
-        child_entity = mock.Mock()
-        child_entity._type = 'Folder'
-        prop_val = mock.Mock()
-        prop_val.ManagedObjectReference = [child_entity]
-        session = fake.FakeSession()
-        with mock.patch.object(session, '_call_method',
-                               side_effect=[prop_val, 'fake-name']):
-            ret = vm_util._get_folder(session, 'fake-parent', 'fake-name')
-            self.assertEqual(ret, child_entity)
-            expected_invoke_api = [mock.call(vutil, 'get_object_property',
-                                             'fake-parent',
-                                             'childEntity'),
-                                   mock.call(vutil, 'get_object_property',
-                                             child_entity, 'name')]
-            self.assertEqual(expected_invoke_api,
-                             session._call_method.mock_calls)
+            self.assertEqual('Folder', ret._type)
+            self.assertEqual('folder-1', ret.value)
+            session._call_method.assert_called_once_with(session.vim,
+                                                         'CreateFolder',
+                                                         parent_folder,
+                                                         name=child_name)
 
     def test_folder_path_ref_cache(self):
         path = 'OpenStack/Project (e2b86092bf064181ade43deb3188f8e4)'
