@@ -27,115 +27,74 @@ import os
 import random
 import tempfile
 
-if os.name != 'nt':
-    import crypt
-
 from oslo_concurrency import processutils
 from oslo_log import log as logging
 from oslo_serialization import jsonutils
-from oslo_utils import units
 
 import nova.conf
 from nova import exception
 from nova.i18n import _
-from nova.i18n import _LE
-from nova.i18n import _LW
-from nova import utils
+import nova.privsep.fs
+import nova.privsep.libvirt
 from nova.virt.disk.mount import api as mount
 from nova.virt.disk.vfs import api as vfs
 from nova.virt.image import model as imgmodel
 from nova.virt import images
 
+if os.name != 'nt':
+    import crypt
+
 
 LOG = logging.getLogger(__name__)
 
 CONF = nova.conf.CONF
 
-_MKFS_COMMAND = {}
-_DEFAULT_MKFS_COMMAND = None
 
-FS_FORMAT_EXT2 = "ext2"
-FS_FORMAT_EXT3 = "ext3"
-FS_FORMAT_EXT4 = "ext4"
-FS_FORMAT_XFS = "xfs"
-FS_FORMAT_NTFS = "ntfs"
-FS_FORMAT_VFAT = "vfat"
+# NOTE(mikal): Here as a transition step
+SUPPORTED_FS_TO_EXTEND = nova.privsep.fs.SUPPORTED_FS_TO_EXTEND
 
-SUPPORTED_FS_TO_EXTEND = (
-    FS_FORMAT_EXT2,
-    FS_FORMAT_EXT3,
-    FS_FORMAT_EXT4)
-
-_DEFAULT_FILE_SYSTEM = FS_FORMAT_VFAT
-_DEFAULT_FS_BY_OSTYPE = {'linux': FS_FORMAT_EXT4,
-                         'windows': FS_FORMAT_NTFS}
 
 for s in CONF.virt_mkfs:
     # NOTE(yamahata): mkfs command may includes '=' for its options.
     #                 So item.partition('=') doesn't work here
     os_type, mkfs_command = s.split('=', 1)
     if os_type:
-        _MKFS_COMMAND[os_type] = mkfs_command
-    if os_type == 'default':
-        _DEFAULT_MKFS_COMMAND = mkfs_command
-
-
-def get_fs_type_for_os_type(os_type):
-    return os_type if _MKFS_COMMAND.get(os_type) else 'default'
-
-
-def get_file_extension_for_os_type(os_type, specified_fs=None):
-    mkfs_command = _MKFS_COMMAND.get(os_type, _DEFAULT_MKFS_COMMAND)
-    if mkfs_command:
-        extension = mkfs_command
-    else:
-        if not specified_fs:
-            specified_fs = CONF.default_ephemeral_format
-            if not specified_fs:
-                specified_fs = _DEFAULT_FS_BY_OSTYPE.get(os_type,
-                                                         _DEFAULT_FILE_SYSTEM)
-        extension = specified_fs
-    return utils.get_hash_str(extension)[:7]
+        nova.privsep.fs.load_mkfs_command(os_type, mkfs_command)
 
 
 def mkfs(os_type, fs_label, target, run_as_root=True, specified_fs=None):
-    """Format a file or block device using
-       a user provided command for each os type.
-       If user has not provided any configuration,
-       format type will be used according to a
-       default_ephemeral_format configuration
-       or a system defaults.
-    """
-
-    mkfs_command = (_MKFS_COMMAND.get(os_type, _DEFAULT_MKFS_COMMAND) or
-                    '') % {'fs_label': fs_label, 'target': target}
-    if mkfs_command:
-        utils.execute(*mkfs_command.split(), run_as_root=run_as_root)
-    else:
-        if not specified_fs:
-            specified_fs = CONF.default_ephemeral_format
-            if not specified_fs:
-                specified_fs = _DEFAULT_FS_BY_OSTYPE.get(os_type,
-                                                         _DEFAULT_FILE_SYSTEM)
-
-        utils.mkfs(specified_fs, target, fs_label, run_as_root=run_as_root)
+    nova.privsep.fs.configurable_mkfs(
+        os_type, fs_label, target, run_as_root,
+        CONF.default_ephemeral_format, specified_fs)
 
 
 def resize2fs(image, check_exit_code=False, run_as_root=False):
+    # NOTE(mikal): note that the check_exit_code kwarg here only refers to
+    # resize2fs, not the precursor e2fsck. Yes, I agree it's confusing.
     try:
-        utils.execute('e2fsck',
-                      '-fp',
-                      image,
-                      check_exit_code=[0, 1, 2],
-                      run_as_root=run_as_root)
+        if run_as_root:
+            nova.privsep.fs.e2fsck(image)
+        else:
+            nova.privsep.fs.unprivileged_e2fsck(image)
+
     except processutils.ProcessExecutionError as exc:
         LOG.debug("Checking the file system with e2fsck has failed, "
                   "the resize will be aborted. (%s)", exc)
+
     else:
-        utils.execute('resize2fs',
-                      image,
-                      check_exit_code=check_exit_code,
-                      run_as_root=run_as_root)
+        if run_as_root:
+            nova.privsep.fs.resize2fs(image, check_exit_code)
+        else:
+            nova.privsep.fs.unprivileged_resize2fs(image, check_exit_code)
+
+
+def get_disk_info(path):
+    """Get QEMU info of a disk image
+
+    :param path: Path to the disk image
+    :returns: oslo_utils.imageutils.QemuImgInfo object for the image.
+    """
+    return images.qemu_img_info(path)
 
 
 def get_disk_size(path):
@@ -159,20 +118,14 @@ def extend(image, size):
     if not isinstance(image, imgmodel.LocalImage):
         return
 
-    if (image.format == imgmodel.FORMAT_PLOOP):
-        if not can_resize_image(image.path, size):
-            return
-
-        utils.execute('prl_disk_tool', 'resize',
-                      '--size', '%dM' % (size // units.Mi),
-                      '--resize_partition',
-                      '--hdd', image.path, run_as_root=True)
-        return
-
     if not can_resize_image(image.path, size):
         return
 
-    utils.execute('qemu-img', 'resize', image.path, size)
+    if (image.format == imgmodel.FORMAT_PLOOP):
+        nova.privsep.libvirt.ploop_resize(image.path, size)
+        return
+
+    processutils.execute('qemu-img', 'resize', image.path, size)
 
     if (image.format != imgmodel.FORMAT_RAW and
         not CONF.resize_fs_using_block_device):
@@ -243,8 +196,8 @@ def is_image_extendable(image):
             # provides a bad configuration for libguestfs reported in
             # the bug lp#1413142. When resolved we should remove this
             # except to let the error to be propagated.
-            LOG.warning(_LW('Unable to mount image %(image)s with '
-                            'error %(error)s. Cannot resize.'),
+            LOG.warning('Unable to mount image %(image)s with '
+                        'error %(error)s. Cannot resize.',
                         {'image': image, 'error': e})
         finally:
             if fs is not None:
@@ -254,7 +207,7 @@ def is_image_extendable(image):
     else:
         # For raw, we can directly inspect the file system
         try:
-            utils.execute('e2label', image.path)
+            processutils.execute('e2label', image.path)
         except processutils.ProcessExecutionError as e:
             LOG.debug('Unable to determine label for image %(image)s with '
                       'error %(error)s. Cannot resize.',
@@ -407,8 +360,8 @@ def inject_data(image, key=None, net=None, metadata=None, admin_password=None,
             inject_val = items[inject]
             if inject_val:
                 raise
-        LOG.warning(_LW('Ignoring error injecting data into image %(image)s '
-                        '(%(e)s)'), {'image': image, 'e': e})
+        LOG.warning('Ignoring error injecting data into image %(image)s '
+                    '(%(e)s)', {'image': image, 'e': e})
         return False
 
     try:
@@ -432,8 +385,8 @@ def setup_container(image, container_dir):
     img = _DiskImage(image=image, mount_dir=container_dir)
     dev = img.mount()
     if dev is None:
-        LOG.error(_LE("Failed to mount container filesystem '%(image)s' "
-                    "on '%(target)s': %(errors)s"),
+        LOG.error("Failed to mount container filesystem '%(image)s' "
+                  "on '%(target)s': %(errors)s",
                   {"image": img, "target": container_dir,
                    "errors": img.errors})
         raise exception.NovaException(img.errors)
@@ -455,17 +408,15 @@ def teardown_container(container_dir, container_root_device=None):
         if container_root_device:
             if 'loop' in container_root_device:
                 LOG.debug("Release loop device %s", container_root_device)
-                utils.execute('losetup', '--detach', container_root_device,
-                              run_as_root=True, attempts=3)
+                nova.privsep.fs.loopremove(container_root_device)
             elif 'nbd' in container_root_device:
                 LOG.debug('Release nbd device %s', container_root_device)
-                utils.execute('qemu-nbd', '-d', container_root_device,
-                              run_as_root=True)
+                nova.privsep.fs.nbd_disconnect(container_root_device)
             else:
-                LOG.debug('No release necessary for block device %s' %
+                LOG.debug('No release necessary for block device %s',
                           container_root_device)
     except Exception:
-        LOG.exception(_LE('Failed to teardown container filesystem'))
+        LOG.exception('Failed to teardown container filesystem')
 
 
 def clean_lxc_namespace(container_dir):
@@ -478,7 +429,7 @@ def clean_lxc_namespace(container_dir):
         img = _DiskImage(image=None, mount_dir=container_dir)
         img.umount()
     except Exception:
-        LOG.exception(_LE('Failed to umount container filesystem'))
+        LOG.exception('Failed to umount container filesystem')
 
 
 def inject_data_into_fs(fs, key, net, metadata, admin_password, files,
@@ -511,8 +462,8 @@ def inject_data_into_fs(fs, key, net, metadata, admin_password, files,
             except Exception as e:
                 if inject in mandatory:
                     raise
-                LOG.warning(_LW('Ignoring error injecting %(inject)s into '
-                                'image (%(e)s)'), {'inject': inject, 'e': e})
+                LOG.warning('Ignoring error injecting %(inject)s into '
+                            'image (%(e)s)', {'inject': inject, 'e': e})
                 status = False
     return status
 
@@ -521,8 +472,8 @@ def _inject_files_into_fs(files, fs):
     for (path, contents) in files:
         # NOTE(wangpan): Ensure the parent dir of injecting file exists
         parent_dir = os.path.dirname(path)
-        if (len(parent_dir) > 0 and parent_dir != "/"
-                and not fs.has_file(parent_dir)):
+        if (len(parent_dir) > 0 and parent_dir != "/" and
+                not fs.has_file(parent_dir)):
             fs.make_path(parent_dir)
             fs.set_ownership(parent_dir, "root", "root")
             fs.set_permissions(parent_dir, 0o744)
@@ -701,8 +652,8 @@ def _set_passwd(username, admin_passwd, passwd_data, shadow_data):
         msg = _('User %(username)s not found in password file.')
         raise exception.NovaException(msg % username)
 
-    # update password in the shadow file.It's an error if the
-    # the user doesn't exist.
+    # update password in the shadow file. It's an error if the
+    # user doesn't exist.
     new_shadow = list()
     found = False
     for entry in s_file:
