@@ -28,11 +28,10 @@ import time
 from oslo_concurrency import lockutils
 from oslo_concurrency import processutils
 from oslo_log import log as logging
+from oslo_utils import encodeutils
 
 import nova.conf
-from nova.i18n import _LE
-from nova.i18n import _LI
-from nova.i18n import _LW
+import nova.privsep.path
 from nova import utils
 from nova.virt import imagecache
 from nova.virt.libvirt import utils as libvirt_utils
@@ -53,33 +52,6 @@ def get_cache_fname(image_id):
     return hashlib.sha1(image_id.encode('utf-8')).hexdigest()
 
 
-def get_info_filename(base_path):
-    """Construct a filename for storing additional information about a base
-    image.
-
-    Returns a filename.
-    """
-
-    base_file = os.path.basename(base_path)
-    return (CONF.libvirt.image_info_filename_pattern
-            % {'image': base_file})
-
-
-def is_valid_info_file(path):
-    """Test if a given path matches the pattern for info files."""
-
-    digest_size = hashlib.sha1().digestsize * 2
-    regexp = (CONF.libvirt.image_info_filename_pattern
-              % {'image': ('([0-9a-f]{%(digest_size)d}|'
-                           '[0-9a-f]{%(digest_size)d}_sm|'
-                           '[0-9a-f]{%(digest_size)d}_[0-9]+)'
-                           % {'digest_size': digest_size})})
-    m = re.match(regexp, path)
-    if m:
-        return True
-    return False
-
-
 class ImageCacheManager(imagecache.ImageCacheManager):
     def __init__(self):
         super(ImageCacheManager, self).__init__()
@@ -122,31 +94,9 @@ class ImageCacheManager(imagecache.ImageCacheManager):
         self.unexplained_images, self.originals, and self.back_swap_images.
         """
 
-        digest_size = hashlib.sha1().digestsize * 2
+        digest_size = hashlib.sha1().digest_size * 2
         for ent in os.listdir(base_dir):
-            path = os.path.join(base_dir, ent)
-            if is_valid_info_file(path):
-                # TODO(mdbooth): In Newton we ignore these files, because if
-                # we're on shared storage they may be in use by a pre-Newton
-                # compute host. However, we have already removed all uses of
-                # these files in Newton, so once we can be sure that all
-                # compute hosts are running at least Newton (i.e. in  Ocata),
-                # we can be sure that nothing is using info files any more.
-                # Therefore in Ocata, we should update this to simply delete
-                # these files here, i.e.:
-                #   os.unlink(path)
-                #
-                # This will obsolete the code to cleanup these files in
-                # _remove_old_enough_file, so when updating this code to
-                # delete immediately, the cleanup code in
-                # _remove_old_enough_file can be removed.
-                #
-                # This cleanup code will delete all info files the first
-                # time it runs in Ocata, which means we can delete this
-                # block entirely in P.
-                pass
-
-            elif len(ent) == digest_size:
+            if len(ent) == digest_size:
                 self._store_image(base_dir, ent, original=True)
 
             elif len(ent) > digest_size + 2 and ent[digest_size] == '_':
@@ -183,16 +133,15 @@ class ImageCacheManager(imagecache.ImageCacheManager):
                     if backing_file:
                         backing_path = os.path.join(
                             CONF.instances_path,
-                            CONF.image_cache_subdirectory_name,
+                            CONF.image_cache.subdirectory_name,
                             backing_file)
                         if backing_path not in inuse_images:
                             inuse_images.append(backing_path)
 
                         if backing_path in self.unexplained_images:
-                            LOG.warning(_LW('Instance %(instance)s is using a '
-                                         'backing file %(backing)s which '
-                                         'does not appear in the image '
-                                         'service'),
+                            LOG.warning('Instance %(instance)s is using a '
+                                        'backing file %(backing)s which '
+                                        'does not appear in the image service',
                                         {'instance': ent,
                                          'backing': backing_file})
                             self.unexplained_images.remove(backing_path)
@@ -201,8 +150,7 @@ class ImageCacheManager(imagecache.ImageCacheManager):
     def _find_base_file(self, base_dir, fingerprint):
         """Find the base file matching this fingerprint.
 
-        Yields the name of the base file, a boolean which is True if the image
-        is "small", and a boolean which indicates if this is a resized image.
+        Yields the name of a base file which exists.
         Note that it is possible for more than one yield to result from this
         check.
 
@@ -211,19 +159,19 @@ class ImageCacheManager(imagecache.ImageCacheManager):
         # The original file from glance
         base_file = os.path.join(base_dir, fingerprint)
         if os.path.exists(base_file):
-            yield base_file, False, False
+            yield base_file
 
         # An older naming style which can be removed sometime after Folsom
         base_file = os.path.join(base_dir, fingerprint + '_sm')
         if os.path.exists(base_file):
-            yield base_file, True, False
+            yield base_file
 
-        # Resized images
+        # Resized images (also legacy)
         resize_re = re.compile('.*/%s_[0-9]+$' % fingerprint)
         for img in self.unexplained_images:
             m = resize_re.match(img)
             if m:
-                yield img, False, True
+                yield img
 
     @staticmethod
     def _get_age_of_file(base_file):
@@ -254,32 +202,17 @@ class ImageCacheManager(imagecache.ImageCacheManager):
             if not exists or age < maxage:
                 return
 
-            LOG.info(_LI('Removing base or swap file: %s'), base_file)
+            LOG.info('Removing base or swap file: %s', base_file)
             try:
                 os.remove(base_file)
-
-                # TODO(mdbooth): We have removed all uses of info files in
-                # Newton and we no longer create them, but they may still
-                # exist from before we upgraded, and they may still be
-                # created by older compute hosts if we're on shared storage.
-                # While there may still be pre-Newton computes writing here,
-                # the only safe place to delete info files is here,
-                # when deleting the cache entry. Once we can be sure that
-                # all computes are running at least Newton (i.e. in Ocata),
-                # we can delete these files unconditionally during the
-                # periodic task, which will make this code obsolete.
-                signature = get_info_filename(base_file)
-                if os.path.exists(signature):
-                    os.remove(signature)
             except OSError as e:
-                LOG.error(_LE('Failed to remove %(base_file)s, '
-                              'error was %(error)s'),
+                LOG.error('Failed to remove %(base_file)s, '
+                          'error was %(error)s',
                           {'base_file': base_file,
                            'error': e})
 
         if age < maxage:
-            LOG.info(_LI('Base or swap file too young to remove: %s'),
-                         base_file)
+            LOG.info('Base or swap file too young to remove: %s', base_file)
         else:
             _inner_remove_old_enough_file()
             if remove_lock:
@@ -299,69 +232,33 @@ class ImageCacheManager(imagecache.ImageCacheManager):
 
     def _remove_swap_file(self, base_file):
         """Remove a single swap base file if it is old enough."""
-        maxage = CONF.remove_unused_original_minimum_age_seconds
+        maxage = CONF.image_cache.remove_unused_original_minimum_age_seconds
 
         self._remove_old_enough_file(base_file, maxage, remove_lock=False)
 
     def _remove_base_file(self, base_file):
         """Remove a single base file if it is old enough."""
-        maxage = CONF.libvirt.remove_unused_resized_minimum_age_seconds
+        maxage = CONF.image_cache.remove_unused_resized_minimum_age_seconds
         if base_file in self.originals:
-            maxage = CONF.remove_unused_original_minimum_age_seconds
+            maxage = (
+                CONF.image_cache.remove_unused_original_minimum_age_seconds)
 
         self._remove_old_enough_file(base_file, maxage)
 
-    def _handle_base_image(self, img_id, base_file):
-        """Handle the checks for a single base image."""
+    def _mark_in_use(self, img_id, base_file):
+        """Mark a single base image as in use."""
 
-        image_in_use = False
-
-        LOG.info(_LI('image %(id)s at (%(base_file)s): checking'),
-                 {'id': img_id,
-                  'base_file': base_file})
+        LOG.info('image %(id)s at (%(base_file)s): checking',
+                 {'id': img_id, 'base_file': base_file})
 
         if base_file in self.unexplained_images:
             self.unexplained_images.remove(base_file)
 
-        if img_id in self.used_images:
-            local, remote, instances = self.used_images[img_id]
+        self.active_base_files.append(base_file)
 
-            if local > 0 or remote > 0:
-                image_in_use = True
-                LOG.info(_LI('image %(id)s at (%(base_file)s): '
-                             'in use: on this node %(local)d local, '
-                             '%(remote)d on other nodes sharing this instance '
-                             'storage'),
-                         {'id': img_id,
-                          'base_file': base_file,
-                          'local': local,
-                          'remote': remote})
-
-                self.active_base_files.append(base_file)
-
-                if not base_file:
-                    LOG.warning(_LW('image %(id)s at (%(base_file)s): warning '
-                                 '-- an absent base file is in use! '
-                                 'instances: %(instance_list)s'),
-                                {'id': img_id,
-                                 'base_file': base_file,
-                                 'instance_list': ' '.join(instances)})
-
-        if base_file:
-            if not image_in_use:
-                LOG.debug('image %(id)s at (%(base_file)s): image is not in '
-                          'use',
-                          {'id': img_id,
-                           'base_file': base_file})
-                self.removable_base_files.append(base_file)
-
-            else:
-                LOG.debug('image %(id)s at (%(base_file)s): image is in '
-                          'use',
-                          {'id': img_id,
-                           'base_file': base_file})
-                if os.path.exists(base_file):
-                    libvirt_utils.update_mtime(base_file)
+        LOG.debug('image %(id)s at (%(base_file)s): image is in use',
+                  {'id': img_id, 'base_file': base_file})
+        nova.privsep.path.utime(base_file)
 
     def _age_and_verify_swap_images(self, context, base_dir):
         LOG.debug('Verify swap images')
@@ -369,29 +266,26 @@ class ImageCacheManager(imagecache.ImageCacheManager):
         for ent in self.back_swap_images:
             base_file = os.path.join(base_dir, ent)
             if ent in self.used_swap_images and os.path.exists(base_file):
-                libvirt_utils.update_mtime(base_file)
+                nova.privsep.path.utime(base_file)
             elif self.remove_unused_base_images:
                 self._remove_swap_file(base_file)
 
         error_images = self.used_swap_images - self.back_swap_images
         for error_image in error_images:
-            LOG.warning(_LW('%s swap image was used by instance'
-                         ' but no back files existing!'), error_image)
+            LOG.warning('%s swap image was used by instance'
+                        ' but no back files existing!', error_image)
 
     def _age_and_verify_cached_images(self, context, all_instances, base_dir):
         LOG.debug('Verify base images')
         # Determine what images are on disk because they're in use
         for img in self.used_images:
-            fingerprint = hashlib.sha1(img).hexdigest()
+            fingerprint = hashlib.sha1(
+                    encodeutils.safe_encode(img)).hexdigest()
             LOG.debug('Image id %(id)s yields fingerprint %(fingerprint)s',
                       {'id': img,
                        'fingerprint': fingerprint})
-            for result in self._find_base_file(base_dir, fingerprint):
-                base_file, image_small, image_resized = result
-                self._handle_base_image(img, base_file)
-
-                if not image_small and not image_resized:
-                    self.originals.append(base_file)
+            for base_file in self._find_base_file(base_dir, fingerprint):
+                self._mark_in_use(img, base_file)
 
         # Elements remaining in unexplained_images might be in use
         inuse_backing_images = self._list_backing_images()
@@ -401,16 +295,16 @@ class ImageCacheManager(imagecache.ImageCacheManager):
 
         # Anything left is an unknown base image
         for img in self.unexplained_images:
-            LOG.warning(_LW('Unknown base file: %s'), img)
+            LOG.warning('Unknown base file: %s', img)
             self.removable_base_files.append(img)
 
         # Dump these lists
         if self.active_base_files:
-            LOG.info(_LI('Active base files: %s'),
+            LOG.info('Active base files: %s',
                      ' '.join(self.active_base_files))
 
         if self.removable_base_files:
-            LOG.info(_LI('Removable base files: %s'),
+            LOG.info('Removable base files: %s',
                      ' '.join(self.removable_base_files))
 
             if self.remove_unused_base_images:
@@ -433,7 +327,7 @@ class ImageCacheManager(imagecache.ImageCacheManager):
         # created, but may remain from previous versions.
 
         base_dir = os.path.join(CONF.instances_path,
-                                CONF.image_cache_subdirectory_name)
+                                CONF.image_cache.subdirectory_name)
         if not os.path.exists(base_dir):
             LOG.debug('Skipping verification, no base directory at %s',
                       base_dir)
@@ -456,3 +350,32 @@ class ImageCacheManager(imagecache.ImageCacheManager):
         # perform the aging and image verification
         self._age_and_verify_cached_images(context, all_instances, base_dir)
         self._age_and_verify_swap_images(context, base_dir)
+
+    def get_disk_usage(self):
+        try:
+            # If the cache is on a different device than the instance dir then
+            # it does not consume the same disk resource as instances.
+            if (os.stat(CONF.instances_path).st_dev !=
+                    os.stat(self.cache_dir).st_dev):
+                return 0
+
+            # NOTE(gibi): we need to use the disk size occupied from the file
+            # system as images in the cache will not grow to their virtual
+            # size.
+            # NOTE(gibi): st.blocks is always measured in 512 byte blocks see
+            # man fstat
+            return sum(
+                os.stat(os.path.join(self.cache_dir, f)).st_blocks * 512
+                for f in os.listdir(self.cache_dir)
+                if os.path.isfile(os.path.join(self.cache_dir, f)))
+        except OSError:
+            # NOTE(gibi): An error here can mean many things. E.g. the cache
+            # dir does not exists yet, the cache dir is deleted between the
+            # listdir() and the stat() calls, or a file is deleted between
+            # the listdir() and the stat() calls.
+            return 0
+
+    @property
+    def cache_dir(self):
+        return os.path.join(
+            CONF.instances_path, CONF.image_cache.subdirectory_name)
