@@ -29,11 +29,11 @@ from oslo_vmware import exceptions as vexc
 from oslo_vmware.objects import datastore as ds_obj
 from oslo_vmware import pbm
 from oslo_vmware import vim_util as vutil
-import six
+
 
 import nova.conf
 from nova import exception
-from nova.i18n import _, _LE, _LI, _LW
+from nova.i18n import _
 from nova.network import model as network_model
 from nova.virt.vmwareapi import constants
 from nova.virt.vmwareapi import vim_util
@@ -46,6 +46,14 @@ ALL_SUPPORTED_NETWORK_DEVICES = ['VirtualE1000', 'VirtualE1000e',
                                  'VirtualPCNet32', 'VirtualSriovEthernetCard',
                                  'VirtualVmxnet', 'VirtualVmxnet3']
 
+CONTROLLER_TO_ADAPTER_TYPE = {
+    "VirtualLsiLogicController": constants.DEFAULT_ADAPTER_TYPE,
+    "VirtualBusLogicController": constants.ADAPTER_TYPE_BUSLOGIC,
+    "VirtualIDEController": constants.ADAPTER_TYPE_IDE,
+    "VirtualLsiLogicSASController": constants.ADAPTER_TYPE_LSILOGICSAS,
+    "ParaVirtualSCSIController": constants.ADAPTER_TYPE_PARAVIRTUAL
+}
+
 # A simple cache for storing inventory folder references.
 # Format: {inventory_path: folder_ref}
 _FOLDER_PATH_REF_MAPPING = {}
@@ -92,7 +100,7 @@ class ExtraSpecs(object):
     def __init__(self, cpu_limits=None, hw_version=None,
                  storage_policy=None, cores_per_socket=None,
                  memory_limits=None, disk_io_limits=None,
-                 vif_limits=None):
+                 vif_limits=None, firmware=None, hw_video_ram=None):
         """ExtraSpecs object holds extra_specs for the instance."""
         self.cpu_limits = cpu_limits or Limits()
         self.memory_limits = memory_limits or Limits()
@@ -101,6 +109,8 @@ class ExtraSpecs(object):
         self.hw_version = hw_version
         self.storage_policy = storage_policy
         self.cores_per_socket = cores_per_socket
+        self.firmware = firmware
+        self.hw_video_ram = hw_video_ram
 
 
 def vm_refs_cache_reset():
@@ -136,13 +146,6 @@ def vm_ref_cache_from_instance(func):
     return wrapper
 
 
-def vm_ref_cache_from_name(func):
-    @functools.wraps(func)
-    def wrapper(session, name):
-        id = name
-        return _vm_ref_cache(id, func, session, name)
-    return wrapper
-
 # the config key which stores the VNC port
 VNC_CONFIG_KEY = 'config.extraConfig["RemoteDisplay.vnc.port"]'
 
@@ -206,7 +209,8 @@ def get_vm_create_spec(client_factory, instance, data_store_name,
     config_spec.version = extra_specs.hw_version
 
     # Allow nested hypervisor instances to host 64 bit VMs.
-    if os_type in ("vmkernel5Guest", "vmkernel6Guest", "windowsHyperVGuest"):
+    if os_type in ("vmkernel5Guest", "vmkernel6Guest", "vmkernel65Guest",
+                   "windowsHyperVGuest"):
         config_spec.nestedHVEnabled = "True"
 
     # Append the profile spec
@@ -242,16 +246,24 @@ def get_vm_create_spec(client_factory, instance, data_store_name,
             client_factory, extra_specs.memory_limits,
             'ns0:ResourceAllocationInfo')
 
+    if extra_specs.firmware:
+        config_spec.firmware = extra_specs.firmware
+
     devices = []
-    for vif_info in vif_infos:
+    for i, vif_info in enumerate(vif_infos):
         vif_spec = _create_vif_spec(client_factory, vif_info,
-                                    extra_specs.vif_limits)
+                                    extra_specs.vif_limits, i)
         devices.append(vif_spec)
 
     serial_port_spec = create_serial_port_spec(client_factory)
     if serial_port_spec:
         devices.append(serial_port_spec)
 
+    virtual_device_config_spec = create_video_card_spec(client_factory,
+                                                        extra_specs)
+    if virtual_device_config_spec:
+        devices.append(virtual_device_config_spec)
+
     config_spec.deviceChange = devices
 
     # add vm-uuid and iface-id.x values for Neutron
@@ -293,6 +305,18 @@ def get_vm_create_spec(client_factory, instance, data_store_name,
     return config_spec
 
 
+def create_video_card_spec(client_factory, extra_specs):
+    if extra_specs.hw_video_ram:
+        video_card = client_factory.create('ns0:VirtualMachineVideoCard')
+        video_card.videoRamSizeInKB = extra_specs.hw_video_ram
+        video_card.key = -1
+        virtual_device_config_spec = client_factory.create(
+            'ns0:VirtualDeviceConfigSpec')
+        virtual_device_config_spec.operation = "add"
+        virtual_device_config_spec.device = video_card
+        return virtual_device_config_spec
+
+
 def create_serial_port_spec(client_factory):
     """Creates config spec for serial port."""
     if not CONF.vmware.serial_port_service_uri:
@@ -400,7 +424,7 @@ def convert_vif_model(name):
     return name
 
 
-def _create_vif_spec(client_factory, vif_info, vif_limits=None):
+def _create_vif_spec(client_factory, vif_info, vif_limits=None, offset=0):
     """Builds a config spec for the addition of a new network
     adapter to the VM.
     """
@@ -462,7 +486,7 @@ def _create_vif_spec(client_factory, vif_info, vif_limits=None):
     # The Server assigns a Key to the device. Here we pass a -ve temporary key.
     # -ve because actual keys are +ve numbers and we don't
     # want a clash with the key that server might associate with the device
-    net_device.key = -47
+    net_device.key = -47 - offset
     net_device.addressType = "manual"
     net_device.macAddress = mac_address
     net_device.wakeOnLanEnabled = True
@@ -514,6 +538,44 @@ def get_network_detach_config_spec(client_factory, device, port_index):
     return config_spec
 
 
+def update_vif_spec(client_factory, vif_info, device):
+    """Updates the backing for the VIF spec."""
+    network_spec = client_factory.create('ns0:VirtualDeviceConfigSpec')
+    network_spec.operation = 'edit'
+    network_ref = vif_info['network_ref']
+    network_name = vif_info['network_name']
+    if network_ref and network_ref['type'] == 'OpaqueNetwork':
+        backing = client_factory.create(
+                'ns0:VirtualEthernetCardOpaqueNetworkBackingInfo')
+        backing.opaqueNetworkId = network_ref['network-id']
+        backing.opaqueNetworkType = network_ref['network-type']
+        # Configure externalId
+        if network_ref['use-external-id']:
+            if hasattr(device, 'externalId'):
+                device.externalId = vif_info['iface_id']
+            else:
+                dp = client_factory.create('ns0:DynamicProperty')
+                dp.name = "__externalId__"
+                dp.val = vif_info['iface_id']
+                device.dynamicProperty = [dp]
+    elif (network_ref and
+            network_ref['type'] == "DistributedVirtualPortgroup"):
+        backing = client_factory.create(
+                'ns0:VirtualEthernetCardDistributedVirtualPortBackingInfo')
+        portgroup = client_factory.create(
+                    'ns0:DistributedVirtualSwitchPortConnection')
+        portgroup.switchUuid = network_ref['dvsw']
+        portgroup.portgroupKey = network_ref['dvpg']
+        backing.port = portgroup
+    else:
+        backing = client_factory.create(
+                  'ns0:VirtualEthernetCardNetworkBackingInfo')
+        backing.deviceName = network_name
+    device.backing = backing
+    network_spec.device = device
+    return network_spec
+
+
 def get_storage_profile_spec(session, storage_policy):
     """Gets the vm profile spec configured for storage policy."""
     profile_id = pbm.get_profile_id_by_name(session, storage_policy)
@@ -591,7 +653,7 @@ def get_vm_extra_config_spec(client_factory, extra_opts):
     config_spec = client_factory.create('ns0:VirtualMachineConfigSpec')
     # add the key value pairs
     extra_config = []
-    for key, value in six.iteritems(extra_opts):
+    for key, value in extra_opts.items():
         opt = client_factory.create('ns0:OptionValue')
         opt.key = key
         opt.value = value
@@ -620,14 +682,17 @@ def _get_device_disk_type(device):
             return constants.DEFAULT_DISK_TYPE
 
 
-def get_vmdk_info(session, vm_ref, uuid=None):
-    """Returns information for the primary VMDK attached to the given VM."""
+def get_hardware_devices(session, vm_ref):
     hardware_devices = session._call_method(vutil,
                                             "get_object_property",
                                             vm_ref,
                                             "config.hardware.device")
-    if hardware_devices.__class__.__name__ == "ArrayOfVirtualDevice":
-        hardware_devices = hardware_devices.VirtualDevice
+    return vim_util.get_array_items(hardware_devices)
+
+
+def get_vmdk_info(session, vm_ref, uuid=None):
+    """Returns information for the primary VMDK attached to the given VM."""
+    hardware_devices = get_hardware_devices(session, vm_ref)
     vmdk_file_path = None
     vmdk_controller_key = None
     disk_type = None
@@ -649,16 +714,9 @@ def get_vmdk_info(session, vm_ref, uuid=None):
                 if root_disk and path.basename == root_disk:
                     root_device = device
                 vmdk_device = device
-        elif device.__class__.__name__ == "VirtualLsiLogicController":
-            adapter_type_dict[device.key] = constants.DEFAULT_ADAPTER_TYPE
-        elif device.__class__.__name__ == "VirtualBusLogicController":
-            adapter_type_dict[device.key] = constants.ADAPTER_TYPE_BUSLOGIC
-        elif device.__class__.__name__ == "VirtualIDEController":
-            adapter_type_dict[device.key] = constants.ADAPTER_TYPE_IDE
-        elif device.__class__.__name__ == "VirtualLsiLogicSASController":
-            adapter_type_dict[device.key] = constants.ADAPTER_TYPE_LSILOGICSAS
-        elif device.__class__.__name__ == "ParaVirtualSCSIController":
-            adapter_type_dict[device.key] = constants.ADAPTER_TYPE_PARAVIRTUAL
+        elif device.__class__.__name__ in CONTROLLER_TO_ADAPTER_TYPE:
+            adapter_type_dict[device.key] = CONTROLLER_TO_ADAPTER_TYPE[
+                device.__class__.__name__]
 
     if root_disk:
         vmdk_device = root_device
@@ -678,7 +736,7 @@ scsi_controller_classes = {
     'ParaVirtualSCSIController': constants.ADAPTER_TYPE_PARAVIRTUAL,
     'VirtualLsiLogicController': constants.DEFAULT_ADAPTER_TYPE,
     'VirtualLsiLogicSASController': constants.ADAPTER_TYPE_LSILOGICSAS,
-    'VirtualBusLogicController': constants.ADAPTER_TYPE_PARAVIRTUAL,
+    'VirtualBusLogicController': constants.ADAPTER_TYPE_BUSLOGIC,
 }
 
 
@@ -932,17 +990,30 @@ def clone_vm_spec(client_factory, location,
     return clone_spec
 
 
-def relocate_vm_spec(client_factory, datastore=None, host=None,
-                     disk_move_type="moveAllDiskBackingsAndAllowSharing"):
-    """Builds the VM relocation spec."""
+def relocate_vm_spec(client_factory, res_pool=None, datastore=None, host=None,
+                     disk_move_type="moveAllDiskBackingsAndAllowSharing",
+                     devices=None):
     rel_spec = client_factory.create('ns0:VirtualMachineRelocateSpec')
     rel_spec.datastore = datastore
+    rel_spec.host = host
+    rel_spec.pool = res_pool
     rel_spec.diskMoveType = disk_move_type
-    if host:
-        rel_spec.host = host
+    if devices is not None:
+        rel_spec.deviceChange = devices
     return rel_spec
 
 
+def relocate_vm(session, vm_ref, res_pool=None, datastore=None, host=None,
+                disk_move_type="moveAllDiskBackingsAndAllowSharing",
+                devices=None):
+    client_factory = session.vim.client.factory
+    rel_spec = relocate_vm_spec(client_factory, res_pool, datastore, host,
+                                disk_move_type, devices)
+    relocate_task = session._call_method(session.vim, "RelocateVM_Task",
+                                         vm_ref, spec=rel_spec)
+    session._wait_for_task(relocate_task)
+
+
 def get_machine_id_change_spec(client_factory, machine_id_str):
     """Builds the machine id change config spec."""
     virtual_machine_config_spec = client_factory.create(
@@ -955,25 +1026,6 @@ def get_machine_id_change_spec(client_factory, machine_id_str):
     return virtual_machine_config_spec
 
 
-def get_add_vswitch_port_group_spec(client_factory, vswitch_name,
-                                    port_group_name, vlan_id):
-    """Builds the virtual switch port group add spec."""
-    vswitch_port_group_spec = client_factory.create('ns0:HostPortGroupSpec')
-    vswitch_port_group_spec.name = port_group_name
-    vswitch_port_group_spec.vswitchName = vswitch_name
-
-    # VLAN ID of 0 means that VLAN tagging is not to be done for the network.
-    vswitch_port_group_spec.vlanId = int(vlan_id)
-
-    policy = client_factory.create('ns0:HostNetworkPolicy')
-    nicteaming = client_factory.create('ns0:HostNicTeamingPolicy')
-    nicteaming.notifySwitches = True
-    policy.nicTeaming = nicteaming
-
-    vswitch_port_group_spec.policy = policy
-    return vswitch_port_group_spec
-
-
 def get_vnc_config_spec(client_factory, port):
     """Builds the vnc config spec."""
     virtual_machine_config_spec = client_factory.create(
@@ -987,7 +1039,7 @@ def get_vnc_config_spec(client_factory, port):
     opt_port.value = port
     opt_keymap = client_factory.create('ns0:OptionValue')
     opt_keymap.key = "RemoteDisplay.vnc.keyMap"
-    opt_keymap.value = CONF.vnc.keymap
+    opt_keymap.value = CONF.vmware.vnc_keymap
 
     extras = [opt_enabled, opt_port, opt_keymap]
 
@@ -1053,30 +1105,15 @@ def _get_object_from_results(session, results, value, func):
 
 
 def _get_vm_ref_from_name(session, vm_name):
-    """Get reference to the VM with the name specified."""
-    vms = session._call_method(vim_util, "get_objects",
-                "VirtualMachine", ["name"])
-    return _get_object_from_results(session, vms, vm_name,
-                                    _get_object_for_value)
-
-
-@vm_ref_cache_from_name
-def get_vm_ref_from_name(session, vm_name):
-    return (_get_vm_ref_from_vm_uuid(session, vm_name) or
-            _get_vm_ref_from_name(session, vm_name))
-
-
-def _get_vm_ref_from_uuid(session, instance_uuid):
-    """Get reference to the VM with the uuid specified.
+    """Get reference to the VM with the name specified.
 
     This method reads all of the names of the VM's that are running
-    on the backend, then it filters locally the matching
-    instance_uuid. It is far more optimal to use
-    _get_vm_ref_from_vm_uuid.
+    on the backend, then it filters locally the matching vm_name.
+    It is far more optimal to use _get_vm_ref_from_vm_uuid.
     """
     vms = session._call_method(vim_util, "get_objects",
                 "VirtualMachine", ["name"])
-    return _get_object_from_results(session, vms, instance_uuid,
+    return _get_object_from_results(session, vms, vm_name,
                                     _get_object_for_value)
 
 
@@ -1128,7 +1165,7 @@ def search_vm_ref_by_identifier(session, identifier):
     """
     vm_ref = (_get_vm_ref_from_vm_uuid(session, identifier) or
               _get_vm_ref_from_extraconfig(session, identifier) or
-              _get_vm_ref_from_uuid(session, identifier))
+              _get_vm_ref_from_name(session, identifier))
     return vm_ref
 
 
@@ -1152,18 +1189,21 @@ def get_vm_state(session, instance):
     vm_ref = get_vm_ref(session, instance)
     vm_state = session._call_method(vutil, "get_object_property",
                                     vm_ref, "runtime.powerState")
-    return vm_state
+    return constants.POWER_STATES[vm_state]
 
 
 def get_stats_from_cluster(session, cluster):
     """Get the aggregate resource stats of a cluster."""
     vcpus = 0
-    mem_info = {'total': 0, 'free': 0}
+    max_vcpus_per_host = 0
+    used_mem_mb = 0
+    total_mem_mb = 0
+    max_mem_mb_per_host = 0
     # Get the Host and Resource Pool Managed Object Refs
     prop_dict = session._call_method(vutil,
                                      "get_object_properties_dict",
                                      cluster,
-                                     ["host", "resourcePool"])
+                                     ["host"])
     if prop_dict:
         host_ret = prop_dict.get('host')
         if host_ret:
@@ -1171,27 +1211,29 @@ def get_stats_from_cluster(session, cluster):
             result = session._call_method(vim_util,
                          "get_properties_for_a_collection_of_objects",
                          "HostSystem", host_mors,
-                         ["summary.hardware", "summary.runtime"])
+                         ["summary.hardware", "summary.runtime",
+                          "summary.quickStats"])
             for obj in result.objects:
-                hardware_summary = obj.propSet[0].val
-                runtime_summary = obj.propSet[1].val
+                host_props = propset_dict(obj.propSet)
+                hardware_summary = host_props['summary.hardware']
+                runtime_summary = host_props['summary.runtime']
+                stats_summary = host_props['summary.quickStats']
                 if (runtime_summary.inMaintenanceMode is False and
                     runtime_summary.connectionState == "connected"):
                     # Total vcpus is the sum of all pCPUs of individual hosts
                     # The overcommitment ratio is factored in by the scheduler
-                    vcpus += hardware_summary.numCpuThreads
-
-        res_mor = prop_dict.get('resourcePool')
-        if res_mor:
-            res_usage = session._call_method(vutil, "get_object_property",
-                                             res_mor, "summary.runtime.memory")
-            if res_usage:
-                # maxUsage is the memory limit of the cluster available to VM's
-                mem_info['total'] = int(res_usage.maxUsage / units.Mi)
-                # overallUsage is the hypervisor's view of memory usage by VM's
-                consumed = int(res_usage.overallUsage / units.Mi)
-                mem_info['free'] = mem_info['total'] - consumed
-    stats = {'vcpus': vcpus, 'mem': mem_info}
+                    threads = hardware_summary.numCpuThreads
+                    vcpus += threads
+                    max_vcpus_per_host = max(max_vcpus_per_host, threads)
+                    used_mem_mb += stats_summary.overallMemoryUsage
+                    mem_mb = hardware_summary.memorySize // units.Mi
+                    total_mem_mb += mem_mb
+                    max_mem_mb_per_host = max(max_mem_mb_per_host, mem_mb)
+    stats = {'cpu': {'vcpus': vcpus,
+                     'max_vcpus_per_host': max_vcpus_per_host},
+             'mem': {'total': total_mem_mb,
+                     'free': total_mem_mb - used_mem_mb,
+                     'max_mem_mb_per_host': max_mem_mb_per_host}}
     return stats
 
 
@@ -1279,7 +1321,7 @@ def get_all_cluster_mors(session):
             return results.objects
 
     except Exception as excep:
-        LOG.warning(_LW("Failed to get cluster references %s"), excep)
+        LOG.warning("Failed to get cluster references %s", excep)
 
 
 def get_cluster_ref_by_name(session, cluster_name):
@@ -1326,10 +1368,10 @@ def create_vm(session, instance, vm_folder, config_spec, res_pool_ref):
         # Consequently, a value which we don't recognise may in fact be valid.
         with excutils.save_and_reraise_exception():
             if config_spec.guestId not in constants.VALID_OS_TYPES:
-                LOG.warning(_LW('vmware_ostype from image is not recognised: '
-                                '\'%(ostype)s\'. An invalid os type may be '
-                                'one cause of this instance creation failure'),
-                         {'ostype': config_spec.guestId})
+                LOG.warning('vmware_ostype from image is not recognised: '
+                            '\'%(ostype)s\'. An invalid os type may be '
+                            'one cause of this instance creation failure',
+                            {'ostype': config_spec.guestId})
     LOG.debug("Created VM on the ESX host", instance=instance)
     return task_info.result
 
@@ -1343,9 +1385,9 @@ def destroy_vm(session, instance, vm_ref=None):
         destroy_task = session._call_method(session.vim, "Destroy_Task",
                                             vm_ref)
         session._wait_for_task(destroy_task)
-        LOG.info(_LI("Destroyed the VM"), instance=instance)
+        LOG.info("Destroyed the VM", instance=instance)
     except Exception:
-        LOG.exception(_LE('Destroy VM failed'), instance=instance)
+        LOG.exception('Destroy VM failed', instance=instance)
 
 
 def create_virtual_disk(session, dc_ref, adapter_type, disk_type,
@@ -1573,23 +1615,6 @@ def get_swap(session, vm_ref):
             return device
 
 
-def _get_folder(session, parent_folder_ref, name):
-    # Get list of child entities for the parent folder
-    prop_val = session._call_method(vutil, 'get_object_property',
-                                    parent_folder_ref,
-                                    'childEntity')
-    if prop_val:
-        child_entities = prop_val.ManagedObjectReference
-
-        # Return if the child folder with input name is already present
-        for child_entity in child_entities:
-            if child_entity._type != 'Folder':
-                continue
-            child_entity_name = vim_util.get_entity_name(session, child_entity)
-            if child_entity_name == name:
-                return child_entity
-
-
 def create_folder(session, parent_folder_ref, name):
     """Creates a folder in vCenter
 
@@ -1597,15 +1622,12 @@ def create_folder(session, parent_folder_ref, name):
     The moref of the folder is returned.
     """
 
-    folder = _get_folder(session, parent_folder_ref, name)
-    if folder:
-        return folder
     LOG.debug("Creating folder: %(name)s. Parent ref: %(parent)s.",
               {'name': name, 'parent': parent_folder_ref.value})
     try:
         folder = session._call_method(session.vim, "CreateFolder",
                                       parent_folder_ref, name=name)
-        LOG.info(_LI("Created folder: %(name)s in parent %(parent)s."),
+        LOG.info("Created folder: %(name)s in parent %(parent)s.",
                  {'name': name, 'parent': parent_folder_ref.value})
     except vexc.DuplicateName as e:
         LOG.debug("Folder already exists: %(name)s. Parent ref: %(parent)s.",
